<?php
/**
 * Class to handle just the weather map functionality
 */
class NewspaperAI_Weather_Map {
    private static $instance = null;
    private $noaa_api;

    public static function get_instance() {
        if (null === self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        $this->noaa_api = NewspaperAI_NOAA_API::get_instance();
        
        // Register scripts and styles
        add_action('wp_enqueue_scripts', array($this, 'register_assets'));
        add_action('admin_enqueue_scripts', array($this, 'register_assets'));
    }

    public function register_assets() {
        // Register Leaflet CSS
        wp_register_style(
            'leaflet-css',
            'https://unpkg.com/leaflet@1.9.4/dist/leaflet.css',
            array(),
            '1.9.4'
        );

        // Register Leaflet JS
        wp_register_script(
            'leaflet-js',
            'https://unpkg.com/leaflet@1.9.4/dist/leaflet.js',
            array('jquery'),
            '1.9.4',
            true
        );
    }

    public function get_map_html($lat, $lng, $city, $state) {
        // Ensure scripts are loaded
        wp_enqueue_style('leaflet-css');
        wp_enqueue_script('leaflet-js');
        
        // Generate unique map ID
        $map_id = 'weather-map-' . uniqid();
        
        // Add map initialization script
        add_action('wp_footer', function() use ($map_id, $lat, $lng, $city, $state) {
            ?>
            <script>
                jQuery(document).ready(function($) {
                    // Wait for Leaflet to load
                    var checkLeaflet = setInterval(function() {
                        if (typeof L !== 'undefined') {
                            clearInterval(checkLeaflet);
                            initMap();
                        }
                    }, 100);

                    function initMap() {
                        var mapElement = document.getElementById('<?php echo esc_js($map_id); ?>');
                        if (!mapElement) {
                            console.error('Map element not found');
                            return;
                        }

                        var map = L.map('<?php echo esc_js($map_id); ?>').setView([
                            <?php echo floatval($lat); ?>,
                            <?php echo floatval($lng); ?>
                        ], 8);

                        L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
                            attribution: ' OpenStreetMap contributors',
                            maxZoom: 18
                        }).addTo(map);

                        L.marker([
                            <?php echo floatval($lat); ?>,
                            <?php echo floatval($lng); ?>
                        ]).addTo(map)
                          .bindPopup('<?php echo esc_js($city) . ', ' . esc_js($state); ?>')
                          .openPopup();

                        // Force a resize after map is shown
                        setTimeout(function() {
                            map.invalidateSize();
                        }, 100);
                    }
                });
            </script>
            <?php
        }, 99);
        
        // Return the map container HTML with styles
        return sprintf(
            '<style>
                #%1$s {
                    height: 400px;
                    width: 100%%;
                    border-radius: 8px;
                    overflow: hidden;
                    border: 1px solid #e0e0e0;
                    background: #f8f9fa;
                    margin: 1rem 0;
                    position: relative;
                    z-index: 1;
                }
                .leaflet-container {
                    height: 100%% !important;
                    width: 100%% !important;
                    z-index: 1;
                }
            </style>
            <div id="%1$s" class="weather-map"></div>',
            esc_attr($map_id)
        );
    }
}
