<?php
/**
 * Fix Default Weather Images
 * 
 * This script registers and processes default weather images from the plugin directory.
 * Run this script directly from the WordPress admin area.
 */

// Prevent direct access
if (!defined('ABSPATH')) {
    // For Local by Flywheel environment
    $path = dirname(dirname(dirname(dirname(dirname(dirname(__FILE__)))))) . '/';
    define('ABSPATH', $path);
    require_once(ABSPATH . 'wp-load.php');
    
    // Ensure we have database connection
    global $wpdb;
    if (!$wpdb->db_connect()) {
        die('Error: Unable to connect to the WordPress database. Please ensure Local is running.');
    }
}

// Ensure user is an admin
if (!current_user_can('manage_options')) {
    wp_die('You do not have sufficient permissions to access this page.');
}

// Load the fallback image system
require_once dirname(__FILE__) . '/class-weather-images-fallback.php';
$fallback = NewspaperAI_Weather_Images_Fallback::get_instance();

// Define base weather types available in the plugin
$base_types = [
    'sunny', 'cloudy', 'fog', 'lightning', 'overcast', 'rain', 'snow'
];

// Define mapping from API weather conditions to image types
$condition_map = [
    'sunny' => 'sunny',
    'mostly sunny' => 'sunny',
    'partly sunny' => 'cloudy',
    'clear' => 'sunny',
    'partly cloudy' => 'cloudy',
    'mostly cloudy' => 'cloudy',
    'cloudy' => 'cloudy',
    'overcast' => 'overcast',
    'rain' => 'rain',
    'rain_light' => 'rain',
    'rain_heavy' => 'rain',
    'showers' => 'rain',
    'thunderstorm' => 'lightning',
    'thunderstorm_severe' => 'lightning',
    'snow' => 'snow',
    'snow_light' => 'snow',
    'snow_heavy' => 'snow',
    'fog' => 'fog',
    'mist' => 'fog',
    'haze' => 'fog',
    'windy' => 'cloudy',
    'hot' => 'sunny',
    'cold' => 'snow',
    'sunrise' => 'sunny',
    'sunset' => 'sunny',
    'sleet' => 'snow'
];

// Create a list of all weather types we need to process
// This includes both base types and all mapped conditions
$weather_types = array_unique(array_merge($base_types, array_keys($condition_map)));

// Plugin assets directory URL
$plugin_assets_url = plugin_dir_url(dirname(dirname(__FILE__))) . 'assets/images/weather/';

// Set up directories
$plugin_dir = plugin_dir_path(dirname(dirname(__FILE__)));
$plugin_images_dir = $plugin_dir . 'assets/images/weather/';
$upload_dir = wp_upload_dir();
$uploads_images_dir = $upload_dir['basedir'] . '/weather-images/';

// Create directories if they don't exist
wp_mkdir_p($plugin_images_dir);
wp_mkdir_p($uploads_images_dir);

echo '<div style="font-family: Arial, sans-serif; max-width: 800px; margin: 20px auto; padding: 20px; border: 1px solid #ddd; border-radius: 5px;">';
echo '<h1>Generate Weather Images</h1>';
echo '<p>Downloading and processing default weather images from S3...</p>';
echo '<ul>';

$success_count = 0;
$error_count = 0;

// First, verify the base images exist in the plugin directory
foreach ($base_types as $type) {
    for ($variation = 1; $variation <= 3; $variation++) {
        // Local filename format
        $local_filename = $type . '-' . $variation . '.jpg';
        $plugin_image_path = $plugin_images_dir . $local_filename;
        
        echo "<li>Verifying base image: {$local_filename}... ";
        
        if (file_exists($plugin_image_path)) {
            echo "<span style='color: green;'>Image exists in plugin directory.</span>";
            $success_count++;
        } else {
            echo "<span style='color: orange;'>Image missing. Generating fallback...</span> ";
            
            // Try to generate a fallback image
            $file_path = $fallback->get_fallback_image($type, $variation);
            
            if ($file_path && file_exists($file_path)) {
                // Copy to plugin directory
                if (copy($file_path, $plugin_image_path)) {
                    echo "<span style='color: green;'>Created fallback image in plugin directory.</span>";
                    $success_count++;
                } else {
                    echo "<span style='color: red;'>Failed to save fallback to plugin directory.</span>";
                    $error_count++;
                }
            } else {
                echo "<span style='color: red;'>Failed to create fallback image.</span>";
                $error_count++;
            }
        }
        
        echo "</li>";
    }
}

// Now create the mapped variations by copying the base images
foreach ($condition_map as $weather_condition => $base_type) {
    // Skip if this is one of the base S3 types we already downloaded
    if (in_array($weather_condition, $s3_types)) {
        continue;
    }
    
    for ($variation = 1; $variation <= 3; $variation++) {
        $target_filename = $weather_condition . '-' . $variation . '.jpg';
        $source_filename = $base_type . '-' . $variation . '.jpg';
        
        $target_path = $plugin_images_dir . $target_filename;
        $source_path = $plugin_images_dir . $source_filename;
        
        echo "<li>Creating mapped image: {$target_filename} from {$source_filename}... ";
        
        // Only create if the target doesn't exist and the source does
        if (!file_exists($target_path) && file_exists($source_path)) {
            if (copy($source_path, $target_path)) {
                echo "<span style='color: green;'>Created successfully.</span>";
                $success_count++;
            } else {
                echo "<span style='color: red;'>Failed to create.</span>";
                $error_count++;
            }
        } else if (file_exists($target_path)) {
            echo "<span style='color: blue;'>Already exists.</span>";
        } else {
            echo "<span style='color: red;'>Source image not found.</span>";
            $error_count++;
        }
        
        echo "</li>";
    }
}

// Create special UI-specific mappings that match the UI display names
echo "<li style='background-color: #f0f8ff; padding: 5px;'><strong>Creating UI-specific mappings for front-end display...</strong></li>";

$ui_mappings = [
    'partly_cloudy' => 'cloudy',  // UI: "Partly Cloudy/Partly Sunny"
    'storm' => 'lightning'        // UI: "Thunderstorm/Severe"
];

foreach ($ui_mappings as $ui_type => $base_type) {
    for ($variation = 1; $variation <= 3; $variation++) {
        $target_filename = $ui_type . '-' . $variation . '.jpg';
        $source_filename = $base_type . '-' . $variation . '.jpg';
        
        $target_path = $plugin_images_dir . $target_filename;
        $source_path = $plugin_images_dir . $source_filename;
        
        echo "<li style='margin-left: 20px;'>Creating UI-specific image: {$target_filename} from {$source_filename}... ";
        
        if (!file_exists($target_path) && file_exists($source_path)) {
            if (copy($source_path, $target_path)) {
                echo "<span style='color: green;'>Created successfully.</span>";
                $success_count++;
            } else {
                echo "<span style='color: red;'>Failed to create.</span>";
                $error_count++;
            }
        } else if (file_exists($target_path)) {
            echo "<span style='color: blue;'>Already exists.</span>";
        } else {
            echo "<span style='color: red;'>Source image not found.</span>";
            $error_count++;
        }
        
        echo "</li>";
    }
}

echo '</ul>';
echo '<div style="margin-top: 20px; padding: 10px; background-color: #f5f5f5; border-radius: 5px;">';
echo "<p><strong>Summary:</strong> {$success_count} images processed successfully, {$error_count} errors.</p>";
echo '</div>';

// Display the condition mapping table
echo '<div style="margin-top: 20px;">';
echo '<h2>Weather Condition Mapping</h2>';
echo '<p>The following table shows how weather conditions from the API are mapped to image types:</p>';
echo '<table style="width: 100%; border-collapse: collapse; margin-top: 10px;">';
echo '<tr style="background-color: #f0f0f0;"><th style="padding: 8px; border: 1px solid #ddd; text-align: left;">API Weather Condition</th><th style="padding: 8px; border: 1px solid #ddd; text-align: left;">Image Type</th></tr>';

foreach ($condition_map as $condition => $image_type) {
    echo '<tr>';
    echo '<td style="padding: 8px; border: 1px solid #ddd;">' . esc_html($condition) . '</td>';
    echo '<td style="padding: 8px; border: 1px solid #ddd;">' . esc_html($image_type) . '</td>';
    echo '</tr>';
}

echo '</table>';
echo '</div>';

// Add information about using these images
echo '<div style="background-color: #f8f9fa; padding: 15px; border-left: 4px solid #4CAF50; margin: 20px 0;">';
echo '<h3>What happens next?</h3>';
echo '<p>These generated images will be used as fallback images when creating weather posts. When a post is created, the system will:';
echo '<ol>';
echo '<li>Check if you have uploaded a custom image for the weather condition</li>';
echo '<li>If not, it will use the default image included with the plugin</li>';
echo '<li>If that image is missing, it will use one of these generated fallback images</li>';
echo '</ol>';
echo '<p>You can upload custom images for each weather condition in the <a href="' . admin_url('admin.php?page=weatherai&tab=images') . '">Weather Images tab</a>.</p>';
echo '</div>';

// Add a button to return to the weather settings page
echo '<p><a href="' . admin_url('admin.php?page=weatherai') . '" class="button button-primary">Return to Weather Settings</a></p>';

echo '</div>';

// Register images in the WordPress database
echo '<div style="margin-top: 20px; padding: 15px; background-color: #f0f8ff; border-radius: 5px;">';
echo '<h2>Registering Images in Database</h2>';
echo '<p>Now registering all downloaded images in the WordPress database...</p>';
echo '<ul>';

// Get current weather images option
$weather_images = get_option('weather_write_images', array());
$site_url = site_url();
$plugin_url = trailingslashit($site_url) . 'wp-content/plugins/Publisher_Plugin/';
$images_url = $plugin_url . 'assets/images/weather/';

$db_success = 0;
$db_error = 0;

// Define the weather types used in the UI
$ui_weather_types = array(
    'sunny', 'partly_cloudy', 'cloudy', 'rain', 'storm', 'snow', 'fog',
    'clear', 'partly_sunny', 'mostly_cloudy', 'overcast', 'rain_light', 'rain_heavy',
    'showers', 'thunderstorm', 'thunderstorm_severe', 'snow_light', 'snow_heavy',
    'mist', 'haze', 'windy', 'hot', 'cold', 'sunrise', 'sunset', 'sleet', 'lightning'
);

// Process each weather type
foreach ($ui_weather_types as $type) {
    for ($variation = 1; $variation <= 3; $variation++) {
        $image_path = $plugin_images_dir . $type . '-' . $variation . '.jpg';
        $image_url = $images_url . $type . '-' . $variation . '.jpg';
        
        // Check if the image file exists
        if (file_exists($image_path)) {
            echo "<li>Registering image: {$type}-{$variation}.jpg... ";
            
            // Create the database key (with underscore format)
            $db_key = $type . '_' . $variation;
            
            // Check if we already have this image in the database
            if (isset($weather_images[$db_key]) && 
                ((is_array($weather_images[$db_key]) && !empty($weather_images[$db_key]['url'])) || 
                 (!is_array($weather_images[$db_key]) && !empty($weather_images[$db_key])))) {
                echo "<span style='color: blue;'>Already in database.</span></li>";
                continue;
            }
            
            // Create a WordPress attachment for the image if it doesn't exist
            $attachment_id = create_attachment_from_file($image_path, $type . '-' . $variation . '.jpg');
            
            if ($attachment_id) {
                // Update the database entry
                $weather_images[$db_key] = array(
                    'id' => $attachment_id,
                    'url' => $image_url
                );
                
                echo "<span style='color: green;'>Added to database with ID: {$attachment_id}</span></li>";
                $db_success++;
            } else {
                // If we can't create an attachment, just store the URL
                $weather_images[$db_key] = array(
                    'id' => 0,
                    'url' => $image_url
                );
                echo "<span style='color: orange;'>Added URL to database (no attachment)</span></li>";
                $db_success++;
            }
        }
    }
}

// Save the updated weather images option
update_option('weather_write_images', $weather_images);

echo '</ul>';
echo "<p><strong>Database registration complete:</strong> {$db_success} images registered, {$db_error} errors.</p>";
echo '</div>';

// Update option to indicate images have been downloaded
update_option('weather_write_default_images_downloaded', 'yes');

/**
 * Download a file from a URL to a temporary location
 * 
 * @param string $url URL to download from
 * @return string|false Path to temporary file or false on failure
 */
function download_temp_file($url) {
    // Use WordPress HTTP API to download the file
    $response = wp_remote_get($url, array(
        'timeout' => 30,
        'sslverify' => false
    ));
    
    // Check for errors
    if (is_wp_error($response) || wp_remote_retrieve_response_code($response) !== 200) {
        return false;
    }
    
    // Get the file content
    $file_data = wp_remote_retrieve_body($response);
    
    // Create a temporary file
    $temp_file = wp_tempnam('weather-image-');
    if (file_put_contents($temp_file, $file_data)) {
        return $temp_file;
    }
    
    return false;
}

/**
 * Convert a PNG file to JPG
 * 
 * @param string $png_file Path to PNG file
 * @return string|false Path to JPG file or false on failure
 */
function convert_png_to_jpg($png_file) {
    // Check if GD is available
    if (!function_exists('imagecreatefrompng')) {
        return false;
    }
    
    // Create image from PNG
    $image = @imagecreatefrompng($png_file);
    if (!$image) {
        return false;
    }
    
    // Create a new JPG file
    $jpg_file = wp_tempnam('weather-image-') . '.jpg';
    
    // Set background to white for transparent PNGs
    $bg = imagecreatetruecolor(imagesx($image), imagesy($image));
    imagefill($bg, 0, 0, imagecolorallocate($bg, 255, 255, 255));
    imagealphablending($bg, true);
    imagecopy($bg, $image, 0, 0, 0, 0, imagesx($image), imagesy($image));
    
    // Save as JPG
    if (imagejpeg($bg, $jpg_file, 90)) {
        // Clean up
        imagedestroy($image);
        imagedestroy($bg);
        return $jpg_file;
    }
    
    // Clean up on failure
    imagedestroy($image);
    imagedestroy($bg);
    return false;
}

/**
 * Create a WordPress attachment from a file
 * 
 * @param string $file_path Path to the file
 * @param string $filename Filename to use for the attachment
 * @return int|false Attachment ID or false on failure
 */
function create_attachment_from_file($file_path, $filename) {
    // Check if file exists
    if (!file_exists($file_path)) {
        return false;
    }
    
    // Get file type
    $filetype = wp_check_filetype($filename, null);
    
    // Prepare attachment data
    $attachment = array(
        'guid' => $file_path,
        'post_mime_type' => $filetype['type'],
        'post_title' => preg_replace('/\.[^.]+$/', '', $filename),
        'post_content' => '',
        'post_status' => 'inherit'
    );
    
    // Insert the attachment
    $attachment_id = wp_insert_attachment($attachment, $file_path);
    
    if (!$attachment_id) {
        return false;
    }
    
    // Generate metadata for the attachment
    require_once(ABSPATH . 'wp-admin/includes/image.php');
    $attachment_data = wp_generate_attachment_metadata($attachment_id, $file_path);
    wp_update_attachment_metadata($attachment_id, $attachment_data);
    
    return $attachment_id;
}
