<?php
/**
 * Fix Weather Images Script
 * 
 * This script tests different CDN URLs and image formats to find working images
 * and updates the plugin settings accordingly.
 */

// Bootstrap WordPress
define('WP_USE_THEMES', false);
require_once('../../../../wp-load.php');

// Verify we're in WordPress
if (!function_exists('add_action')) {
    die('This script must be run within WordPress');
}

// Security check - only run if admin is logged in
if (!current_user_can('manage_options')) {
    wp_die('You must be an administrator to run this script');
}

// Define the CDN URLs to test
$cdn_urls = [
    'https://d27h5i1ihr021y.cloudfront.net/weatherwrite/weather/',
    'https://d27h5i1ihr021y.cloudfront.net/images/weather/',
    'https://d27h5i1ihr021y.cloudfront.net/weather/',
    'https://d27h5i1ihr021y.cloudfront.net/default/',
    'https://d27h5i1ihr021y.cloudfront.net/weatherwrite/',
];

// Define image formats to test
$image_formats = [
    'type/variation' => function($type, $variation, $cdn) { 
        return $cdn . $type . '/' . $variation . '.jpg'; 
    },
    'type_variation' => function($type, $variation, $cdn) { 
        return $cdn . $type . '_' . $variation . '.png'; 
    },
    'typevariation' => function($type, $variation, $cdn) { 
        return $cdn . $type . $variation . '.png'; 
    },
];

// Test types and variations
$test_types = ['sunny', 'partly_cloudy', 'overcast'];
$test_variations = [1, 2, 3];

// Function to check if URL exists
function url_exists($url) {
    $headers = @get_headers($url);
    return $headers && strpos($headers[0], '200') !== false;
}

// Find working CDN URL and format
$working_cdn = null;
$working_format = null;
$working_format_name = '';

echo '<h1>Testing CDN URLs and Image Formats</h1>';
echo '<table border="1" cellpadding="5" style="border-collapse: collapse;">';
echo '<tr><th>CDN URL</th><th>Format</th><th>Example URL</th><th>Status</th></tr>';

foreach ($cdn_urls as $cdn_url) {
    foreach ($image_formats as $format_name => $format_function) {
        $test_url = $format_function($test_types[0], $test_variations[0], $cdn_url);
        $exists = url_exists($test_url);
        
        echo '<tr>';
        echo '<td>' . htmlspecialchars($cdn_url) . '</td>';
        echo '<td>' . htmlspecialchars($format_name) . '</td>';
        echo '<td>' . htmlspecialchars($test_url) . '</td>';
        echo '<td>' . ($exists ? '<span style="color:green">Works</span>' : '<span style="color:red">Fails</span>') . '</td>';
        echo '</tr>';
        
        if ($exists && !$working_cdn) {
            $working_cdn = $cdn_url;
            $working_format = $format_function;
            $working_format_name = $format_name;
        }
    }
}

echo '</table>';

if ($working_cdn) {
    echo '<h2>Found Working Configuration</h2>';
    echo '<p>CDN URL: ' . htmlspecialchars($working_cdn) . '</p>';
    echo '<p>Format: ' . htmlspecialchars($working_format_name) . '</p>';
    
    // Update the Weather_Images class with the working CDN URL and format
    $weather_images_file = __DIR__ . '/class-weather-images.php';
    
    if (file_exists($weather_images_file)) {
        $file_contents = file_get_contents($weather_images_file);
        
        // Update CDN URL
        $file_contents = preg_replace(
            '/private \$cdn_url = \'[^\']+\';/',
            'private $cdn_url = \'' . $working_cdn . '\';',
            $file_contents
        );
        
        // Update image URL format based on the working format
        if ($working_format_name == 'type/variation') {
            $replacement = '        // Return default image from CDN if no custom image found
        $image_url = $this->cdn_url . $type . \'/\' . $variation . \'.jpg\';
        error_log(\'Weather Write - Loading default image from CDN: \' . $image_url);
        return $image_url;';
        } elseif ($working_format_name == 'type_variation') {
            $replacement = '        // Return default image from CDN if no custom image found
        $image_url = $this->cdn_url . $type . \'_\' . $variation . \'.png\';
        error_log(\'Weather Write - Loading default image from CDN: \' . $image_url);
        return $image_url;';
        } else {
            $replacement = '        // Return default image from CDN if no custom image found
        $image_url = $this->cdn_url . $type . $variation . \'.png\';
        error_log(\'Weather Write - Loading default image from CDN: \' . $image_url);
        return $image_url;';
        }
        
        // Replace the get_weather_image_url method's return statement
        $file_contents = preg_replace(
            '/\/\/ Return default image from CDN.*?return [^;]+;/s',
            $replacement,
            $file_contents
        );
        
        // Save the updated file
        if (file_put_contents($weather_images_file, $file_contents)) {
            echo '<p style="color:green">Successfully updated class-weather-images.php with working configuration!</p>';
        } else {
            echo '<p style="color:red">Failed to update class-weather-images.php</p>';
        }
    } else {
        echo '<p style="color:red">Could not find class-weather-images.php</p>';
    }
    
    // Display sample images with the working configuration
    echo '<h2>Sample Images with Working Configuration</h2>';
    echo '<div style="display: grid; grid-template-columns: repeat(auto-fill, minmax(200px, 1fr)); gap: 20px;">';
    
    foreach ($test_types as $type) {
        foreach ($test_variations as $variation) {
            $img_url = $working_format($type, $variation, $working_cdn);
            echo '<div style="border: 1px solid #ddd; padding: 10px; border-radius: 4px;">';
            echo '<p><strong>' . ucwords(str_replace('_', ' ', $type)) . ' (Variation ' . $variation . ')</strong></p>';
            echo '<img src="' . htmlspecialchars($img_url) . '" style="max-width: 100%; height: auto;" alt="' . htmlspecialchars($type . ' ' . $variation) . '">';
            echo '<p style="font-size: 12px; word-break: break-all;">' . htmlspecialchars($img_url) . '</p>';
            echo '</div>';
        }
    }
    
    echo '</div>';
    
    // Clear any cached images
    echo '<h2>Actions</h2>';
    echo '<p>Clearing image cache...</p>';
    delete_option('weather_write_images');
    delete_option('newspaper_ai_weather_images');
    echo '<p style="color:green">Image cache cleared!</p>';
    
    echo '<p><a href="../../../admin.php?page=newspaper-ai-weather&tab=images" style="display: inline-block; background: #2271b1; color: white; padding: 10px 15px; text-decoration: none; border-radius: 3px;">Go to Weather Images Tab</a></p>';
} else {
    echo '<h2 style="color:red">No Working Configuration Found</h2>';
    echo '<p>Could not find a working CDN URL and image format combination.</p>';
}
