<?php
/**
 * S3 Upload Script for Weather Images
 * 
 * This script creates sample weather images and provides instructions for uploading them to S3.
 * NOTE: This script does not directly upload to S3 as it would require AWS credentials.
 * Instead, it generates the images locally and provides instructions for manual upload.
 */

// Bootstrap WordPress
define('WP_USE_THEMES', false);
require_once('../../../../wp-load.php');

// Verify we're in WordPress
if (!function_exists('add_action')) {
    die('This script must be run within WordPress');
}

// Security check - only run if admin is logged in
if (!current_user_can('manage_options')) {
    wp_die('You must be an administrator to run this script');
}

// Create a temporary directory for the images
$temp_dir = WP_CONTENT_DIR . '/uploads/weather-images-temp/';
if (!file_exists($temp_dir)) {
    wp_mkdir_p($temp_dir);
}

// Define weather types
$weather_types = [
    'sunny', 'clear', 'partly_cloudy', 'mostly_cloudy', 'overcast',
    'rain_light', 'rain_heavy', 'showers', 'thunderstorm', 'thunderstorm_severe',
    'snow_light', 'snow_heavy', 'fog', 'mist', 'windy', 'hot', 'cold',
    'sunrise', 'sunset', 'sleet'
];

// Function to create a placeholder image with text
function create_placeholder_image($path, $type, $variation) {
    // Create a 600x400 image
    $image = imagecreatetruecolor(600, 400);
    
    // Define colors
    $bg_color = imagecolorallocate($image, 135, 206, 235); // Sky blue
    $text_color = imagecolorallocate($image, 255, 255, 255); // White
    $border_color = imagecolorallocate($image, 0, 0, 0); // Black
    
    // Customize background color based on weather type
    switch ($type) {
        case 'sunny':
        case 'clear':
            $bg_color = imagecolorallocate($image, 135, 206, 235); // Sky blue
            break;
        case 'partly_cloudy':
        case 'mostly_cloudy':
            $bg_color = imagecolorallocate($image, 169, 169, 169); // Light gray
            break;
        case 'overcast':
            $bg_color = imagecolorallocate($image, 105, 105, 105); // Dark gray
            break;
        case 'rain_light':
        case 'rain_heavy':
        case 'showers':
            $bg_color = imagecolorallocate($image, 65, 105, 225); // Royal blue
            break;
        case 'thunderstorm':
        case 'thunderstorm_severe':
            $bg_color = imagecolorallocate($image, 47, 79, 79); // Dark slate gray
            break;
        case 'snow_light':
        case 'snow_heavy':
        case 'sleet':
            $bg_color = imagecolorallocate($image, 220, 220, 220); // Gainsboro
            break;
        case 'fog':
        case 'mist':
            $bg_color = imagecolorallocate($image, 211, 211, 211); // Light gray
            break;
        default:
            $bg_color = imagecolorallocate($image, 135, 206, 235); // Sky blue
    }
    
    // Fill background
    imagefill($image, 0, 0, $bg_color);
    
    // Add border
    imagerectangle($image, 0, 0, 599, 399, $border_color);
    
    // Format the type name for display
    $display_type = ucwords(str_replace('_', ' ', $type));
    
    // Add text
    $text = $display_type . ' (Variation ' . $variation . ')';
    $font_size = 5;
    
    // Get text dimensions
    $text_width = imagefontwidth($font_size) * strlen($text);
    $text_height = imagefontheight($font_size);
    
    // Center the text
    $x = (600 - $text_width) / 2;
    $y = (400 + $text_height) / 2;
    
    // Draw the text
    imagestring($image, $font_size, $x, $y, $text, $text_color);
    
    // Save the image
    imagejpeg($image, $path, 90);
    
    // Free memory
    imagedestroy($image);
}

// Create images for each type and variation
$created_files = [];
foreach ($weather_types as $type) {
    for ($variation = 1; $variation <= 3; $variation++) {
        $image_path = $temp_dir . $type . '-' . $variation . '.jpg';
        create_placeholder_image($image_path, $type, $variation);
        $created_files[] = $image_path;
    }
}

// Create a zip file of all images
$zip_path = WP_CONTENT_DIR . '/uploads/weather-images.zip';
$zip = new ZipArchive();
if ($zip->open($zip_path, ZipArchive::CREATE | ZipArchive::OVERWRITE) === TRUE) {
    foreach ($created_files as $file) {
        $zip->addFile($file, basename($file));
    }
    $zip->close();
}

// Output HTML
echo '<div style="font-family: Arial, sans-serif; max-width: 800px; margin: 0 auto; padding: 20px;">';
echo '<h1>Weather Images S3 Upload Instructions</h1>';

echo '<p>This script has created sample weather images for all weather types and variations. These images need to be uploaded to an S3 bucket to be used as default images for the Weather Write plugin.</p>';

echo '<h2>Step 1: Download the Images</h2>';
echo '<p>Download the zip file containing all the weather images:</p>';
echo '<p><a href="' . content_url('/uploads/weather-images.zip') . '" style="display: inline-block; background: #2271b1; color: white; padding: 10px 15px; text-decoration: none; border-radius: 3px;">Download Weather Images ZIP</a></p>';

echo '<h2>Step 2: Create an S3 Bucket</h2>';
echo '<ol>';
echo '<li>Log in to the AWS Management Console</li>';
echo '<li>Navigate to the S3 service</li>';
echo '<li>Click "Create bucket"</li>';
echo '<li>Name the bucket "weatherwrite-default-images" (or choose another name)</li>';
echo '<li>Select a region close to your users</li>';
echo '<li>Configure other settings as needed (default settings are usually fine)</li>';
echo '<li>Click "Create bucket"</li>';
echo '</ol>';

echo '<h2>Step 3: Upload the Images to S3</h2>';
echo '<ol>';
echo '<li>Open the bucket you created</li>';
echo '<li>Click "Upload"</li>';
echo '<li>Drag and drop the extracted image files or click "Add files" to select them</li>';
echo '<li>Click "Upload"</li>';
echo '</ol>';

echo '<h2>Step 4: Set Permissions</h2>';
echo '<ol>';
echo '<li>Select all uploaded images</li>';
echo '<li>Click "Actions" and select "Make public"</li>';
echo '<li>Confirm by clicking "Make public"</li>';
echo '</ol>';

echo '<h2>Step 5: Update the S3 Base URL in the Installer</h2>';
echo '<p>After uploading the images to your S3 bucket, you need to update the S3 base URL in the installer class:</p>';
echo '<pre style="background: #f5f5f5; padding: 15px; border-radius: 5px; overflow-x: auto;">';
echo htmlspecialchars('// In class-weather-images-installer.php
private $s3_base_url = \'https://weatherwrite-default-images.s3.amazonaws.com/\';

// Replace with your actual S3 bucket URL
// Format: https://[bucket-name].s3.[region].amazonaws.com/');
echo '</pre>';

echo '<h2>Sample Images</h2>';
echo '<div style="display: grid; grid-template-columns: repeat(auto-fill, minmax(200px, 1fr)); gap: 20px;">';

// Show a few sample images
$sample_types = ['sunny', 'partly_cloudy', 'overcast', 'rain_light', 'thunderstorm'];
foreach ($sample_types as $type) {
    $image_path = $temp_dir . $type . '-1.jpg';
    $image_url = content_url('/uploads/weather-images-temp/' . $type . '-1.jpg');
    
    echo '<div style="border: 1px solid #ddd; padding: 10px; border-radius: 4px;">';
    echo '<p><strong>' . ucwords(str_replace('_', ' ', $type)) . '</strong></p>';
    echo '<img src="' . esc_url($image_url) . '" style="max-width: 100%; height: auto;" alt="' . esc_attr($type) . '">';
    echo '</div>';
}

echo '</div>';

echo '<h2>Next Steps</h2>';
echo '<p>After uploading the images to S3 and updating the S3 base URL in the installer class, you can test the installer by visiting:</p>';
echo '<p><a href="' . plugins_url('/Publisher_Plugin/modules/weather/test-installer.php', dirname(dirname(dirname(__FILE__)))) . '" style="display: inline-block; background: #2271b1; color: white; padding: 10px 15px; text-decoration: none; border-radius: 3px;">Test Installer</a></p>';

echo '</div>';
