<?php
/**
 * Geocoding functionality for Weather Module
 */

if (!defined('ABSPATH')) {
    exit;
}

class NewspaperAI_Geocoding {
    private static $instance = null;
    private $options_key = 'newspaper_ai_weather_location';

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        add_action('admin_init', array($this, 'register_settings'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_assets'));
        
        // Add action handlers
        add_action('admin_post_newspaper_ai_update_location', array($this, 'handle_update_location'));
        add_action('admin_post_nopriv_newspaper_ai_update_location', array($this, 'handle_unauthorized'));
        add_action('admin_notices', array($this, 'show_admin_notices'));
    }

    public function show_admin_notices() {
        $screen = get_current_screen();
        if ($screen->id !== 'newspaper-ai_page_newspaper-ai-weather-settings') {
            return;
        }

        if (isset($_GET['message']) && $_GET['message'] === 'location_updated') {
            ?>
            <div class="notice notice-success is-dismissible">
                <p><?php _e('Location updated successfully!', 'newspaper-ai'); ?></p>
            </div>
            <?php
        }

        if (isset($_GET['error'])) {
            $error_message = '';
            switch ($_GET['error']) {
                case 'empty_address':
                    $error_message = __('Please enter a valid address.', 'newspaper-ai');
                    break;
                case 'geocoding_failed':
                    $error_message = __('Failed to find coordinates for the provided address. Please check the address and try again.', 'newspaper-ai');
                    break;
            }

            if ($error_message) {
                ?>
                <div class="notice notice-error is-dismissible">
                    <p><?php echo esc_html($error_message); ?></p>
                </div>
                <?php
            }
        }
    }

    /**
     * Get the current location settings
     */
    public function get_location() {
        $defaults = array(
            'street' => '',
            'city' => '',
            'state' => '',
            'zip' => '',
            'lat' => '',
            'lng' => '',
            'last_updated' => '',
        );
        
        $location = wp_parse_args(get_option($this->options_key, array()), $defaults);

        return $location;
    }

    /**
     * Register settings for location
     */
    public function register_settings() {
        register_setting('newspaper_ai_weather_options', $this->options_key, array(
            'type' => 'array',
            'sanitize_callback' => array($this, 'sanitize_location'),
            'default' => array(
                'street' => '',
                'city' => '',
                'state' => '',
                'zip' => '',
                'lat' => '',
                'lng' => '',
                'last_updated' => '',
            ),
        ));
    }

    /**
     * Sanitize the location data
     */
    public function sanitize_location($input) {
        if (!is_array($input)) {
            return array();
        }

        $sanitized = array(
            'street' => sanitize_text_field($input['street'] ?? ''),
            'city' => sanitize_text_field($input['city'] ?? ''),
            'state' => sanitize_text_field($input['state'] ?? ''),
            'zip' => sanitize_text_field($input['zip'] ?? ''),
            'lat' => '',
            'lng' => '',
            'last_updated' => current_time('mysql')
        );

        // Only geocode if we have both city and state
        if (!empty($sanitized['city']) && !empty($sanitized['state'])) {
            $coordinates = $this->geocode_address($sanitized);
            if ($coordinates) {
                $sanitized['lat'] = $coordinates['lat'];
                $sanitized['lng'] = $coordinates['lng'];
            }
        }

        return $sanitized;
    }

    /**
     * Render location fields
     */
    public function render_location_fields() {
        $location = get_option('newspaper_ai_weather_location', array());
        ?>
        <div id="location-settings" class="location-fields">
            <div class="form-field">
                <label for="newspaper_ai_weather_location[street]"><?php echo esc_html__('Street Address', 'newspaper-ai'); ?></label>
                <input type="text" 
                       id="newspaper_ai_weather_location[street]" 
                       name="newspaper_ai_weather_location[street]" 
                       value="<?php echo esc_attr($location['street'] ?? ''); ?>"
                       class="regular-text">
            </div>

            <div class="form-field">
                <label for="newspaper_ai_weather_location[city]"><?php echo esc_html__('City', 'newspaper-ai'); ?></label>
                <input type="text" 
                       id="newspaper_ai_weather_location[city]" 
                       name="newspaper_ai_weather_location[city]" 
                       value="<?php echo esc_attr($location['city'] ?? ''); ?>"
                       class="regular-text">
            </div>

            <div class="form-field">
                <label for="newspaper_ai_weather_location[state]"><?php echo esc_html__('State', 'newspaper-ai'); ?></label>
                <input type="text" 
                       id="newspaper_ai_weather_location[state]" 
                       name="newspaper_ai_weather_location[state]" 
                       value="<?php echo esc_attr($location['state'] ?? ''); ?>"
                       class="regular-text">
            </div>

            <div class="form-field">
                <label for="newspaper_ai_weather_location[zip]"><?php echo esc_html__('ZIP Code', 'newspaper-ai'); ?></label>
                <input type="text" 
                       id="newspaper_ai_weather_location[zip]" 
                       name="newspaper_ai_weather_location[zip]" 
                       value="<?php echo esc_attr($location['zip'] ?? ''); ?>"
                       class="regular-text">
            </div>

            <div class="location-preview">
                <div id="location-map" style="height: 300px; margin: 16px 0; border-radius: 8px;"></div>
                <div class="location-info">
                    <p><strong><?php echo esc_html__('Current Location:', 'newspaper-ai'); ?></strong> 
                       <?php echo esc_html(implode(', ', array_filter(array(
                           $location['street'] ?? '',
                           $location['city'] ?? '',
                           $location['state'] ?? '',
                           $location['zip'] ?? ''
                       )))); ?>
                    </p>
                    <p><strong><?php echo esc_html__('Coordinates:', 'newspaper-ai'); ?></strong>
                       <?php if (!empty($location['lat']) && !empty($location['lng'])): ?>
                           <?php echo esc_html(round($location['lat'], 4) . ', ' . round($location['lng'], 4)); ?>
                       <?php else: ?>
                           <?php echo esc_html__('Not set', 'newspaper-ai'); ?>
                       <?php endif; ?>
                    </p>
                </div>
            </div>
        </div>

        <script>
        jQuery(document).ready(function($) {
            // Initialize map
            var lat = <?php echo !empty($location['lat']) ? esc_js($location['lat']) : 'null'; ?>;
            var lng = <?php echo !empty($location['lng']) ? esc_js($location['lng']) : 'null'; ?>;
            
            if (lat && lng) {
                var map = L.map('location-map').setView([lat, lng], 13);
                L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
                    attribution: ' OpenStreetMap contributors'
                }).addTo(map);
                L.marker([lat, lng]).addTo(map);
            }
        });
        </script>
        <?php
    }

    public function handle_update_location() {
        // Verify nonce
        if (!isset($_POST['newspaper_ai_location_nonce']) || !wp_verify_nonce($_POST['newspaper_ai_location_nonce'], 'newspaper_ai_update_location')) {
            wp_die(__('Invalid nonce. Please try again.', 'newspaper-ai'));
        }

        // Verify user capabilities
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have permission to perform this action.', 'newspaper-ai'));
        }

        // Get location data
        $location = array(
            'street' => sanitize_text_field($_POST['newspaper_ai_weather_location']['street'] ?? ''),
            'city' => sanitize_text_field($_POST['newspaper_ai_weather_location']['city'] ?? ''),
            'state' => sanitize_text_field($_POST['newspaper_ai_weather_location']['state'] ?? ''),
            'zip' => sanitize_text_field($_POST['newspaper_ai_weather_location']['zip'] ?? '')
        );

        // Build address string
        $address = implode(' ', array_filter($location));
        if (empty($address)) {
            wp_safe_redirect(add_query_arg(array(
                'page' => 'newspaper-ai-weather-settings',
                'error' => 'empty_address'
            ), admin_url('admin.php')));
            exit;
        }

        // Geocode the address
        $coordinates = $this->geocode_address($address);
        if (!$coordinates) {
            wp_safe_redirect(add_query_arg(array(
                'page' => 'newspaper-ai-weather-settings',
                'error' => 'geocoding_failed'
            ), admin_url('admin.php')));
            exit;
        }

        // Update location with coordinates
        $location['lat'] = $coordinates['lat'];
        $location['lng'] = $coordinates['lng'];
        $location['last_updated'] = current_time('mysql', true);

        // Save to options
        update_option('newspaper_ai_weather_location', $location);

        // Redirect back with success message
        wp_safe_redirect(add_query_arg(array(
            'page' => 'newspaper-ai-weather-settings',
            'message' => 'location_updated'
        ), admin_url('admin.php')));
        exit;
    }

    public function handle_unauthorized() {
        wp_die(__('You do not have permission to perform this action.', 'newspaper-ai'));
    }

    public function geocode_address($address) {
        if (is_array($address)) {
            // If address is an array, extract city and state
            if (!empty($address['city']) && !empty($address['state'])) {
                $address = $address['city'] . ', ' . $address['state'];
            } else {
                // Fall back to full address if city/state not available
                $address = implode(' ', array_filter($address));
            }
        }

        error_log('Geocoding address: ' . $address);

        // URL encode the address
        $address = urlencode($address);
        
        // Build the URL for Nominatim geocoding service
        $url = "https://nominatim.openstreetmap.org/search?q={$address}&format=json&limit=1";
        
        error_log('Geocoding URL: ' . $url);

        // Make the request
        $response = wp_remote_get($url, array(
            'headers' => array(
                'User-Agent' => 'WordPress Weather Plugin'
            ),
            'timeout' => 15
        ));
        
        // Check for errors
        if (is_wp_error($response)) {
            error_log('Geocoding Error: ' . $response->get_error_message());
            return false;
        }
        
        // Parse the response
        $body = wp_remote_retrieve_body($response);
        error_log('Geocoding Raw Response: ' . $body);
        
        $data = json_decode($body, true);
        
        if (json_last_error() !== JSON_ERROR_NONE) {
            error_log('Geocoding JSON Error: ' . json_last_error_msg());
            return false;
        }
        
        error_log('Geocoding Parsed Response: ' . print_r($data, true));
        
        if (empty($data[0]['lat']) || empty($data[0]['lon'])) {
            error_log('Geocoding Error: No coordinates found for address');
            return false;
        }
        
        // Ensure we have valid numeric coordinates
        $lat = floatval($data[0]['lat']);
        $lng = floatval($data[0]['lon']);  // Convert 'lon' to 'lng'

        if ($lat === 0.0 || $lng === 0.0) {
            error_log('Geocoding Error: Invalid coordinates (0,0)');
            return false;
        }

        $coordinates = array(
            'lat' => $lat,
            'lng' => $lng
        );

        error_log('Geocoding Success: ' . print_r($coordinates, true));
        return $coordinates;
    }

    /**
     * Enqueue admin assets
     */
    public function enqueue_admin_assets($hook) {
        // Only load on our settings page
        if ($hook !== 'newspaper-ai_page_newspaper-ai-weather-settings') {
            return;
        }

        wp_enqueue_style(
            'leaflet',
            'https://unpkg.com/leaflet@1.9.4/dist/leaflet.css',
            array(),
            '1.9.4'
        );
        
        wp_enqueue_script(
            'leaflet',
            'https://unpkg.com/leaflet@1.9.4/dist/leaflet.js',
            array('jquery'),
            '1.9.4',
            true
        );
    }

    /**
     * Check if we have valid coordinates
     */
    public function has_valid_coordinates() {
        $location = $this->get_location();
        return !empty($location['lat']) && !empty($location['lng']);
    }

    /**
     * Get coordinates as array
     */
    public function get_coordinates() {
        $location = $this->get_location();
        if ($this->has_valid_coordinates()) {
            return array(
                'lat' => $location['lat'],
                'lng' => $location['lng']
            );
        }
        return false;
    }
}
