<?php
/**
 * NOAA API Integration for Weather Module
 */

if (!defined('ABSPATH')) {
    exit;
}

class NewspaperAI_NOAA_API {
    private static $instance = null;
    private $api_base_url = 'https://api.weather.gov';

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        // Nothing to initialize
    }

    /**
     * Get weather forecast for a specific point
     */
    public function get_forecast($lat, $lng) {
        try {
            // Get current conditions first
            error_log('NOAA API: Getting current conditions...');
            $current_conditions = $this->get_current_conditions($lat, $lng);
            if (!$current_conditions) {
                error_log('NOAA API ERROR: Failed to get current conditions');
                return false;
            }

            // Get the daily forecast
            $points_data = $this->get_points_data($lat, $lng);
            if (!$points_data || !isset($points_data['properties']['forecast'])) {
                error_log('NOAA API ERROR: No forecast URL found in points data');
                return false;
            }

            $forecast_url = $points_data['properties']['forecast'];
            error_log('NOAA API - Forecast URL: ' . $forecast_url);
            
            $response = wp_remote_get($forecast_url);
            if (is_wp_error($response)) {
                error_log('NOAA API ERROR: Failed to fetch forecast - ' . $response->get_error_message());
                return array();
            }

            $body = wp_remote_retrieve_body($response);
            $data = json_decode($body, true);

            if (!$data || !isset($data['properties']['periods'])) {
                error_log('NOAA API ERROR: Invalid forecast data received');
                error_log('Response body: ' . print_r($body, true));
                return false;
            }

            $periods = $data['properties']['periods'];
            $forecast = array();

            // Find today's forecast period (first daytime period)
            $today_forecast = null;
            foreach ($periods as $period) {
                if ($period['isDaytime']) {
                    $today_forecast = $period;
                    break;
                }
            }

            if (!$today_forecast) {
                error_log('NOAA API ERROR: Could not find today\'s forecast period');
                return array();
            }

            // Log the conditions we're getting
            error_log('NOAA API - Current Conditions: ' . print_r($current_conditions, true));
            error_log('NOAA API - Today\'s Forecast: ' . print_r($today_forecast, true));
            error_log('NOAA API - Short Forecast: ' . $today_forecast['shortForecast']);
            
            // Calculate relative humidity from temperature and dew point if direct measurement isn't available
            $humidity = null;
            if (isset($current_conditions['raw_data']['relativeHumidity']['value'])) {
                $humidity = round($current_conditions['raw_data']['relativeHumidity']['value']);
            } elseif (isset($current_conditions['raw_data']['temperature']['value']) && 
                      isset($current_conditions['raw_data']['dewpoint']['value'])) {
                $t = $current_conditions['raw_data']['temperature']['value'];
                $td = $current_conditions['raw_data']['dewpoint']['value'];
                // Magnus formula for relative humidity
                $gamma_t = (17.271 * $t) / (237.7 + $t);
                $gamma_td = (17.271 * $td) / (237.7 + $td);
                $humidity = round(100 * (exp($gamma_td) / exp($gamma_t)));
            }

            // Get sunrise/sunset times from the grid data
            $sun_times = array('sunrise' => null, 'sunset' => null);
            if (isset($points_data['properties']['forecastGridData'])) {
                $grid_response = wp_remote_get($points_data['properties']['forecastGridData']);
                if (!is_wp_error($grid_response)) {
                    $grid_data = json_decode(wp_remote_retrieve_body($grid_response), true);
                    if (isset($grid_data['properties'])) {
                        if (isset($grid_data['properties']['sunrise']['values'][0]['value'])) {
                            $sun_times['sunrise'] = strtotime($grid_data['properties']['sunrise']['values'][0]['value']);
                        }
                        if (isset($grid_data['properties']['sunset']['values'][0]['value'])) {
                            $sun_times['sunset'] = strtotime($grid_data['properties']['sunset']['values'][0]['value']);
                        }
                    }
                }
            }

            // Find the correct high and low temperatures
            $current_temp = $current_conditions['temperature'];
            $high = $today_forecast['temperature'];
            
            // If current temperature is higher than the forecasted high, use that instead
            if ($current_temp > $high) {
                error_log("NOAA API: Current temp ({$current_temp}) is higher than forecasted high ({$high}). Using current temp as high.");
                $high = $current_temp;
            }
            
            $low = null;
            $current_date = date('Y-m-d');
            foreach ($periods as $period) {
                $period_date = substr($period['startTime'], 0, 10);
                if ($period_date === $current_date && !$period['isDaytime']) {
                    $low = $period['temperature'];
                    break;
                }
            }

            // Create today's forecast by combining current conditions with forecast
            $forecast['today'] = array(
                // Current conditions (real-time data)
                'temperature' => $current_conditions['temperature'],
                'station' => $current_conditions['station'],
                'station_name' => $current_conditions['station_name'],
                'observationTime' => $current_conditions['timestamp'],
                'humidity' => $humidity,
                'pressure' => isset($current_conditions['raw_data']['barometricPressure']['value']) ? round($current_conditions['raw_data']['barometricPressure']['value'] * 0.0295301, 2) : null,
                'visibility' => isset($current_conditions['raw_data']['visibility']['value']) ? round($current_conditions['raw_data']['visibility']['value'] * 0.000621371, 1) : null,
                'dewpoint' => isset($current_conditions['raw_data']['dewpoint']['value']) ? $this->celsius_to_fahrenheit($current_conditions['raw_data']['dewpoint']['value']) : null,
                'raw_data' => $current_conditions['raw_data'], // Add raw data for additional metrics
                'sunrise' => $sun_times['sunrise'],
                'sunset' => $sun_times['sunset'],
                
                // Forecast data
                'high' => $high,
                'low' => $low,
                'temperatureUnit' => $today_forecast['temperatureUnit'],
                'windSpeed' => $today_forecast['windSpeed'],
                'windDirection' => $today_forecast['windDirection'],
                'shortForecast' => $today_forecast['shortForecast'],
                'detailedForecast' => $today_forecast['detailedForecast'],
                'icon' => $today_forecast['icon'],
                'name' => 'Today',
                'startTime' => $today_forecast['startTime'],
                'endTime' => $today_forecast['endTime'],
                
                // Location data from points
                'city' => isset($points_data['properties']['relativeLocation']['properties']['city']) ? 
                    $points_data['properties']['relativeLocation']['properties']['city'] : '',
                'state' => isset($points_data['properties']['relativeLocation']['properties']['state']) ? 
                    $points_data['properties']['relativeLocation']['properties']['state'] : ''
            );

            error_log('NOAA API: Created today\'s forecast:');
            error_log(print_r($forecast['today'], true));

            // Process weekly forecast
            $forecast['weekly'] = array();
            $current_date = '';
            foreach ($periods as $period) {
                if ($period['isDaytime']) {
                    $period_date = substr($period['startTime'], 0, 10);
                    if ($period_date === $current_date) {
                        continue;  // Skip duplicate dates
                    }
                    $current_date = $period_date;
                    
                    $forecast['weekly'][] = array(
                        'date' => $period_date,
                        'name' => $period['name'],
                        'high' => $period['temperature'],
                        'low' => isset($periods[$period['number']]) ? $periods[$period['number']]['temperature'] : null,
                        'temperatureUnit' => $period['temperatureUnit'],
                        'windSpeed' => $period['windSpeed'],
                        'windDirection' => $period['windDirection'],
                        'shortForecast' => $period['shortForecast'],
                        'icon' => $period['icon']
                    );
                }
            }

            error_log('NOAA API: Final processed forecast:');
            error_log(print_r($forecast, true));
            return $forecast;
            
        } catch (Exception $e) {
            error_log('NOAA API Error in get_forecast: ' . $e->getMessage());
            return array();
        }
    }

    /**
     * Get current conditions for a specific point
     */
    public function get_current_conditions($lat, $lng) {
        error_log("\n=== NOAA API: Starting current conditions request ===");
        error_log("Location: lat=$lat, lng=$lng");
        
        // First get the points data to find the nearest observation station
        $points_data = $this->get_points_data($lat, $lng);
        if (!$points_data || !isset($points_data['properties']['observationStations'])) {
            error_log('NOAA API ERROR: No observation stations URL found');
            error_log('Points data: ' . print_r($points_data, true));
            return false;
        }

        // Get the list of observation stations
        $stations_url = $points_data['properties']['observationStations'];
        error_log('NOAA API - Points URL: ' . $points_data['properties']['forecastGridData']);
        error_log('NOAA API - Station URL: ' . $stations_url);
        
        $response = wp_remote_get($stations_url);
        if (is_wp_error($response)) {
            error_log('NOAA API ERROR: Failed to fetch stations - ' . $response->get_error_message());
            return false;
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);
        
        if (!$data || !isset($data['features']) || empty($data['features'])) {
            error_log('NOAA API ERROR: No stations found in response');
            error_log('Response body: ' . print_r($body, true));
            return false;
        }

        // Log all available stations for debugging
        error_log('Available stations:');
        foreach ($data['features'] as $index => $station) {
            error_log(sprintf(
                'Station %d: %s (Distance: approx %s miles)',
                $index + 1,
                $station['properties']['name'],
                isset($station['properties']['distance']) ? $station['properties']['distance'] : 'unknown'
            ));
        }

        // Get the first (nearest) station
        $station = $data['features'][0];
        $station_url = $station['id'] . '/observations/latest';
        error_log('NOAA API: Using nearest station: ' . $station['properties']['name']);
        error_log('Station URL: ' . $station_url);
        
        // Get the current observation
        error_log('NOAA API: Fetching current observation data...');
        $response = wp_remote_get($station_url, array(
            'timeout' => 15,
            'headers' => array(
                'Accept' => 'application/geo+json'
            )
        ));

        if (is_wp_error($response)) {
            error_log('NOAA API ERROR: Failed to fetch observation - ' . $response->get_error_message());
            return false;
        }

        $response_code = wp_remote_retrieve_response_code($response);
        if ($response_code !== 200) {
            error_log('NOAA API ERROR: Unexpected response code ' . $response_code);
            error_log('Response headers: ' . print_r(wp_remote_retrieve_headers($response), true));
            return false;
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);
        
        if (!$data) {
            error_log('NOAA API ERROR: Failed to decode observation data');
            error_log('Raw response: ' . $body);
            return false;
        }

        if (!isset($data['properties'])) {
            error_log('NOAA API ERROR: No properties in observation data');
            error_log('Data structure: ' . print_r($data, true));
            return false;
        }

        $properties = $data['properties'];
        
        // Detailed temperature checking
        if (!isset($properties['temperature'])) {
            error_log('NOAA API ERROR: No temperature object in properties');
            error_log('Available properties: ' . print_r(array_keys($properties), true));
            return false;
        }

        $temp_data = $properties['temperature'];
        error_log('NOAA API: Temperature data structure: ' . print_r($temp_data, true));

        // If temperature value is null or not set, try to get it from the hourly forecast
        if (!isset($temp_data['value']) || $temp_data['value'] === null) {
            error_log('NOAA API: Temperature value is null, trying hourly forecast...');
            $hourly_url = $points_data['properties']['forecastHourly'];
            $hourly_response = wp_remote_get($hourly_url);
            
            if (!is_wp_error($hourly_response)) {
                $hourly_data = json_decode(wp_remote_retrieve_body($hourly_response), true);
                if ($hourly_data && isset($hourly_data['properties']['periods'][0]['temperature'])) {
                    $temp_fahrenheit = $hourly_data['properties']['periods'][0]['temperature'];
                    $temp_data['value'] = ($temp_fahrenheit - 32) * 5/9; // Convert F to C
                    error_log('NOAA API: Successfully got temperature from hourly forecast: ' . $temp_fahrenheit . '°F');
                }
            }
        }

        // If still no temperature, try the grid data
        if (!isset($temp_data['value']) || $temp_data['value'] === null) {
            error_log('NOAA API: Still no temperature, trying forecast grid data...');
            $grid_url = $points_data['properties']['forecastGridData'];
            $grid_response = wp_remote_get($grid_url);
            
            if (!is_wp_error($grid_response)) {
                $grid_data = json_decode(wp_remote_retrieve_body($grid_response), true);
                if ($grid_data && isset($grid_data['properties']['temperature']['values'][0]['value'])) {
                    $temp_data['value'] = $grid_data['properties']['temperature']['values'][0]['value'];
                    error_log('NOAA API: Successfully got temperature from grid data: ' . $temp_data['value'] . '°C');
                }
            }
        }

        if (!isset($temp_data['value'])) {
            error_log('NOAA API ERROR: No temperature value in temperature object');
            return false;
        }

        $celsius = $temp_data['value'];
        if ($celsius === null) {
            error_log('NOAA API ERROR: Temperature value is null');
            // Try the next nearest station instead of giving up
            if (isset($data['features'][1])) {
                error_log('NOAA API: Attempting to use next nearest station...');
                // Recursive call with next station
                return $this->get_current_conditions($lat, $lng);
            }
            return false;
        }

        $fahrenheit = $this->celsius_to_fahrenheit($celsius);
        error_log(sprintf(
            'NOAA API: Temperature conversion - Celsius: %s → Fahrenheit: %s',
            $celsius,
            $fahrenheit
        ));

        // Log all available properties for debugging
        error_log('NOAA API: All available properties from station:');
        error_log(print_r($properties, true));
        
        error_log("=== NOAA API: Successfully retrieved current conditions ===\n");
        
        // Extract weather condition from textDescription
        $condition = '';
        if (isset($properties['textDescription'])) {
            $condition = $properties['textDescription'];
            error_log('NOAA API: Found condition from textDescription: ' . $condition);
        } elseif (isset($properties['icon'])) {
            // Extract condition from icon URL if available
            $icon_parts = explode('/', $properties['icon']);
            $condition = end($icon_parts);
            $condition = str_replace(['_', ','], ' ', $condition);
            $condition = str_replace('.png', '', $condition);
            error_log('NOAA API: Extracted condition from icon: ' . $condition);
        }

        return array(
            'temperature' => $fahrenheit,
            'station' => $station_url,
            'station_name' => $station['properties']['name'],
            'condition' => $condition,
            'timestamp' => isset($properties['timestamp']) ? $properties['timestamp'] : current_time('mysql'),
            'raw_data' => $properties
        );
    }

    /**
     * Get extended forecast for a specific point
     */
    public function get_extended_forecast($lat, $lng) {
        error_log("NOAA API: Getting extended forecast for lat: $lat, lng: $lng");
        
        // Get forecast data
        $forecast = $this->get_forecast($lat, $lng);
        if (!$forecast) {
            error_log("NOAA API Error: Failed to get forecast data for extended forecast");
            return false;
        }

        if (empty($forecast['weekly'])) {
            error_log("NOAA API Error: No weekly forecast in forecast data");
            error_log("NOAA API Forecast Data: " . print_r($forecast, true));
            return false;
        }

        // Get all periods
        $periods = $forecast['weekly'];
        error_log("NOAA API: Successfully retrieved extended forecast with " . count($periods) . " periods");
        
        // Format the periods
        $formatted_periods = array();
        foreach ($periods as $period) {
            $formatted_periods[] = array(
                'name' => $period['name'],
                'temperatureHigh' => $period['high'],
                'temperatureLow' => $period['low'],
                'temperatureUnit' => $period['temperatureUnit'],
                'shortForecast' => $period['shortForecast'],
                'detailedForecast' => $period['detailedForecast'],
                'windSpeed' => $period['windSpeed'],
                'windDirection' => $period['windDirection']
            );
        }

        return $formatted_periods;
    }

    /**
     * Get hourly forecast for a specific point
     */
    public function get_hourly_forecast($lat, $lng) {
        try {
            $points_data = $this->get_points_data($lat, $lng);
            if (!$points_data || !isset($points_data['properties']['forecastHourly'])) {
                error_log('NOAA API: No hourly forecast URL found in points data');
                return array();
            }

            $hourly_url = $points_data['properties']['forecastHourly'];
            $response = wp_remote_get($hourly_url);

            if (is_wp_error($response)) {
                error_log('NOAA API Error fetching hourly forecast: ' . $response->get_error_message());
                return array();
            }

            $body = wp_remote_retrieve_body($response);
            $data = json_decode($body, true);

            if (!$data || !isset($data['properties']['periods'])) {
                error_log('NOAA API: Invalid hourly forecast data received');
                error_log('Response body: ' . print_r($body, true));
                return array();
            }

            // Get current hour in the timezone from NOAA API
            $timezone = isset($data['properties']['timezone']) ? $data['properties']['timezone'] : 'America/Chicago';
            $current_time = new DateTime('now', new DateTimeZone($timezone));
            $current_timestamp = $current_time->getTimestamp();

            error_log('Current time in ' . $timezone . ': ' . $current_time->format('Y-m-d H:i:s'));

            $hourly_forecast = array();
            
            foreach ($data['properties']['periods'] as $period) {
                $period_time = new DateTime($period['startTime']);
                $period_timestamp = $period_time->getTimestamp();
                
                // Log the time comparison
                error_log(sprintf(
                    'Period time: %s, Current time: %s, Diff: %.2f hours',
                    $period_time->format('Y-m-d H:i:s'),
                    $current_time->format('Y-m-d H:i:s'),
                    ($period_timestamp - $current_timestamp) / 3600
                ));

                // Only include periods starting from the current hour and up to 24 hours ahead
                if ($period_timestamp >= $current_timestamp && 
                    $period_timestamp < ($current_timestamp + (24 * 3600)) && 
                    count($hourly_forecast) < 24) {
                    
                    $hourly_forecast[] = array(
                        'hour' => $period_time->format('ga'),
                        'temperature' => $period['temperature'],
                        'shortForecast' => $period['shortForecast'],
                        'windSpeed' => $period['windSpeed'],
                        'windDirection' => $period['windDirection'],
                        'timestamp' => $period_timestamp
                    );
                }
            }

            error_log('Final hourly forecast: ' . print_r($hourly_forecast, true));
            return $hourly_forecast;

        } catch (Exception $e) {
            error_log('NOAA API Error in get_hourly_forecast: ' . $e->getMessage());
            return array();
        }
    }

    /**
     * Get radar data URLs for overlaying on the map
     */
    public function get_radar_data() {
        return array(
            'base_reflectivity' => 'https://opengeo.ncep.noaa.gov/geoserver/conus/conus_bref_qcd/ows',
            'precipitation' => 'https://opengeo.ncep.noaa.gov/geoserver/conus/conus_qpf_1hr/ows'
        );
    }

    /**
     * Get radar image for a specific point
     */
    public function get_radar_image($lat, $lng) {
        return $this->get_radar_data();
    }

    /**
     * Fetch and cache radar image from NOAA
     */
    public function fetch_and_cache_radar() {
        try {
            // Create cache directory if it doesn't exist
            $cache_dir = NEWSPAPER_AI_PATH . 'cache/radar';
            if (!file_exists($cache_dir)) {
                wp_mkdir_p($cache_dir);
            }

            // Clean old files (keep last 24 hours)
            $files = glob($cache_dir . '/*');
            $now = time();
            foreach ($files as $file) {
                if (is_file($file)) {
                    if ($now - filemtime($file) >= 24 * 60 * 60) {
                        unlink($file);
                    }
                }
            }

            // Get coordinates from settings
            $location = get_option('newspaper_ai_weather_location', array());
            if (empty($location['lat']) || empty($location['lng'])) {
                throw new Exception('Location not set');
            }

            // Calculate bounding box (roughly 100 mile radius)
            $lat = floatval($location['lat']);
            $lng = floatval($location['lng']);
            $miles = 100;
            
            // Approximate degrees for bounding box
            $lat_degree = $miles/69; // 1 degree lat = ~69 miles
            $lng_degree = $miles/(cos($lat * M_PI/180) * 69);
            
            $bbox = [
                'west' => $lng - $lng_degree,
                'east' => $lng + $lng_degree,
                'north' => $lat + $lat_degree,
                'south' => $lat - $lat_degree
            ];

            // NOAA Ridge Radar REST API endpoint
            $radar_url = sprintf(
                'https://opengeo.ncep.noaa.gov/geoserver/conus/conus_bref_qcd/ows?service=WMS&version=1.3.0&request=GetMap&layers=conus_bref_qcd&bbox=%f,%f,%f,%f&width=800&height=800&format=image/png&transparent=true&crs=EPSG:4326',
                $bbox['south'],
                $bbox['west'],
                $bbox['north'],
                $bbox['east']
            );

            // Fetch radar image
            $response = wp_remote_get($radar_url);
            if (is_wp_error($response)) {
                throw new Exception('Failed to fetch radar image: ' . $response->get_error_message());
            }

            $image_data = wp_remote_retrieve_body($response);
            if (empty($image_data)) {
                throw new Exception('Empty radar image response');
            }

            // Save image with timestamp
            $timestamp = time();
            $filename = sprintf('radar_%s.png', $timestamp);
            $filepath = $cache_dir . '/' . $filename;
            
            if (file_put_contents($filepath, $image_data) === false) {
                throw new Exception('Failed to save radar image');
            }

            // Store metadata
            update_option('newspaper_ai_radar_metadata', array(
                'last_update' => $timestamp,
                'bbox' => $bbox,
                'filename' => $filename
            ));

            return true;
        } catch (Exception $e) {
            error_log('Weather Radar Error: ' . $e->getMessage());
            return false;
        }
    }

    /**
     * Get the latest cached radar image info
     */
    public function get_latest_radar_info() {
        try {
            $location = get_option('newspaper_ai_weather_location', array());
            if (empty($location['lat']) || empty($location['lng'])) {
                return null;
            }

            // Use the national radar composite instead of local station
            $radar_url = 'https://radar.weather.gov/ridge/standard/CONUS_loop.gif';
            
            // Check if radar image exists
            $headers = get_headers($radar_url);
            if (!$headers || strpos($headers[0], '200') === false) {
                error_log("National radar image not found");
                return null;
            }

            // National radar bounds (covers entire US)
            $bbox = array(
                21.0833, // South latitude
                -127.6167, // West longitude
                50.4167, // North latitude
                -66.9533 // East longitude
            );

            return array(
                'url' => $radar_url,
                'bbox' => $bbox,
                'timestamp' => current_time('timestamp')
            );
        } catch (Exception $e) {
            error_log('Error getting radar info: ' . $e->getMessage());
            return null;
        }
    }

    private function get_radar_station($lat, $lng) {
        // This is a simplified version - in reality you'd want to look up the nearest station
        // based on the lat/lng coordinates. For now we'll return a default station.
        return 'KOHX'; // Nashville radar
    }

    private function get_radar_bbox($station) {
        // Hardcoded bounding boxes for common stations
        $bboxes = array(
            'KOHX' => array(34.7, -87.5, 36.8, -85.2), // Nashville
            // Add more stations as needed
        );

        return isset($bboxes[$station]) ? $bboxes[$station] : null;
    }

    /**
     * Get points data for a specific lat/lng
     */
    public function get_points_data($lat, $lng) {
        $url = sprintf('%s/points/%s,%s', $this->api_base_url, $lat, $lng);
        error_log('NOAA API - Points URL: ' . $url);
        
        $response = wp_remote_get($url);
        
        if (is_wp_error($response)) {
            error_log('NOAA API Error fetching points data: ' . $response->get_error_message());
            return array();
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);

        if (!$data || !isset($data['properties'])) {
            error_log('NOAA API: Invalid points data received');
            error_log('Response body: ' . print_r($body, true));
            return array();
        }

        return $data;
    }

    public function celsius_to_fahrenheit($celsius) {
        if ($celsius === null) return null;
        return round(($celsius * 9/5) + 32);
    }
}
