<?php
/**
 * Reintent API Integration Class
 */
class NewspaperAI_Reintent_API {
    private static $instance = null;

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    // URLs
    const PURCHASE_URL = 'https://buy.stripe.com/7sIeYC8Ca6UJbDi8ww';
    const BILLING_URL = 'https://billing.stripe.com/p/login/7sI6oCd9ibIHdlScMM';
    const UPGRADE_URL = 'https://buy.stripe.com/14kg2G9Gefrf4aQ001';

    /**
     * Get the URL for purchasing a premium plan
     *
     * @return string Purchase URL
     */
    public function get_purchase_url() {
        return self::PURCHASE_URL;
    }

    /**
     * Get the URL for accessing billing portal
     *
     * @return string Billing URL
     */
    public function get_billing_url() {
        return self::BILLING_URL;
    }
    private $api_key;
    private $api_url = 'https://1xt5le1gn6.execute-api.us-east-2.amazonaws.com/v1/chatgpt';
    private $status_check_url = 'https://gdf0c610i0.execute-api.us-east-2.amazonaws.com/Testing/check-status';
    private $model = 'gpt-4o-mini';
    private $billing_url = 'https://billing.stripe.com/p/login/7sI6oCd9ibIHdlScMM';
    private $purchase_url = 'https://buy.stripe.com/7sIeYC8Ca6UJbDi8ww';
    private $upgrade_url = 'https://buy.stripe.com/14kg2G9Gefrf4aQ001';
    private $credits_per_report = 3;
    private $standard_daily_reports = 3;
    private $premium_daily_reports = 6;

    private function __construct() {
        $this->api_key = get_option('newspaperai_reintent_api_key', '');
        
        // Add AJAX handler for resync
        add_action('wp_ajax_reintent_resync_account', array($this, 'handle_resync_account'));
        
        if (is_admin()) {
            // Add with high priority to ensure it runs
            add_action('admin_notices', array($this, 'display_api_error_notices'), 99);
            add_action('wp_ajax_dismiss_reintent_notice', array($this, 'dismiss_notice'));
            
            // Check status on admin init
            add_action('admin_init', array($this, 'verify_account_status'));
        }
    }

    /**
     * Check if Reintent integration is configured
     */
    public function is_configured() {
        $configured = !empty($this->api_key);
        error_log('Reintent API - is_configured check - API key present: ' . ($configured ? 'yes' : 'no'));
        return $configured;
    }

    /**
     * Verify account status with ReIntent API
     */
    /**
     * Verify account status with ReIntent API
     * @return bool True if account is active, false otherwise
     */
    private $status_verified = false;

    /**
     * Force refresh the account status cache
     */
    public function force_refresh_status() {
        // Clear all transients
        delete_transient('reintent_account_status');
        delete_transient('reintent_credits_remaining');
        
        // Clear all notice dismissals
        delete_option('reintent_notice_dismissed_insufficient_credits');
        delete_option('reintent_notice_dismissed_inactive_user');
        delete_option('reintent_notice_dismissed_no_api_key');
        delete_option('reintent_notice_dismissed_low_credits_standard');
        delete_option('reintent_notice_dismissed_low_credits_premium');
        
        // Clear any API errors
        delete_option('reintent_api_error');
        
        // Force a fresh check
        return $this->verify_account_status(true);
    }

    /**
     * Verify account status with ReIntent API
     * 
     * @param bool $force_refresh Force a refresh of the cached status
     * @return bool True if account is active, false otherwise
     */
    public function verify_account_status($force_refresh = false) {
        // First check if API key exists
        if (!$this->is_configured()) {
            error_log('ReIntent API - No API key set, user considered inactive');
            // Cache inactive status for no API key
            set_transient('reintent_account_status', 'inactive', 12 * HOUR_IN_SECONDS);
            return false;
        }

        // For non-admin pages, use cached status
        if (!is_admin()) {
            error_log('ReIntent API - Not in admin, using cached status');
            $cached_status = get_transient('reintent_account_status');
            // If no cached status, consider inactive
            if ($cached_status === false) {
                error_log('ReIntent API - No cached status, considering inactive');
                return false;
            }
            return $cached_status === 'active';
        }

        // Check cache first unless force refresh
        if (!$force_refresh) {
            $cached_status = get_transient('reintent_account_status');
            if ($cached_status !== false) {
                newspaper_ai_debug_log('Using cached account status', 'ReIntentAPI');
                return $cached_status === 'active';
            }
        }

        // Clear any old persistent dismissals before check
        delete_option('reintent_notice_dismissed_insufficient_credits');
        delete_option('reintent_notice_dismissed_inactive_user');
        delete_option('reintent_notice_dismissed_no_api_key');
        delete_option('reintent_notice_dismissed_low_credits_standard');
        delete_option('reintent_notice_dismissed_low_credits_premium');
        if (!$this->is_configured()) {
            newspaper_ai_debug_log('No API key set, user considered inactive', 'ReIntentAPI');
            update_option('reintent_api_error', 'no_api_key');
            return false;
        }

        try {
            newspaper_ai_debug_log('Checking ReIntent account status...', 'ReIntentAPI');
            
            $response = wp_remote_post($this->status_check_url, array(
                'headers' => array(
                    'Content-Type' => 'application/json',
                ),
                'timeout' => 15,
                'body' => json_encode(array(
                    'access_key' => $this->api_key
                ))
            ));

            if (is_wp_error($response)) {
                newspaper_ai_debug_log('Status check failed: ' . $response->get_error_message(), 'ReIntentAPI');
                return false;
            }

            $response_code = wp_remote_retrieve_response_code($response);
            $response_body = json_decode(wp_remote_retrieve_body($response), true);
            
            newspaper_ai_debug_log('Status check response code: ' . $response_code . ', body: ' . print_r($response_body, true), 'ReIntentAPI');

            // Check for error responses first
            if ($response_code === 404 && isset($response_body['message']) && $response_body['message'] === 'User not found') {
                newspaper_ai_debug_log('Invalid API key detected - treating as inactive user', 'ReIntentAPI');
                update_option('reintent_api_error', 'inactive_user');
                // Cache inactive status for 12 hours
                set_transient('reintent_account_status', 'inactive', 12 * HOUR_IN_SECONDS);
                return false;
            }

            // Only proceed if we have a successful response
            if ($response_code === 200 && isset($response_body['status'])) {
                // Cache credits for 1 hour if available
                if (isset($response_body['credits'])) {
                    set_transient('reintent_credits_remaining', $response_body['credits'], HOUR_IN_SECONDS);
                }
                // Cache active status for 12 hours
                set_transient('reintent_account_status', 'active', 12 * HOUR_IN_SECONDS);
                // Caching disabled for testing
                // First check account status
                if ($response_body['status'] !== 'active') {
                    newspaper_ai_debug_log('Account status is not active: ' . $response_body['status'], 'ReIntentAPI');
                    update_option('reintent_api_error', 'inactive_user');
                    return false;
                }

                // Then check credits and plan type
                $credits = isset($response_body['credits']) ? intval($response_body['credits']) : 0;
                $plan_type = isset($response_body['plan_type']) ? $response_body['plan_type'] : '';
                
                // Update stored values
                update_option('reintent_credits_remaining', $credits);
                update_option('reintent_plan_type', $plan_type);
                
                newspaper_ai_debug_log("Credits: {$credits}, Plan: {$plan_type}", 'ReIntentAPI');
                
                // Calculate reports remaining and warning thresholds
                $reports_remaining = floor($credits / $this->credits_per_report);
                $min_reports = ($plan_type === 'premium') ? $this->premium_daily_reports : $this->standard_daily_reports;
                
                newspaper_ai_debug_log("Reports remaining: {$reports_remaining}, Min reports: {$min_reports}", 'ReIntentAPI');
                
                // Check credit status
                if ($credits < $this->credits_per_report) {
                    // Less than one report's worth of credits
                    newspaper_ai_debug_log('Setting insufficient credits error', 'ReIntentAPI');
                    update_option('reintent_api_error', 'insufficient_credits');
                    return false;
                } else if ($reports_remaining <= $min_reports) {
                    // Less than or equal to one day's worth of reports
                    $error_type = ($plan_type === 'premium') ? 'low_credits_premium' : 'low_credits_standard';
                    newspaper_ai_debug_log("Setting low credits warning: {$error_type}, Reports remaining: {$reports_remaining}", 'ReIntentAPI');
                    update_option('reintent_api_error', $error_type);
                    return true; // Still return true as they can still use the service
                }

                // If we get here, everything is good
                newspaper_ai_debug_log('Account is active and has sufficient credits, clearing errors', 'ReIntentAPI');
                delete_option('reintent_api_error');
                return true;
            }

            return false;
        } catch (Exception $e) {
            newspaper_ai_debug_log('Status check error: ' . $e->getMessage(), 'ReIntentAPI');
            return false;
        }
    }

    /**
     * Generate text using Reintent API
     */
    public function generate_text($prompt) {
        newspaper_ai_debug_log('Starting text generation', 'ReIntentAPI');
        
        if (!$this->is_configured()) {
            newspaper_ai_debug_log('Not configured, API key missing', 'ReIntentAPI');
            return false;
        }

        // Always check status while testing
        $this->verify_account_status();

        try {
            $request_body = array(
                'prompt' => $prompt,
                'model' => $this->model,
                'max_tokens' => 800
            );
            error_log('Reintent API - Request URL: ' . $this->api_url);
            error_log('Reintent API - Request body: ' . json_encode($request_body));
            
            $response = wp_remote_post($this->api_url, array(
                'headers' => array(
                    'x-api-key' => $this->api_key,
                    'Content-Type' => 'application/json',
                ),
                'timeout' => 15,
                'body' => json_encode($request_body)
            ));

            if (is_wp_error($response)) {
                throw new Exception('Reintent API request failed: ' . $response->get_error_message());
            }

            $response_code = wp_remote_retrieve_response_code($response);
            $response_body = wp_remote_retrieve_body($response);
            error_log('Reintent API - Response code: ' . $response_code);
            error_log('Reintent API - Raw response: ' . $response_body);
            
            $body = json_decode($response_body, true);
            if (json_last_error() !== JSON_ERROR_NONE) {
                error_log('Reintent API - JSON decode error: ' . json_last_error_msg());
                throw new Exception('Failed to parse Reintent response: ' . json_last_error_msg());
            }
            
            error_log('Reintent API - Decoded response: ' . print_r($body, true));
            
            if (isset($body['message'])) {
                if ($body['message'] === 'Insufficient credits' || $body['message'] === 'User account is inactive') {
                    // Always do a fresh status check on API error
                    $status_check = $this->verify_account_status();
                    
                    if (!$status_check) {
                        throw new Exception($body['message']);
                    }
                }
            }

            // Check for low credits
            if (isset($body['credits_remaining'])) {
                $credits = floatval($body['credits_remaining']);
                $reports_remaining = floor($credits / $this->credits_per_report);
                error_log('Weather Write - Credits remaining: ' . $credits . ', Reports remaining: ' . $reports_remaining);
                
                // Get the plan type based on scheduled reports
                $scheduled_reports = $this->get_daily_scheduled_reports();
                $is_standard_plan = $scheduled_reports <= $this->standard_daily_reports;
                error_log('Weather Write - Daily scheduled reports: ' . $scheduled_reports . ', Is standard plan: ' . ($is_standard_plan ? 'yes' : 'no'));
                
                // Show warning if less than 4 days worth of reports remaining
                $min_reports_needed = max(10, $scheduled_reports * 4); // At least 10 reports or 4 days worth
                error_log('Weather Write - Minimum reports needed: ' . $min_reports_needed);
                
                // Get current error state
                $current_error = get_option('reintent_api_error', '');
                
                if ($reports_remaining < $min_reports_needed) {
                    $warning_type = $is_standard_plan ? 'low_credits_standard' : 'low_credits_premium';
                    // Only set warning if it's not already dismissed
                    if (!get_transient('reintent_notice_dismissed_' . $warning_type)) {
                        update_option('reintent_api_error', $warning_type);
                        update_option('reintent_credits_remaining', $reports_remaining);
                        error_log('Weather Write - Setting low credits warning: ' . $warning_type . ' with ' . $reports_remaining . ' reports remaining');
                    }
                } else {
                    // If credits are now sufficient, clear any existing low credits warnings
                    if ($current_error === 'low_credits_standard' || $current_error === 'low_credits_premium') {
                        delete_option('reintent_api_error');
                        delete_option('reintent_credits_remaining');
                        delete_transient('reintent_notice_dismissed_low_credits_standard');
                        delete_transient('reintent_notice_dismissed_low_credits_premium');
                        error_log('Weather Write - Clearing low credits warning, now have ' . $reports_remaining . ' reports remaining');
                    }
                }
            }

            // Check for invalid or missing access key
            if (isset($body['message'])) {
                if ($body['message'] === 'User not found' || $body['message'] === 'Missing access_key in request') {
                    newspaper_ai_debug_log('Invalid or missing access key: ' . $body['message'], 'ReIntentAPI');
                    update_option('reintent_api_error', 'invalid_key');
                    return false;
                }
            }

            if (!isset($body['response'])) {
                error_log('Reintent API - Unexpected response format');
                update_option('reintent_api_error', 'invalid_key');
                throw new Exception('Unexpected Reintent response format');
            }

            $response = $body['response'];
            
            // Clean up any markdown code block markers
            $response = preg_replace('/^```html\s*|^```\s*|\s*```$/', '', $response);
            
            return trim($response);

        } catch (Exception $e) {
            error_log('Text generation error: ' . $e->getMessage());
            return false;
        }
    }

    /**
     * Display admin notices for API errors
     */
    public function display_api_error_notices() {
        if (!current_user_can('manage_options')) {
            newspaper_ai_debug_log('User cannot manage options, skipping notices', 'ReIntentAPI');
            return;
        }

        $error = get_option('reintent_api_error', '');
        newspaper_ai_debug_log('Current error state: ' . $error, 'ReIntentAPI');
        
        if (empty($error)) {
            newspaper_ai_debug_log('No error state set', 'ReIntentAPI');
            return;
        }

        // Check if notice is temporarily dismissed
        $notice_dismissed = get_transient('reintent_notice_dismissed_' . $error);
        if ($notice_dismissed) {
            return;
        }

        $message = '';
        $class = 'notice notice-error is-dismissible reintent-notice';

        switch ($error) {
            case 'invalid_key':
                $message = 'Your Access Key is invalid, please <a href="mailto:mike.freeman@reintent.com">contact our support team</a>.';
                break;
                
            case 'insufficient_credits':
                $message = sprintf(
                    'Your Weather Write account has insufficient credits. Please <a href="%s" target="_blank">log in to your billing portal</a> to update your plan.',
                    esc_url($this->billing_url)
                );
                break;
            case 'inactive_user':
                $message = sprintf(
                    'Your Weather Write account is currently inactive. Please <a href="%s" target="_blank">log in to your billing portal</a> to reactivate your account.',
                    esc_url($this->billing_url)
                );
                break;
            case 'no_api_key':
                $message = sprintf(
                    'To get the most out of Weather Write\'s features, please <a href="%s" target="_blank">purchase a premium plan</a>. Premium plans offer automated scheduling and auto-generated weather reports for your location.',
                    esc_url($this->purchase_url)
                );
                $class = 'notice notice-info is-dismissible reintent-notice';
                break;
                
            case 'low_credits_standard':
                $credits = get_option('reintent_credits_remaining', 0);
                $reports_remaining = floor($credits / $this->credits_per_report);
                $message = sprintf(
                    'You have %d weather reports remaining on your Weather Write account. Consider <a href="%s" target="_blank">upgrading to our premium plan</a> to double your daily report limit from %d to %d reports per day.',
                    $reports_remaining,
                    esc_url($this->upgrade_url),
                    $this->standard_daily_reports,
                    $this->premium_daily_reports
                );
                $class = 'notice notice-warning is-dismissible reintent-notice';
                break;
                
            case 'low_credits_premium':
                $credits = get_option('reintent_credits_remaining', 0);
                $reports_remaining = floor($credits / $this->credits_per_report);
                $message = sprintf(
                    'You have %d weather reports remaining on your Weather Write premium account. Please <a href="%s" target="_blank">purchase additional credits</a> to ensure uninterrupted service.',
                    $reports_remaining,
                    esc_url($this->billing_url)
                );
                $class = 'notice notice-warning is-dismissible reintent-notice';
                break;
        }

        if (!empty($message)) {
            printf(
                '<div class="%1$s" data-error-type="%2$s"><p>%3$s</p></div>',
                esc_attr($class),
                esc_attr($error),
                $message
            );
        }

        // Add JavaScript for AJAX dismissal
        ?>
        <script type="text/javascript">
        jQuery(document).ready(function($) {
            $('.reintent-notice').on('click', '.notice-dismiss', function() {
                var $notice = $(this).closest('.reintent-notice');
                $.post(ajaxurl, {
                    action: 'dismiss_reintent_notice',
                    error_type: $notice.data('error-type'),
                    nonce: '<?php echo wp_create_nonce('dismiss_reintent_notice'); ?>'
                });
            });
        });
        </script>
        <?php
    }

    /**
     * Handle AJAX dismissal of notices
     */
    /**
     * Handle account resync request
     */
    public function handle_resync_account() {
        check_ajax_referer('reintent_resync_account', 'nonce');
        
        if (!current_user_can('manage_options')) {
            wp_send_json_error('Unauthorized');
        }

        // Clear all transients
        delete_transient('reintent_account_status');
        delete_transient('reintent_credits_remaining');
        
        // Clear all notice dismissals
        delete_option('reintent_notice_dismissed_insufficient_credits');
        delete_option('reintent_notice_dismissed_inactive_user');
        delete_option('reintent_notice_dismissed_no_api_key');
        delete_option('reintent_notice_dismissed_low_credits_standard');
        delete_option('reintent_notice_dismissed_low_credits_premium');
        
        // Clear any API errors
        delete_option('reintent_api_error');
        
        // Make sure we have the latest key
        $this->api_key = get_option('newspaperai_reintent_api_key', '');
        
        // Force a fresh check
        $this->verify_account_status(true);
        wp_send_json_success();
    }

    public function dismiss_notice() {
        if (!current_user_can('manage_options')) {
            wp_die(-1);
        }

        check_ajax_referer('dismiss_reintent_notice', 'nonce');

        $error_type = isset($_POST['error_type']) ? sanitize_text_field($_POST['error_type']) : '';
        if (!empty($error_type)) {
            // All notices will reappear on page refresh
            set_transient('reintent_notice_dismissed_' . $error_type, true, 30);
        }

        wp_die();
    }

    /**
     * Get remaining credits from cache or API
     * 
     * @param bool $force_refresh Force a refresh of the cached credits
     * @return int Number of credits remaining
     */
    public function get_remaining_credits($force_refresh = false) {
        if (!$force_refresh) {
            $cached_credits = get_transient('reintent_credits_remaining');
            if ($cached_credits !== false) {
                return (int) $cached_credits;
            }
        }

        // If we're here, either cache is expired or force refresh was requested
        $this->verify_account_status(true);
        return (int) get_option('reintent_credits_remaining', 0);
    }

    /**
     * Get the number of daily scheduled reports
     */
    private function get_daily_scheduled_reports() {
        $schedule = get_option('newspaperai_weather_schedule', array());
        $daily_count = 0;
        
        if (!empty($schedule) && is_array($schedule)) {
            foreach ($schedule as $day => $times) {
                if (is_array($times)) {
                    $daily_count = max($daily_count, count($times));
                }
            }
        }
        
        return $daily_count;
    }
}
