<?php
/**
 * Weather Images Fallback
 * 
 * Handles creating and managing fallback weather images when default images are missing
 */
class NewspaperAI_Weather_Images_Fallback {
    private static $instance = null;
    private $upload_dir;
    private $weather_images_dir;
    
    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    private function __construct() {
        // Set up paths
        $upload_dir = wp_upload_dir();
        $this->upload_dir = $upload_dir;
        $this->weather_images_dir = $upload_dir['basedir'] . '/weather-images/';
        
        // Make sure the directory exists
        wp_mkdir_p($this->weather_images_dir);
    }
    
    /**
     * Create a fallback image for a weather condition
     * 
     * @param string $type Weather condition type (e.g., 'sunny', 'cloudy')
     * @param int $variation Variation number (1-3)
     * @return string|false Path to the created image, or false on failure
     */
    public function create_fallback_image($type, $variation) {
        // Define image dimensions
        $width = 1200;
        $height = 800;
        
        // Create the image
        $image = imagecreatetruecolor($width, $height);
        
        // Define colors based on weather type
        $colors = $this->get_colors_for_type($type);
        $bg_color = imagecolorallocate($image, $colors['bg'][0], $colors['bg'][1], $colors['bg'][2]);
        $text_color = imagecolorallocate($image, $colors['text'][0], $colors['text'][1], $colors['text'][2]);
        
        // Fill the background
        imagefill($image, 0, 0, $bg_color);
        
        // Add a pattern based on variation
        $this->add_pattern($image, $variation, $colors);
        
        // Add text
        $display_text = str_replace('_', ' ', ucwords($type));
        $font_size = 40;
        
        // Calculate text position to center it
        $text_box = imagettfbbox($font_size, 0, 5, $display_text);
        $text_width = $text_box[2] - $text_box[0];
        $text_height = $text_box[1] - $text_box[7];
        $text_x = ($width - $text_width) / 2;
        $text_y = ($height - $text_height) / 2 + $text_height;
        
        // Draw text
        imagestring($image, 5, $text_x, $text_y - 20, $display_text, $text_color);
        imagestring($image, 3, $text_x, $text_y + 20, "Variation " . $variation, $text_color);
        
        // Save the image
        $filename = $type . '-' . $variation . '.jpg';
        $file_path = $this->weather_images_dir . $filename;
        
        // Create JPEG
        imagejpeg($image, $file_path, 90);
        imagedestroy($image);
        
        error_log('Weather Write - Created fallback image: ' . $file_path);
        
        return file_exists($file_path) ? $file_path : false;
    }
    
    /**
     * Get colors for a weather type
     * 
     * @param string $type Weather condition type
     * @return array Array with 'bg' and 'text' color values
     */
    private function get_colors_for_type($type) {
        $colors = [
            'sunny' => [
                'bg' => [255, 200, 100], // Light orange
                'text' => [50, 50, 50]   // Dark gray
            ],
            'clear' => [
                'bg' => [100, 180, 255], // Light blue
                'text' => [255, 255, 255] // White
            ],
            'partly_cloudy' => [
                'bg' => [180, 220, 255], // Light blue
                'text' => [50, 50, 50]   // Dark gray
            ],
            'mostly_cloudy' => [
                'bg' => [150, 170, 200], // Grayish blue
                'text' => [255, 255, 255] // White
            ],
            'overcast' => [
                'bg' => [120, 120, 120], // Gray
                'text' => [255, 255, 255] // White
            ],
            'rain_light' => [
                'bg' => [100, 130, 170], // Blue gray
                'text' => [255, 255, 255] // White
            ],
            'rain_heavy' => [
                'bg' => [70, 90, 120],   // Dark blue gray
                'text' => [255, 255, 255] // White
            ],
            'showers' => [
                'bg' => [90, 110, 150],  // Medium blue gray
                'text' => [255, 255, 255] // White
            ],
            'thunderstorm' => [
                'bg' => [60, 60, 100],   // Dark purple blue
                'text' => [255, 255, 255] // White
            ],
            'thunderstorm_severe' => [
                'bg' => [40, 40, 70],    // Very dark purple
                'text' => [255, 255, 255] // White
            ],
            'snow_light' => [
                'bg' => [220, 230, 240], // Very light blue
                'text' => [50, 50, 50]   // Dark gray
            ],
            'snow_heavy' => [
                'bg' => [200, 210, 220], // Light gray blue
                'text' => [50, 50, 50]   // Dark gray
            ],
            'fog' => [
                'bg' => [200, 200, 200], // Light gray
                'text' => [50, 50, 50]   // Dark gray
            ],
            'mist' => [
                'bg' => [210, 220, 220], // Very light gray
                'text' => [50, 50, 50]   // Dark gray
            ],
            'windy' => [
                'bg' => [180, 200, 220], // Light blue gray
                'text' => [50, 50, 50]   // Dark gray
            ],
            'hot' => [
                'bg' => [255, 150, 100], // Orange
                'text' => [50, 50, 50]   // Dark gray
            ],
            'cold' => [
                'bg' => [200, 240, 255], // Very light blue
                'text' => [50, 50, 50]   // Dark gray
            ],
            'sunrise' => [
                'bg' => [255, 170, 100], // Orange
                'text' => [50, 50, 50]   // Dark gray
            ],
            'sunset' => [
                'bg' => [255, 120, 80],  // Deep orange
                'text' => [255, 255, 255] // White
            ],
            'sleet' => [
                'bg' => [180, 190, 200], // Light gray
                'text' => [50, 50, 50]   // Dark gray
            ]
        ];
        
        // Default colors if type not found
        if (!isset($colors[$type])) {
            return [
                'bg' => [150, 150, 150], // Medium gray
                'text' => [255, 255, 255] // White
            ];
        }
        
        return $colors[$type];
    }
    
    /**
     * Add a pattern to the image based on variation
     * 
     * @param resource $image GD image resource
     * @param int $variation Variation number (1-3)
     * @param array $colors Color array
     */
    private function add_pattern($image, $variation, $colors) {
        $width = imagesx($image);
        $height = imagesy($image);
        
        // Create a slightly different color for the pattern
        $pattern_color = imagecolorallocate(
            $image, 
            min(255, $colors['bg'][0] - 30), 
            min(255, $colors['bg'][1] - 30), 
            min(255, $colors['bg'][2] - 30)
        );
        
        switch ($variation) {
            case 1:
                // Diagonal lines
                for ($i = -$height; $i < $width; $i += 20) {
                    imageline($image, $i, 0, $i + $height, $height, $pattern_color);
                }
                break;
                
            case 2:
                // Grid pattern
                for ($i = 0; $i < $width; $i += 40) {
                    imageline($image, $i, 0, $i, $height, $pattern_color);
                }
                for ($i = 0; $i < $height; $i += 40) {
                    imageline($image, 0, $i, $width, $i, $pattern_color);
                }
                break;
                
            case 3:
                // Circles
                $max_radius = min($width, $height) / 4;
                for ($radius = $max_radius; $radius > 0; $radius -= 40) {
                    imageellipse($image, $width / 2, $height / 2, $radius * 2, $radius * 2, $pattern_color);
                }
                break;
        }
    }
    
    /**
     * Get or create a fallback image
     * 
     * @param string $type Weather condition type
     * @param int $variation Variation number
     * @return string|false Path to the image or false on failure
     */
    public function get_fallback_image($type, $variation) {
        $filename = $type . '-' . $variation . '.jpg';
        $file_path = $this->weather_images_dir . $filename;
        
        // Check if the image already exists
        if (file_exists($file_path)) {
            return $file_path;
        }
        
        // Create the image if it doesn't exist
        return $this->create_fallback_image($type, $variation);
    }
    
    /**
     * Add a fallback image to the media library and attach it to a post
     * 
     * @param int $post_id Post ID to attach the image to
     * @param string $type Weather condition type
     * @param int $variation Variation number
     * @return int|false Attachment ID or false on failure
     */
    public function add_fallback_image_to_media_library($post_id, $type, $variation) {
        // Get or create the fallback image
        $file_path = $this->get_fallback_image($type, $variation);
        
        if (!$file_path) {
            error_log('Weather Write - Failed to get fallback image for ' . $type . '-' . $variation);
            return false;
        }
        
        // Prepare attachment data
        $filename = basename($file_path);
        $wp_filetype = wp_check_filetype($filename, null);
        
        $attachment = array(
            'post_mime_type' => $wp_filetype['type'],
            'post_title' => sanitize_text_field(ucwords(str_replace('_', ' ', $type)) . ' Weather'),
            'post_content' => '',
            'post_status' => 'inherit'
        );
        
        // Insert the attachment
        $attach_id = wp_insert_attachment($attachment, $file_path, $post_id);
        
        if (is_wp_error($attach_id)) {
            error_log('Weather Write - Failed to insert attachment: ' . $attach_id->get_error_message());
            return false;
        }
        
        // Generate attachment metadata
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        $attach_data = wp_generate_attachment_metadata($attach_id, $file_path);
        wp_update_attachment_metadata($attach_id, $attach_data);
        
        error_log('Weather Write - Added fallback image to media library with ID: ' . $attach_id);
        
        return $attach_id;
    }
}
