<?php
/**
 * Weather Images Installer
 * 
 * Handles registering default weather images during plugin installation
 */
class NewspaperAI_Weather_Images_Installer {
    private static $instance = null;
    private $images_dir;
    private $images_url;
    
    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }
    
    private function __construct() {
        // Set up paths
        $plugin_dir = plugin_dir_path(dirname(dirname(__FILE__)));
        
        // Ensure we have a proper URL by using site_url
        $site_url = site_url();
        $plugin_url = trailingslashit($site_url) . 'wp-content/plugins/Publisher_Plugin/';
        
        $this->images_dir = $plugin_dir . 'assets/images/weather/';
        $this->images_url = $plugin_url . 'assets/images/weather/';
        
        error_log("Weather Write - Images directory: {$this->images_dir}");
        error_log("Weather Write - Images URL: {$this->images_url}");
        
        // Register activation hook
        register_activation_hook($plugin_dir . 'publisher-plugin.php', array($this, 'on_activation'));
        
        // Add action to check for images on admin init (in case activation hook didn't run)
        add_action('admin_init', array($this, 'check_default_images'));
    }
    
    /**
     * Run on plugin activation
     */
    public function on_activation() {
        $this->register_default_images();
    }
    
    /**
     * Check if default images are registered and register them if not
     */
    public function check_default_images() {
        // Only run this check on the weather page
        if (!isset($_GET['page']) || $_GET['page'] !== 'weatherai') {
            return;
        }
        
        // Check if images are registered in the database
        $weather_images = get_option('weather_write_images', array());
        if (empty($weather_images)) {
            $this->register_default_images();
        }
    }
    
    /**
     * Register default images in the database
     */
    public function register_default_images() {
        // Define the type mapping for all weather conditions
        $type_map = [
            'sunny' => 'sunny',
            'clear' => 'sunny',
            'partly_cloudy' => 'cloudy',
            'partly_sunny' => 'cloudy',
            'mostly_cloudy' => 'cloudy',
            'overcast' => 'overcast',
            'rain_light' => 'rain',
            'rain_heavy' => 'rain',
            'showers' => 'rain',
            'thunderstorm' => 'lightning',
            'thunderstorm_severe' => 'lightning',
            'snow_light' => 'snow',
            'snow_heavy' => 'snow',
            'fog' => 'fog',
            'mist' => 'fog',
            'haze' => 'fog',
            'windy' => 'cloudy',
            'hot' => 'sunny',
            'cold' => 'snow',
            'sunrise' => 'sunny',
            'sunset' => 'sunny',
            'sleet' => 'snow',
            'storm' => 'lightning'
        ];
        
        // Get current weather images option
        $weather_images = get_option('weather_write_images', array());
        
        // Register all weather image types and variations
        $base_types = ['sunny', 'cloudy', 'fog', 'lightning', 'overcast', 'rain', 'snow'];
        
        foreach ($type_map as $weather_type => $base_type) {
            for ($variation = 1; $variation <= 3; $variation++) {
                $image_path = $this->images_dir . $weather_type . '-' . $variation . '.jpg';
                $image_url = $this->images_url . $weather_type . '-' . $variation . '.jpg';
                
                // Create the database key (with underscore format)
                $db_key = $weather_type . '_' . $variation;
                
                // Check if we already have this image in the database
                if (isset($weather_images[$db_key]) && 
                    ((is_array($weather_images[$db_key]) && !empty($weather_images[$db_key]['url'])) || 
                     (!is_array($weather_images[$db_key]) && !empty($weather_images[$db_key])))) {
                    continue;
                }
                
                // Check if the image file exists
                if (file_exists($image_path)) {
                    // Create a WordPress attachment for the image if it doesn't exist
                    $attachment_id = $this->create_attachment_from_file($image_path, $weather_type . '-' . $variation . '.jpg');
                    
                    if ($attachment_id) {
                        // Update the database entry
                        $weather_images[$db_key] = array(
                            'id' => $attachment_id,
                            'url' => $image_url
                        );
                    } else {
                        // If we can't create an attachment, just store the URL
                        $weather_images[$db_key] = array(
                            'id' => 0,
                            'url' => $image_url
                        );
                    }
                }
            }
        }
        
        // Save the updated weather images option
        update_option('weather_write_images', $weather_images);
    }
    
    /**
     * Create a WordPress attachment from a file
     * 
     * @param string $file_path Path to the file
     * @param string $filename Filename to use for the attachment
     * @return int|false Attachment ID or false on failure
     */
    private function create_attachment_from_file($file_path, $filename) {
        // Check if file exists
        if (!file_exists($file_path)) {
            return false;
        }
        
        // Get file type
        $filetype = wp_check_filetype($filename, null);
        
        // Prepare attachment data
        $attachment = array(
            'guid' => $file_path,
            'post_mime_type' => $filetype['type'],
            'post_title' => preg_replace('/\.[^.]+$/', '', $filename),
            'post_content' => '',
            'post_status' => 'inherit'
        );
        
        // Insert the attachment
        $attachment_id = wp_insert_attachment($attachment, $file_path);
        
        if (!$attachment_id) {
            return false;
        }
        
        // Generate metadata for the attachment
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        $attachment_data = wp_generate_attachment_metadata($attachment_id, $file_path);
        wp_update_attachment_metadata($attachment_id, $attachment_data);
        
        return $attachment_id;
    }
    

    
    /**
     * Get the URL for a default image
     * 
     * @param string $type Weather type
     * @param int $variation Variation number (1-3)
     * @return string URL to the image
     */
    public function get_default_image_url($type, $variation) {
        // Normalize the type (replace spaces with underscores) and ensure variation is an integer
        $type = str_replace(' ', '_', strtolower(trim($type)));
        $variation = max(1, min(3, (int)$variation));
        
        // Define the local image path and URL
        $image_path = $this->images_dir . $type . '-' . $variation . '.jpg';
        $image_url = $this->images_url . $type . '-' . $variation . '.jpg';
        
        // Check if the image exists locally
        if (file_exists($image_path)) {
            return $image_url;
        }
        
        // Define type mappings for weather conditions
        $type_map = [
            // UI-specific to base mappings
            'partly_cloudy' => 'cloudy',  // UI: "Partly Cloudy/Partly Sunny"
            'partly_sunny' => 'cloudy',
            'mostly_cloudy' => 'cloudy',
            'storm' => 'lightning',       // UI: "Thunderstorm/Severe"
            'thunderstorm' => 'lightning',
            'thunderstorm_severe' => 'lightning',
            'rain_light' => 'rain',
            'rain_heavy' => 'rain',
            'showers' => 'rain',
            'snow_light' => 'snow',
            'snow_heavy' => 'snow',
            'sleet' => 'snow',
            'cold' => 'snow',
            'mist' => 'fog',
            'haze' => 'fog',
            'clear' => 'sunny',
            'hot' => 'sunny',
            'sunrise' => 'sunny',
            'sunset' => 'sunny',
            'windy' => 'cloudy',
            
            // Alternative names with spaces
            'partly cloudy' => 'partly_cloudy',
            'partly sunny' => 'partly_sunny',
            'mostly cloudy' => 'mostly_cloudy'
        ];
        
        // Get the base type for this weather type
        $base_type = $type;
        if (isset($type_map[$type])) {
            $base_type = $type_map[$type];
            
            // If the mapping points to another UI type (not a base type), resolve it
            if (isset($type_map[$base_type])) {
                $base_type = $type_map[$base_type];
            }
        }
        
        // Check if we should try a different local image path based on the mapping
        if ($type !== $base_type) {
            $mapped_path = $this->images_dir . $base_type . '-' . $variation . '.jpg';
            $mapped_url = $this->images_url . $base_type . '-' . $variation . '.jpg';
            
            if (file_exists($mapped_path)) {
                // Try to copy the image to create the mapped version
                if (copy($mapped_path, $image_path)) {
                    return $image_url;
                } else {
                    // If copy fails, return the mapped URL directly
                    return $mapped_url;
                }
            }
        }
        
        // Try to use a different variation if this one doesn't exist
        for ($i = 1; $i <= 3; $i++) {
            if ($i == $variation) continue;
            $alt_path = $this->images_dir . $type . '-' . $i . '.jpg';
            $alt_url = $this->images_url . $type . '-' . $i . '.jpg';
            
            if (file_exists($alt_path)) {
                return $alt_url;
            }
        }
        
        // Use a fallback image if available
        $fallback_types = ['cloudy', 'sunny', 'overcast'];
        foreach ($fallback_types as $fallback_type) {
            $fallback_path = $this->images_dir . $fallback_type . '-1.jpg';
            $fallback_url = $this->images_url . $fallback_type . '-1.jpg';
            
            if (file_exists($fallback_path)) {
                return $fallback_url;
            }
        }
        
        // If we still don't have an image, return a default
        return $this->images_url . 'cloudy-1.jpg';
    }
}

// Initialize the installer
NewspaperAI_Weather_Images_Installer::get_instance();
