<?php
/**
 * Class to handle weather image management
 */
class NewspaperAI_Weather_Images {
    private static $instance = null;
    private $reintent_api = null;
    private $installer;

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        $this->reintent_api = NewspaperAI_Reintent_API::get_instance();
        add_action('wp_ajax_newspaper_ai_save_weather_image', array($this, 'handle_save_weather_image'));
        add_action('wp_ajax_newspaper_ai_delete_all_weather_images', array($this, 'handle_delete_all_weather_images'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
        
        // Load the installer
        require_once dirname(__FILE__) . '/class-weather-images-installer.php';
        $this->installer = NewspaperAI_Weather_Images_Installer::get_instance();
        
        // Load the fallback image system
        require_once dirname(__FILE__) . '/class-weather-images-fallback.php';
        $this->fallback = NewspaperAI_Weather_Images_Fallback::get_instance();
        
        // Migrate old data if needed
        $this->maybe_migrate_weather_images();
    }

    /**
     * Handle deleting all weather images
     */
    public function handle_delete_all_weather_images() {
        try {
            // Verify nonce
            if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'newspaper_ai_weather_images')) {
                throw new Exception('Invalid nonce');
            }

            // Delete both old and new options
            delete_option('weather_write_images');
            delete_option('newspaper_ai_weather_images');

            wp_send_json_success(array(
                'message' => 'All weather images deleted successfully'
            ));

        } catch (Exception $e) {
            wp_send_json_error($e->getMessage());
        }
    }

    /**
     * Migrate weather images from old option to new option
     */
    private function maybe_migrate_weather_images() {
        $old_images = get_option('newspaper_ai_weather_images', array());
        $new_images = get_option('weather_write_images', array());

        // Only migrate if we have old data and no new data
        if (!empty($old_images) && empty($new_images)) {
            update_option('weather_write_images', $old_images);
        }
    }

    /**
     * Check if user has premium access
     */
    public function is_premium() {
        // Check account status - this handles both no API key and inactive key
        $status = $this->reintent_api->verify_account_status();
        error_log('Weather Write - Premium check: ' . ($status ? 'true' : 'false'));
        return $status;
    }

    /**
     * Enqueue admin scripts for weather images
     */
    public function enqueue_admin_scripts($hook) {
        error_log('Weather Write - Current page hook: ' . $hook);
        
        // Only load on the Weather Write admin pages
        if (strpos($hook, 'toplevel_page_newspaper-ai-weather') === false) {
            error_log('Weather Write - Skipping script load for hook: ' . $hook);
            return;
        }
        
        error_log('Weather Write - Loading scripts for hook: ' . $hook);

        // Enqueue Dashicons
        wp_enqueue_style('dashicons');

        // Register and enqueue our custom CSS
        $css_url = plugin_dir_url(dirname(__FILE__)) . 'weather/assets/admin/weather-admin.css';
        error_log('Weather Write - Loading CSS from: ' . $css_url);
        error_log('Weather Write - Plugin URL constant: ' . NEWSPAPER_AI_URL);
        error_log('Weather Write - Plugin directory: ' . plugin_dir_path(dirname(__FILE__)));
        
        wp_register_style(
            'newspaper-ai-weather-images',
            $css_url,
            array(),
            NEWSPAPER_AI_VERSION
        );
        
        // Add inline CSS for disabled buttons
        wp_add_inline_style('newspaper-ai-weather-images', '
            .weather-image-upload.disabled {
                opacity: 0.5;
                cursor: not-allowed;
            }
            .weather-image-preview {
                position: relative !important;
                min-height: 150px;
                background: #f8f9fa;
                border: 2px dashed #dee2e6;
                border-radius: 8px;
                display: flex;
                align-items: center;
                justify-content: center;
                overflow: visible;
            }
            .weather-image-preview img {
                max-width: 100%;
                height: auto;
                display: block;
            }
            .weather-image-placeholder {
                display: flex;
                flex-direction: column;
                align-items: center;
                justify-content: center;
                width: 100%;
                height: 100%;
                padding: 20px;
                text-align: center;
            }
            .weather-image-placeholder p {
                margin: 0 0 15px;
                font-size: 16px;
                font-weight: 500;
                color: #495057;
            }
            .weather-image-locked {
                position: absolute;
                top: 0;
                left: 0;
                right: 0;
                bottom: 0;
                background: rgba(73, 80, 87, 0.85);
                display: flex;
                align-items: center;
                justify-content: center;
                text-align: center;
                z-index: 10;
            }
            .weather-image-locked .lock-overlay {
                display: flex;
                flex-direction: column;
                align-items: center;
                justify-content: center;
                color: white;
                text-align: center;
                padding: 20px;
                width: 100%;
                height: 100%;
                background-color: rgba(0, 0, 0, 0.5);
            }
            .weather-image-locked .lock-overlay .dashicons {
                font-size: 48px;
                margin-bottom: 15px;
                color: white;
            }
            .weather-image-locked .lock-overlay p {
                margin: 0 0 15px;
                font-size: 24px;
                line-height: 1.4;
                color: white;
                font-weight: 700;
                text-shadow: 1px 1px 3px rgba(0, 0, 0, 0.7);
                text-transform: uppercase;
                letter-spacing: 1px;
            }
            .weather-image-locked .premium-badge {
                background-color: #f39c12;
                color: white;
                padding: 8px 15px;
                border-radius: 4px;
                font-weight: bold;
                font-size: 18px;
                margin-bottom: 15px;
                box-shadow: 0 2px 4px rgba(0, 0, 0, 0.2);
            }
            .upgrade-button {
                background: #228be6 !important;
                color: white !important;
                border: none !important;
                padding: 12px 24px !important;
                margin-top: 10px !important;
                text-decoration: none !important;
                border-radius: 6px !important;
                font-weight: 600 !important;
                font-size: 15px !important;
                transition: all 0.2s ease !important;
                box-shadow: 0 2px 4px rgba(0,0,0,0.1) !important;
                display: inline-block !important;
                line-height: 1.4 !important;
            }
            .upgrade-button:hover {
                background: #1971c2 !important;
                transform: translateY(-1px) !important;
                box-shadow: 0 4px 6px rgba(0,0,0,0.1) !important;
            }
        ');
        wp_enqueue_style('newspaper-ai-weather-images');

        // We're now using weather-admin.js for all media handling, so we don't need this script
        // The script is loaded in the main weather class with proper dependencies
        error_log('Weather Write - Using main weather-admin.js for media handling');

        $is_premium = $this->is_premium();
        error_log('Weather Write - Setting premium status: ' . ($is_premium ? 'true' : 'false'));
        wp_localize_script('newspaper-ai-weather-images', 'newspaperAIWeather', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'nonce' => wp_create_nonce('save_weather_image'),
            'isPremium' => $is_premium === true ? '1' : '0',
            'upgradeUrl' => NewspaperAI_Reintent_API::UPGRADE_URL,
        ));
    }

    /**
     * Save weather image URL and ID
     */
    public function save_weather_image($type, $variation, $image_id, $image_url) {
        if (empty($type) || empty($variation) || empty($image_id) || empty($image_url)) {
            return false;
        }

        // Allow all variations regardless of premium status
        $is_premium = $this->is_premium();
        error_log('Weather Write - Saving image for ' . $type . ' variation ' . $variation . ' (premium: ' . ($is_premium ? 'true' : 'false') . ')');
        
        // No longer blocking non-premium users from using variations > 1

        $weather_images = get_option('weather_write_images', array());
        $key = $type . '_' . $variation;
        $weather_images[$key] = array(
            'id' => $image_id,
            'url' => $image_url
        );
        return update_option('weather_write_images', $weather_images);
    }

    /**
     * Handle saving weather image via AJAX
     */
    public function handle_save_weather_image() {
        try {
            // Verify nonce
            if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'newspaper_ai_weather_images')) {
                throw new Exception('Invalid nonce');
            }

            // Verify user capabilities
            if (!current_user_can('manage_options')) {
                throw new Exception('Permission denied');
            }

            // Get and validate parameters
            $type = isset($_POST['type']) ? sanitize_text_field($_POST['type']) : '';
            $variation = isset($_POST['variation']) ? intval($_POST['variation']) : 0;
            $image_id = isset($_POST['image_id']) ? intval($_POST['image_id']) : 0;
            $image_url = isset($_POST['image_url']) ? esc_url_raw($_POST['image_url']) : '';

            if (empty($type)) {
                throw new Exception('Missing weather type');
            }
            if (empty($variation)) {
                throw new Exception('Missing variation number');
            }
            if (empty($image_id)) {
                throw new Exception('Missing image ID');
            }
            if (empty($image_url)) {
                throw new Exception('Missing image URL');
            }

            // Check premium status for variations > 1
            $is_premium = $this->is_premium();
            if (!$is_premium && $variation > 1) {
                throw new Exception('Premium account required to use multiple variations. Please upgrade to unlock this feature.');
            }

            // Save the image
            if (!$this->save_weather_image($type, $variation, $image_id, $image_url)) {
                throw new Exception('Failed to save image');
            }

            wp_send_json_success(array(
                'message' => 'Image saved successfully'
            ));

        } catch (Exception $e) {
            wp_send_json_error($e->getMessage());
        }
    }

    /**
     * Get weather image URL
     */
    public function get_weather_image_url($type, $variation) {
        // Normalize the type (replace spaces with underscores)
        $type = str_replace(' ', '_', strtolower(trim($type)));
        $variation = max(1, min(3, (int)$variation));
        
        error_log('Weather Write - Getting image URL for: ' . $type . '-' . $variation);
        
        // Try to get images from new option, fall back to old option if needed
        $weather_images = get_option('weather_write_images', get_option('newspaper_ai_weather_images', array()));
        
        // Try both underscore and hyphen formats
        $keys = [
            $type . '_' . $variation,  // Format used in the database (underscore)
            $type . '-' . $variation    // Format used in filenames (hyphen)
        ];
        
        foreach ($keys as $key) {
            error_log('Weather Write - Checking for key: ' . $key);
            
            if (isset($weather_images[$key])) {
                // Handle both old format (just ID) and new format (array with ID and URL)
                if (is_array($weather_images[$key])) {
                    error_log('Weather Write - Found image in new format: ' . $weather_images[$key]['url']);
                    return $weather_images[$key]['url'];
                } else {
                    // Old format - get URL from attachment ID
                    $url = wp_get_attachment_url($weather_images[$key]);
                    if ($url) {
                        error_log('Weather Write - Found image in old format: ' . $url);
                        return $url;
                    }
                }
            }
        }
        
        // Get default image from the installer
        $image_url = $this->installer->get_default_image_url($type, $variation);
        error_log('Weather Write - Loading default image: ' . $image_url);
        return $image_url;
    }

    /**
     * Get weather image ID
     */
    public function get_weather_image_id($condition) {
        error_log('Weather Write - Getting image ID for condition: ' . $condition);
        
        // Try to get images from new option, fall back to old option if needed
        $weather_images = get_option('weather_write_images', get_option('newspaper_ai_weather_images', array()));
        
        // Normalize the condition
        $condition = strtolower(trim($condition));
        $is_premium = $this->is_premium();
        
        error_log('Weather Write - Normalized condition: ' . $condition);
        
        // Map conditions to image types - MUST match the types defined in the UI template
        $condition_map = array(
            // Sunny variations
            'sunny' => 'sunny',
            'mostly sunny' => 'sunny',
            'clear' => 'clear',
            'hot' => 'hot',
            'fair' => 'sunny',
            'sunshine' => 'sunny',
            'sunrise' => 'sunrise',
            'sunset' => 'sunset',
            
            // Partly cloudy variations
            'partly sunny' => 'partly_sunny',
            'partly cloudy' => 'partly_cloudy',
            'scattered clouds' => 'partly_cloudy',
            'few clouds' => 'partly_cloudy',
            'broken clouds' => 'partly_cloudy',
            
            // Cloudy variations
            'mostly cloudy' => 'mostly_cloudy',
            'cloudy' => 'cloudy',  // Map to cloudy instead of overcast
            'overcast' => 'overcast',
            'windy' => 'windy',
            'breezy' => 'windy',
            
            // Rain variations
            'rain' => 'rain',
            'showers' => 'showers',
            'light rain' => 'rain_light',
            'heavy rain' => 'rain_heavy',
            'scattered showers' => 'showers',
            'drizzle' => 'rain_light',
            'chance of rain' => 'rain_light',
            
            // Storm variations
            'thunderstorm' => 'thunderstorm',
            'severe thunderstorm' => 'thunderstorm_severe',
            'scattered thunderstorms' => 'thunderstorm',
            'storm' => 'storm',
            'lightning' => 'lightning',
            'thunder' => 'thunderstorm',
            
            // Snow variations
            'snow' => 'snow',
            'light snow' => 'snow_light',
            'heavy snow' => 'snow_heavy',
            'winter weather' => 'snow_light',
            'sleet' => 'sleet',
            'freezing rain' => 'sleet',
            'wintry mix' => 'sleet',
            'cold' => 'cold',
            'flurries' => 'snow_light',
            'blizzard' => 'snow_heavy',
            'ice' => 'sleet',
            'frost' => 'cold',
            
            // Fog variations
            'fog' => 'fog',
            'haze' => 'haze',
            'mist' => 'mist',
            'smoky' => 'fog',
            'poor visibility' => 'fog'
        );
        
        // Get the image type from the condition
        $image_type = null;
        foreach ($condition_map as $search => $type) {
            if (strpos($condition, $search) !== false) {
                $image_type = $type;
                error_log('Weather Write - Found match: condition "' . $condition . '" matches "' . $search . '" => ' . $type);
                break;
            }
        }
        
        // Default to overcast if no match found
        if (!$image_type) {
            $image_type = 'overcast'; // Changed from cloudy to overcast to match UI
            error_log('Weather Write - No match found for condition: "' . $condition . '", defaulting to overcast');
        }
        
        error_log('Weather Write - Final image type: ' . $image_type);
        
        // Get available variations based on account status
        $variations = array();
        $max_variations = $is_premium ? 3 : 1;
        
        // Store the image type for default image selection if needed
        $default_image_type = $image_type;
        
        // Always try variation 1 first
        // Try both formats for the key - with underscore and with hyphen
        $key_formats = array(
            $image_type . '_1',  // Format used in the database (underscore)
            $image_type . '-1'   // Format used in filenames (hyphen)
        );
        
        $found = false;
        foreach ($key_formats as $key) {
            error_log('Weather Write - Looking for image with key: ' . $key);
            
            // Check both formats - new format (array with id/url) and old format (just ID)
            if (isset($weather_images[$key])) {
                if (is_array($weather_images[$key]) && isset($weather_images[$key]['id'])) {
                    $variations[] = $weather_images[$key]['id'];
                    error_log('Weather Write - Found image ID in new format: ' . $weather_images[$key]['id']);
                    $found = true;
                } elseif (!is_array($weather_images[$key])) {
                    $variations[] = $weather_images[$key];
                    error_log('Weather Write - Found image ID in old format: ' . $weather_images[$key]);
                    $found = true;
                }
            }
        }
        
        if (!$found) {
            error_log('Weather Write - No image found for any key format');
        }
        
        // If premium, try other variations
        if ($is_premium) {
            for ($i = 2; $i <= $max_variations; $i++) {
                // Try both formats for the key
                $key_formats = array(
                    $image_type . '_' . $i,  // Format used in the database (underscore)
                    $image_type . '-' . $i   // Format used in filenames (hyphen)
                );
                
                foreach ($key_formats as $key) {
                    error_log('Weather Write - Premium user, also checking: ' . $key);
                    
                    if (isset($weather_images[$key])) {
                        if (is_array($weather_images[$key]) && isset($weather_images[$key]['id'])) {
                            $variations[] = $weather_images[$key]['id'];
                            error_log('Weather Write - Found additional image ID in new format: ' . $weather_images[$key]['id']);
                        } elseif (!is_array($weather_images[$key])) {
                            $variations[] = $weather_images[$key];
                            error_log('Weather Write - Found additional image ID in old format: ' . $weather_images[$key]);
                        }
                    }
                }
            }
        }
        
        // Return a random variation if we have any custom images
        if (!empty($variations)) {
            $selected_id = $variations[array_rand($variations)];
            error_log('Weather Write - Selected custom image ID: ' . $selected_id);
            return $selected_id;
        } else {
            // No custom images found, use default images with random variation
            error_log('Weather Write - No custom images found for condition: ' . $condition . ', using default image');
            
            // Generate a random variation number (1-3)
            $random_variation = mt_rand(1, $is_premium ? 3 : 1);
            
            // Store information about the selected default image
            update_option('weather_write_last_default_image', array(
                'type' => $default_image_type,
                'variation' => $random_variation,
                'condition' => $condition
            ));
            
            error_log('Weather Write - Selected default image: ' . $default_image_type . '-' . $random_variation);
            return null; // Return null to indicate that a default image should be used
        }
    }
    
    /**
     * Get and attach a default image to a post
     * 
     * @param int $post_id The post ID to attach the image to
     * @param string $condition The weather condition
     * @return int|false The attachment ID on success, false on failure
     */
    public function attach_default_image($post_id, $condition) {
        error_log('Weather Write - Starting to attach default image for condition: ' . $condition . ' to post ID: ' . $post_id);
        
        // Require image handling functions
        if (!function_exists('wp_generate_attachment_metadata')) {
            require_once(ABSPATH . 'wp-admin/includes/image.php');
            require_once(ABSPATH . 'wp-admin/includes/file.php');
            require_once(ABSPATH . 'wp-admin/includes/media.php');
        }
        
        // Determine type based on condition
        $type = $this->get_image_type_from_condition($condition);
        
        // Check premium status but always allow variation
        $is_premium = $this->is_premium();
        // Allow 3 variations regardless of premium status
        $max_variation = 3;
        
        // Add entropy to the random number generator
        mt_srand((double) microtime() * 1000000 + getmypid());
        $variation = mt_rand(1, $max_variation);
        
        error_log('Weather Write - Randomized image type: ' . $type . ', variation: ' . $variation . ' (max: ' . $max_variation . ')');
        
        // Debug info
        error_log('Weather Write - Debug: Premium status: ' . ($is_premium ? 'true' : 'false') . ', but using max variation: ' . $max_variation);
        
        // Get the default image path from the local plugin directory
        $plugin_dir = plugin_dir_path(dirname(dirname(__FILE__)));
        
        // Map the type to the base image type
        $type_map = [
            'sunny' => 'sunny',
            'clear' => 'sunny',
            'partly_cloudy' => 'cloudy',
            'mostly_cloudy' => 'cloudy',
            'overcast' => 'overcast',
            'rain_light' => 'rain',
            'rain_heavy' => 'rain',
            'showers' => 'rain',
            'thunderstorm' => 'lightning',
            'thunderstorm_severe' => 'lightning',
            'snow_light' => 'snow',
            'snow_heavy' => 'snow',
            'fog' => 'fog',
            'mist' => 'fog'
        ];
        
        // Get the mapped type or use the original if not found
        $mapped_type = isset($type_map[$type]) ? $type_map[$type] : $type;
        
        // Ensure variation is within range (1-3)
        $variation = max(1, min(3, $variation));
        
        // Format the filename using our local image naming convention
        $filename = $mapped_type . '-' . $variation . '.jpg';
        
        // Check if we already have this image in the media library
        $existing_attachment_id = $this->get_existing_weather_attachment($mapped_type, $variation);
        
        if ($existing_attachment_id) {
            error_log('Weather Write - Found existing attachment ID: ' . $existing_attachment_id . ' for ' . $mapped_type . ' variation ' . $variation);
            
            // Set as featured image
            set_post_thumbnail($post_id, $existing_attachment_id);
            update_post_meta($post_id, '_thumbnail_id', $existing_attachment_id);
            
            return $existing_attachment_id;
        }
        
        $image_path = $plugin_dir . 'assets/images/weather/' . $filename;
        error_log('Weather Write - Looking for image at: ' . $image_path);
        
        // Also check in the uploads directory
        $upload_dir = wp_upload_dir();
        $uploads_path = $upload_dir['basedir'] . '/weather-images/' . $filename;
        
        if (file_exists($uploads_path)) {
            error_log('Weather Write - Found image in uploads directory: ' . $uploads_path);
            $image_path = $uploads_path;
        }
        
        // Check if the image exists locally
        if (!file_exists($image_path)) {
            error_log('Weather Write - Image not found locally, attempting to use fallback image');
            
            // Make sure installer is initialized
            if (!$this->installer) {
                error_log('Weather Write - Error: Installer not initialized');
                require_once dirname(__FILE__) . '/class-weather-images-installer.php';
                $this->installer = NewspaperAI_Weather_Images_Installer::get_instance();
            }
            
            // Try to get a default image URL
            if (method_exists($this->installer, 'get_default_image_url')) {
                $default_url = $this->installer->get_default_image_url($type, $variation);
                error_log('Weather Write - Default image URL: ' . ($default_url ? $default_url : 'not available'));
                
                // Extract the filename from the URL
                $filename = basename($default_url);
                
                // Check if the default image exists in the plugin directory
                $plugin_image_dir = $plugin_dir . 'assets/images/weather/';
                $plugin_image_path = $plugin_image_dir . $filename;
                
                if (file_exists($plugin_image_path)) {
                    error_log('Weather Write - Found default image in plugin directory: ' . $plugin_image_path);
                    $image_path = $plugin_image_path;
                } else {
                    // Try to use a different variation
                    for ($i = 1; $i <= 3; $i++) {
                        if ($i == $variation) continue;
                        $alt_filename = $type . '-' . $i . '.jpg';
                        $alt_path = $plugin_image_dir . $alt_filename;
                        
                        if (file_exists($alt_path)) {
                            error_log('Weather Write - Using alternative variation: ' . $i);
                            $image_path = $alt_path;
                            break;
                        }
                    }
                    
                    // If still no image, try a generic fallback
                    if (!file_exists($image_path)) {
                        $fallback_path = $plugin_image_dir . 'cloudy-1.jpg';
                        if (file_exists($fallback_path)) {
                            error_log('Weather Write - Using generic fallback image: cloudy-1.jpg');
                            $image_path = $fallback_path;
                        }
                    }
                }
            } else {
                error_log('Weather Write - Error: get_default_image_url method not found in installer');
            }
            
            // If we still don't have an image, use the fallback image generator
            if (!file_exists($image_path)) {
                error_log('Weather Write - No default image found, falling back to generated placeholder image');
                
                // Use fallback image system instead
                $attachment_id = $this->fallback->add_fallback_image_to_media_library($post_id, $type, $variation);
                
                if ($attachment_id) {
                    // Set as featured image
                    set_post_thumbnail($post_id, $attachment_id);
                    update_post_meta($post_id, '_thumbnail_id', $attachment_id);
                    
                    error_log('Weather Write - Successfully set fallback image as featured image with ID: ' . $attachment_id);
                    return $attachment_id;
                }
                
                return false;
            }
            
            // Create a dedicated directory for weather images in uploads if needed
            $upload_dir = wp_upload_dir();
            $weather_images_dir = $upload_dir['basedir'] . '/weather-images/';
            wp_mkdir_p($weather_images_dir);
            
            // Copy the image to the uploads directory for future use
            $upload_filename = $type . '-' . $variation . '.jpg';
            $target_path = $weather_images_dir . $upload_filename;
            
            if (!file_exists($target_path)) {
                if (copy($image_path, $target_path)) {
                    error_log('Weather Write - Copied default image to uploads directory: ' . $target_path);
                }
            }
            @copy($target_path, $plugin_image_path);
        }
        
        error_log('Weather Write - Using image at: ' . $image_path);
        
        // Verify the post exists
        $post = get_post($post_id);
        if (!$post) {
            error_log('Weather Write - Post does not exist: ' . $post_id);
            return false;
        }
        
        // Set up the upload directory
        $upload_dir = wp_upload_dir();
        $filename = basename($image_path);
        $unique_filename = wp_unique_filename($upload_dir['path'], $filename);
        $target_path = $upload_dir['path'] . '/' . $unique_filename;
        
        // Copy the file to the uploads directory (if it's not already there)
        if ($image_path !== $target_path) {
            if (!copy($image_path, $target_path)) {
                error_log('Weather Write - Failed to copy image to uploads directory');
                return false;
            }
        }
        
        // Get file type
        $filetype = wp_check_filetype($unique_filename, null);
        if (empty($filetype['type'])) {
            error_log('Weather Write - Invalid file type');
            return false;
        }
        
        // Create the attachment
        $attachment = array(
            'guid'           => $upload_dir['url'] . '/' . $unique_filename,
            'post_mime_type' => $filetype['type'],
            'post_title'     => sanitize_text_field($condition . ' weather'),
            'post_content'   => '',
            'post_status'    => 'inherit'
        );
        
        // Insert the attachment
        $attachment_id = wp_insert_attachment($attachment, $target_path, $post_id);
        
        if (is_wp_error($attachment_id)) {
            error_log('Weather Write - Failed to insert attachment: ' . $attachment_id->get_error_message());
            return false;
        }
        
        // Generate attachment metadata
        $attachment_data = wp_generate_attachment_metadata($attachment_id, $target_path);
        wp_update_attachment_metadata($attachment_id, $attachment_data);
        
        // Set as featured image using multiple methods for redundancy
        error_log('Weather Write - Attempting to set featured image using multiple methods');
        
        // Method 1: WordPress standard function
        $result = set_post_thumbnail($post_id, $attachment_id);
        error_log('Weather Write - Method 1 (set_post_thumbnail) result: ' . ($result ? 'success' : 'failed'));
        
        // Method 2: Direct update of post meta
        update_post_meta($post_id, '_thumbnail_id', $attachment_id);
        error_log('Weather Write - Method 2 (update_post_meta) applied');
        
        // Method 3: Use wp_update_post with meta_input
        $post_data = array(
            'ID' => $post_id,
            'meta_input' => array(
                '_thumbnail_id' => $attachment_id
            )
        );
        $update_result = wp_update_post($post_data);
        error_log('Weather Write - Method 3 (wp_update_post) result: ' . (is_wp_error($update_result) ? $update_result->get_error_message() : 'success'));
        
        // Method 4: Direct database update as last resort
        global $wpdb;
        $wpdb->update(
            $wpdb->postmeta,
            array('meta_value' => $attachment_id),
            array('post_id' => $post_id, 'meta_key' => '_thumbnail_id')
        );
        error_log('Weather Write - Method 4 (direct database update) applied');
        
        // Final verification
        $final_thumbnail_id = get_post_thumbnail_id($post_id);
        error_log('Weather Write - Final thumbnail ID after all attempts: ' . $final_thumbnail_id);
        
        // Clear any caches that might be preventing the featured image from showing
        clean_post_cache($post_id);
        error_log('Weather Write - Post cache cleared for post ID: ' . $post_id);
        
        // Store this image ID for future use
        update_option('weather_write_last_default_image_id', $attachment_id);
        
        return $attachment_id;
    }
    
    /**
     * Get an existing weather image attachment from the media library
     * 
     * @param string $type The weather image type
     * @param int $variation The variation number
     * @return int|false The attachment ID if found, false otherwise
     */
    private function get_existing_weather_attachment($type, $variation) {
        // Format the filename using our local image naming convention
        $filename = $type . '-' . $variation . '.jpg';
        
        // Search for attachments with this filename
        $args = array(
            'post_type' => 'attachment',
            'post_status' => 'inherit',
            'posts_per_page' => 1,
            'meta_query' => array(
                array(
                    'key' => '_wp_attached_file',
                    'value' => $filename,
                    'compare' => 'LIKE'
                )
            )
        );
        
        $query = new WP_Query($args);
        
        if ($query->have_posts()) {
            $post = $query->posts[0];
            return $post->ID;
        }
        
        return false;
    }
    
    /**
     * Get the image type from a weather condition
     * 
     * @param string $condition The weather condition
     * @return string The image type
     */
    private function get_image_type_from_condition($condition) {
        $condition = strtolower($condition);
        
        // Map conditions to image types - MUST match the types defined in the UI template
        $condition_map = array(
            // Sunny variations
            'sunny' => 'sunny',
            'mostly sunny' => 'sunny',
            'clear' => 'clear',
            
            // Partly cloudy variations
            'partly sunny' => 'partly_cloudy',
            'partly cloudy' => 'partly_cloudy',
            'scattered clouds' => 'partly_cloudy',
            
            // Cloudy variations
            'mostly cloudy' => 'mostly_cloudy',
            'cloudy' => 'overcast',  // Map to overcast as defined in UI
            'overcast' => 'overcast',
            
            // Rain variations
            'rain' => 'rain_light', // Map to specific rain type
            'showers' => 'showers',
            'light rain' => 'rain_light',
            'heavy rain' => 'rain_heavy',
            'scattered showers' => 'showers',
            
            // Storm variations
            'thunderstorm' => 'thunderstorm', // Fixed to match UI
            'severe thunderstorm' => 'thunderstorm_severe', // Fixed to match UI
            'scattered thunderstorms' => 'thunderstorm',
            'storm' => 'thunderstorm',
            
            // Snow variations
            'snow' => 'snow_light', // Map to specific snow type
            'light snow' => 'snow_light',
            'heavy snow' => 'snow_heavy',
            'winter weather' => 'snow_light',
            
            // Fog variations
            'fog' => 'fog',
            'haze' => 'fog',
            'mist' => 'mist'
        );
        
        // Get the image type from the condition
        foreach ($condition_map as $search => $type) {
            if (strpos($condition, $search) !== false) {
                return $type;
            }
        }
        
        // Default to overcast if no match found
        return 'overcast';
    }
}
