<?php
/**
 * Weather Reporter Class
 * Handles weather report generation and scheduling
 */

if (!defined('ABSPATH')) {
    exit;
}

class NewspaperAI_Weather_Reporter {
    private $noaa_api;
    private $weather_data;
    private $reintent_api;
    private $weather_images;

    private static $instance = null;

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        // Initialize dependencies
        $this->noaa_api = NewspaperAI_NOAA_API::get_instance();

        // Enqueue styles and scripts
        add_action('wp_enqueue_scripts', array($this, 'enqueue_styles'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_styles'));
        add_action('wp_enqueue_scripts', array($this, 'enqueue_font_awesome'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_font_awesome'));
        add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'), 20);
        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'), 20);
        
        // Initialize Reintent API
        require_once plugin_dir_path(__FILE__) . 'class-reintent-api.php';
        $this->reintent_api = NewspaperAI_Reintent_API::get_instance();
        if (!$this->reintent_api->is_configured()) {
            error_log('Weather Reporter: Reintent API key not configured');
            $this->reintent_api = null;
        }
        
        $this->weather_images = NewspaperAI_Weather_Images::get_instance();
    }

    public function enqueue_styles() {
        wp_enqueue_style(
            'weather-styles-v2',
            plugin_dir_url(__FILE__) . 'assets/css/weather-styles-v2.css',
            array(),
            filemtime(plugin_dir_path(__FILE__) . 'assets/css/weather-styles-v2.css')
        );
    }

    public function enqueue_scripts() {
        // Chart.js is now handled by the main weather.php file
    }

    public function enqueue_font_awesome() {
        wp_enqueue_style(
            'font-awesome',
            'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.4/css/all.min.css',
            array(),
            '5.15.4'
        );
    }

    public function enqueue_map_scripts() {
        // Enqueue Leaflet CSS
        wp_enqueue_style(
            'leaflet-css',
            'https://unpkg.com/leaflet@1.9.4/dist/leaflet.css',
            array(),
            '1.9.4'
        );

        // Enqueue Leaflet JS
        wp_enqueue_script(
            'leaflet',
            'https://unpkg.com/leaflet@1.9.4/dist/leaflet.js',
            array('jquery'),
            '1.9.4',
            true
        );
    }

    public function generate_report($weather_data, $is_test = false) {
        $this->weather_data = $weather_data;
        try {
            // Extract data
            $forecast = $weather_data['forecast'];
            $location = $weather_data['location'];
            $city = $location['city'];
            $state = $location['state'];

            // Get today's forecast
            $today = $forecast['today'];
            $today['city'] = $city;
            $today['state'] = $state;

            // Get hourly forecast
            $hourly = $this->noaa_api->get_hourly_forecast($location['lat'], $location['lng']);

            // Generate the title
            $title = $this->generate_title($today);

            // Generate report sections
            $current_conditions = $this->generate_current_conditions($today);
            
            // Make sure scripts are loaded
            wp_enqueue_script('chartjs');
            wp_enqueue_script('weather-script');

            // Debug logging
            error_log('Weather Reporter - Hourly data: ' . print_r($hourly, true));
            error_log('Weather Reporter - Today data: ' . print_r($today, true));
            
            // Generate hourly section after scripts are enqueued
            $hourly_section = $this->generate_hourly_section($hourly);
            $forecast_section = $this->generate_forecast_section($forecast['weekly']);

            // Build the full report HTML
            // Format the current date
            $current_date = current_time('F j, Y');  // e.g., "February 26, 2025"

            // Extract weather data
            $current_temp = isset($today['temperature']) ? round($today['temperature']) : 'N/A';
            $high_temp = isset($today['high']) ? round($today['high']) : 'N/A';
            $low_temp = isset($today['low']) ? round($today['low']) : 'N/A';
            $conditions = isset($today['shortForecast']) ? $today['shortForecast'] : 'N/A';
            $wind_speed = isset($today['windSpeed']) ? $today['windSpeed'] : 'N/A';
            $wind_dir = isset($today['windDirection']) ? $today['windDirection'] : '';
            $wind = $wind_speed . ($wind_dir ? ' ' . $wind_dir : '');
            $humidity = isset($today['humidity']) ? $today['humidity'] : 'N/A';
            $weather_details = sprintf(
                '<div class="weather-overview">
                    <div class="weather-header">
                        <div class="location">%s, %s</div>
                        <div class="current-date">%s</div>
                    </div>
                    <div class="weather-metrics">
                        <div class="metric-box temperature">
                            <div class="current-temp">%s°</div>
                            <div class="temp-range">
                                <span class="high">H:%s°</span>
                                <span class="low">L:%s°</span>
                            </div>
                        </div>
                        <div class="metric-box">
                            <i class="fas fa-wind"></i>
                            <div class="metric-content">
                                <div class="metric-label">Wind</div>
                                <div class="metric-value">%s</div>
                            </div>
                        </div>
                        <div class="metric-box">
                            <i class="fas fa-tint"></i>
                            <div class="metric-content">
                                <div class="metric-label">Humidity</div>
                                <div class="metric-value">%s%%</div>
                            </div>
                        </div>
                        <div class="metric-box">
                            <i class="fas fa-cloud"></i>
                            <div class="metric-content">
                                <div class="metric-label">Conditions</div>
                                <div class="metric-value">%s</div>
                            </div>
                        </div>
                    </div>
                </div>',
                $city,
                strtoupper($state),
                $current_date,
                $current_temp,
                $high_temp,
                $low_temp,
                $wind,
                $humidity,
                $conditions
            );

            // Get the weather narrative without the title
            $narrative = '';
            if ($this->reintent_api) {
                $prompt = $this->generate_weather_prompt($today);
                try {
                    $narrative = $this->reintent_api->generate_text($prompt);
                    // Remove the title if it exists
                    $narrative = preg_replace('/\*\*Nashville Weather Report:[^*]*\*\*\n?/', '', $narrative);
                } catch (Exception $e) {
                    error_log('Weather Reporter - Reintent API error: ' . $e->getMessage());
                    $narrative = '';
                }
            }

            if (empty($narrative)) {
                // Use fallback narrative if Reintent failed or isn't available
                $narrative = $this->generate_fallback_narrative($today);
            }

            // Split narrative into paragraphs and format each one
            $paragraphs = explode("\n\n", $narrative);
            $formatted_paragraphs = array_map(function($p) {
                return sprintf('<p>%s</p>', wp_kses_post(trim($p)));
            }, $paragraphs);
            
            $narrative_section = sprintf(
                '<div class="weather-narrative">%s</div>',
                implode("\n", $formatted_paragraphs)
            );

            $report = sprintf(
                '<div class="weather-report">
                    <div class="weather-content">
                        %s
                        %s
                        %s
                    </div>
                </div><!-- weather_title:%s -->',
                $narrative_section,
                $weather_details,
                $forecast_section,
                esc_attr($title)
            );

            return $report;

        } catch (Exception $e) {
            error_log('Weather report generation error: ' . $e->getMessage());
            return false;
        }
    }

    private function generate_weather_prompt($today) {
        $city = isset($today['city']) ? $today['city'] : 'your location';
        $state = isset($today['state']) ? $today['state'] : '';
        $location = trim("{$city}, {$state}", ", ");
    
        $temp = isset($today['temperature']) ? "{$today['temperature']}°F" : 'an unknown temperature';
        $high = isset($today['high']) ? "{$today['high']}°F" : 'an unknown high';
        $low = isset($today['low']) ? "{$today['low']}°F" : 'an unknown low';
        $conditions = isset($today['shortForecast']) ? $today['shortForecast'] : 'unknown conditions';
        $wind = isset($today['wind']) ? $today['wind'] : 'calm winds';
        // Get humidity from current conditions if available, otherwise use forecast
        $humidity = isset($today['raw_data']['relativeHumidity']['value']) 
            ? round($today['raw_data']['relativeHumidity']['value']) . '%' 
            : (isset($today['humidity']) ? "{$today['humidity']}%" : 'an unknown humidity level');
    
        // Get current date and determine season
        $current_date = current_time('F j, Y');
        $month = (int)current_time('n');
        $season = '';
        if ($month >= 3 && $month <= 5) {
            $season = 'spring';
        } elseif ($month >= 6 && $month <= 8) {
            $season = 'summer';
        } elseif ($month >= 9 && $month <= 11) {
            $season = 'fall';
        } else {
            $season = 'winter';
        }
    
        $current_time = current_time('g:i A');
    
        // Get time context and daylight information
        $current_timestamp = current_time('timestamp');
        $sunset_timestamp = isset($today['sunset']) ? $today['sunset'] : null;
        $sunrise_timestamp = isset($today['sunrise']) ? $today['sunrise'] : null;
        
        $sunrise_time = $sunrise_timestamp ? date('g:i A', $sunrise_timestamp) : 'N/A';
        $sunset_time = $sunset_timestamp ? date('g:i A', $sunset_timestamp) : 'N/A';
        
        // Determine if it's day or night and specific time context
        $is_daytime = ($current_timestamp > $sunrise_timestamp && $current_timestamp < $sunset_timestamp);
        $time_context = '';
        $daylight_context = '';
        
        if ($sunset_timestamp && $current_timestamp > $sunset_timestamp) {
            $hours_after_sunset = round(($current_timestamp - $sunset_timestamp) / 3600);
            $time_context = "The sun set about {$hours_after_sunset} hours ago at {$sunset_time}";
            $daylight_context = "It's fully dark now, with sunrise tomorrow at {$sunrise_time}";
        } elseif ($sunset_timestamp && $current_timestamp <= $sunset_timestamp) {
            $hours_until_sunset = round(($sunset_timestamp - $current_timestamp) / 3600);
            if ($hours_until_sunset <= 1) {
                $time_context = "The sun will set soon at {$sunset_time}";
                $daylight_context = "The sky is beginning to take on evening colors as sunset approaches";
            } else {
                $time_context = "Sunset will be at {$sunset_time}";
                $daylight_context = "There are still {$hours_until_sunset} hours of daylight remaining";
            }
        }

        return "Create an engaging weather report for {$location} that captures both current conditions and expected changes. 
        
        Current Context:
        - Date: {$current_date}
        - Season: Early {$season} (March marks the beginning of {$season})
        - Time: {$current_time}
        - Daylight: {$daylight_context}
        - {$time_context}
        - Temperature: {$temp}
        - Conditions: {$conditions}
        - Wind: {$wind}
        - Humidity: {$humidity}
        
      Structure the report in three sections:
        
        <h2>Current Weather</h2>
        - Start with a **clear, straightforward description** of the current weather. 
        - Describe the **temperature, humidity, and wind**, emphasizing how it actually feels outside.
        - Mention the **sky conditions and overall atmosphere** in a concise way.
        - Example: 'It's currently {$current_time} in {$location}, with {$temp} temperatures and {$conditions}.'
        - Avoid poetic or overly artistic phrasing—keep it **practical and to the point**.
        
        <h2>[Use one of these based on conditions:
          - If temperature changing significantly: 'Temperature Shift Ahead'
          - If wind is the main factor: 'Wind Changes Coming'
          - If precipitation is expected: 'Rain/Snow Expected'
          - If conditions are stable: 'Steady Conditions Ahead'
          - If multiple changes: 'Weather Changes Coming']</h2>
        - Highlight any **significant shifts** happening now or expected soon.
        - Explain how these changes will **affect how it feels outside**, avoiding unnecessary metaphors.
        - Focus on **practical impacts**, such as wind making it feel colder or humidity making it feel heavier.
        - Example: 'Temperatures are dropping fast, and increasing winds will make the evening feel brisk.'
        
        <h2>[Use one of these based on timing:
          - If reporting in morning: 'Today's Outlook'
          - If reporting in afternoon: 'Tonight's Forecast'
          - If reporting in evening: 'Overnight and Tomorrow'
          - If severe weather expected: 'Important Weather Alerts']</h2>
        - Describe how conditions will evolve over the next 24 hours.
        - Focus on **the practical experience** of being outside, rather than poetic descriptions.
        - Help readers **anticipate what to expect** (e.g., morning fog, overnight temperature drops).
        - Keep language **clear, natural, and conversational**.
        
        Guidelines:
        - Use HTML Markup that will be appropriate for the body of a WordPress post. 
        - Use **sensory details sparingly**—only when they add clarity.
        - Make this read like a jack kerouac passage and use visual metaphors to describe the weather. This is extremely important that it reads like something he would write. 
        - Keep descriptions **grounded in real, practical experience**.
        - Maintain a **conversational** and **engaging** tone.
        - Avoid using **Concluson** in the title of a section.
        - Avoid **overly technical terms** unless absolutely necessary.";
}
    

    private function generate_fallback_narrative($today) {
        $current_time = current_time('g:i A');
        $city = isset($today['city']) ? $today['city'] : '';
        $state = isset($today['state']) ? $today['state'] : '';
        $current = isset($today['temperature']) ? $today['temperature'] : 'N/A';
        $conditions = isset($today['shortForecast']) ? strtolower($today['shortForecast']) : 'N/A';
        $wind_speed = isset($today['windSpeed']) ? $today['windSpeed'] : 'N/A';
        $wind_dir = isset($today['windDirection']) ? $today['windDirection'] : 'N/A';
        $detailed = isset($today['detailedForecast']) ? $today['detailedForecast'] : '';

        // Create a more structured fallback narrative
        $paragraph1 = sprintf(
            "As of %s in %s, %s, the weather is %s with a temperature of %s°F. %s winds are coming from the %s.",
            $current_time, $city, $state, $conditions, $current, $wind_speed, $wind_dir
        );

        $paragraph2 = "Based on these conditions, residents should dress appropriately for the temperature and current weather conditions.";

        $paragraph3 = $detailed;

        return $paragraph1 . "\n\n" . $paragraph2 . "\n\n" . $paragraph3;
    }

    private function generate_current_conditions($today) {
        if (!$today) return '';

        error_log('Generating current conditions with data: ' . print_r($today, true));

        // Get current time
        $current_time = current_time('g:i A');
        
        // Format the temperatures
        $high = isset($today['high']) ? $today['high'] : null;
        $low = isset($today['low']) ? $today['low'] : null;
        $current = isset($today['temperature']) ? $today['temperature'] : null;

        // Get location info
        $city = isset($this->weather_data['location']['city']) ? $this->weather_data['location']['city'] : '';
        $state = isset($this->weather_data['location']['state']) ? $this->weather_data['location']['state'] : '';

        error_log(sprintf(
            'Temperature data - Current: %s, High: %s, Low: %s',
            $current,
            $high,
            $low
        ));

        // Build the weather report content
        $current_conditions = sprintf(
            '<div class="weather-header">
                <div class="current-temp">%s°</div>
                <div class="temp-range">
                    <div>High %s°</div>
                    <div>Low %s°</div>
                </div>
            </div>',
            $current,
            $high,
            $low
        );

        // Build the weather details table
        $weather_details = sprintf(
            '<table class="weather-details">
                <tbody>
                    <tr>
                        <td>Humidity</td>
                        <td>%s</td>
                    </tr>
                    <tr>
                        <td>Wind</td>
                        <td>%s</td>
                    </tr>
                    <tr>
                        <td>Pressure</td>
                        <td>%s</td>
                    </tr>
                    <tr>
                        <td>Visibility</td>
                        <td>%s</td>
                    </tr>
                    <tr>
                        <td>Dew Point</td>
                        <td>%s</td>
                    </tr>
                    <tr>
                        <td>Cloud Cover</td>
                        <td>%s</td>
                    </tr>
                    <tr>
                        <td>Sunrise</td>
                        <td>%s</td>
                    </tr>
                    <tr>
                        <td>Sunset</td>
                        <td>%s</td>
                    </tr>
                </tbody>
            </table>',
            (isset($today['humidity']) ? $today['humidity'] : (isset($today['raw_data']['relativeHumidity']['value']) ? round($today['raw_data']['relativeHumidity']['value']) : 'N/A')) . '%',
            isset($today['wind_speed']) && isset($today['wind_direction']) ? 
                sprintf('%s mph %s', $today['wind_speed'], $today['wind_direction']) : 'N/A',
            isset($today['pressure']) ? number_format($today['pressure'], 2) . ' in' : 'N/A',
            isset($today['visibility']) ? $today['visibility'] . ' mi' : 'N/A',
            isset($today['dew_point']) ? $today['dew_point'] . '°' : 'N/A',
            isset($today['cloud_cover']) ? $today['cloud_cover'] . '%' : 'N/A',
            isset($today['sunrise']) ? date('g:i A', $today['sunrise']) : 'N/A',
            isset($today['sunset']) ? date('g:i A', $today['sunset']) : 'N/A'
        );

        // Get weather icon
        $weather_icon = $this->get_weather_icon($today['shortForecast']);

        // Format temperatures for display
        $temp_html = '';
        if ($current !== null) {
            $temp_html .= sprintf('<div class="current-temp">%d°</div>', $current);
        }
        if ($high !== null || $low !== null) {
            $temp_html .= '<div class="high-low">';
            if ($high !== null) {
                $temp_html .= sprintf('<span class="high">High: %d°</span>', $high);
            }
            if ($high !== null && $low !== null) {
                $temp_html .= ' | ';
            }
            if ($low !== null) {
                $temp_html .= sprintf('<span class="low">Low: %d°</span>', $low);
            }
            $temp_html .= '</div>';
        }

        // Debug raw data
        error_log('Raw weather data: ' . print_r($today['raw_data'], true));

        // Get additional metrics from raw data
        $raw_data = isset($today['raw_data']) ? $today['raw_data'] : array();
        
        // Calculate feels like temperature (using current temp as fallback)
        $feels_like = $current; // Default to current temperature
        if (isset($raw_data['heatIndex']) && isset($raw_data['heatIndex']['value']) && $raw_data['heatIndex']['value'] !== null) {
            $feels_like = $this->noaa_api->celsius_to_fahrenheit($raw_data['heatIndex']['value']);
        } elseif (isset($raw_data['windChill']) && isset($raw_data['windChill']['value']) && $raw_data['windChill']['value'] !== null) {
            $feels_like = $this->noaa_api->celsius_to_fahrenheit($raw_data['windChill']['value']);
        }
        $feels_like = round($feels_like);
        
        // Get additional weather metrics with more detailed error logging
        $humidity = null;
        if (isset($raw_data['relativeHumidity']) && isset($raw_data['relativeHumidity']['value'])) {
            $humidity = round($raw_data['relativeHumidity']['value']);
            error_log('Humidity value found: ' . $humidity);
        } else {
            error_log('No humidity data found in raw data');
        }

        $pressure = null;
        if (isset($raw_data['barometricPressure']) && isset($raw_data['barometricPressure']['value'])) {
            $pressure = round($raw_data['barometricPressure']['value'] * 0.0295301, 2); // Convert to inches
            error_log('Pressure value found: ' . $pressure);
        } else {
            error_log('No pressure data found in raw data');
        }

        $visibility = null;
        if (isset($raw_data['visibility']) && isset($raw_data['visibility']['value'])) {
            $visibility = round($raw_data['visibility']['value'] * 0.000621371, 1); // Convert to miles
            error_log('Visibility value found: ' . $visibility);
        } else {
            error_log('No visibility data found in raw data');
        }

        $dewpoint = null;
        if (isset($raw_data['dewpoint']) && isset($raw_data['dewpoint']['value'])) {
            $dewpoint = round($this->noaa_api->celsius_to_fahrenheit($raw_data['dewpoint']['value']));
            error_log('Dewpoint value found: ' . $dewpoint);
        } else {
            error_log('No dewpoint data found in raw data');
        }

        // Get cloud cover from layers
        $cloud_cover = null;
        if (isset($raw_data['cloudLayers']) && is_array($raw_data['cloudLayers'])) {
            foreach ($raw_data['cloudLayers'] as $layer) {
                if (isset($layer['amount'])) {
                    switch ($layer['amount']) {
                        case 'CLR':
                        case 'SKC':
                            $cloud_cover = 0;
                            break;
                        case 'FEW':
                            $cloud_cover = 25;
                            break;
                        case 'SCT':
                            $cloud_cover = 50;
                            break;
                        case 'BKN':
                            $cloud_cover = 75;
                            break;
                        case 'OVC':
                            $cloud_cover = 100;
                            break;
                    }
                    break; // Use the first layer
                }
            }
        }
        
        // Format values with fallbacks
        $humidity_display = $humidity !== null ? $humidity : '--';
        $pressure_display = $pressure !== null ? $pressure : '--';
        $visibility_display = $visibility !== null ? $visibility : '--';
        $dewpoint_display = $dewpoint !== null ? $dewpoint : '--';
        $cloud_cover_display = $cloud_cover !== null ? $cloud_cover : '--';

        return sprintf(
            '<div class="weather-narrative">
                <div class="current-conditions">
                    <div class="main-temp-section">
                        <div class="current-temp">%d°</div>
                        <div class="feels-like">
                            <span class="feels-like-label">Feels like</span>
                            <span class="feels-like-temp">%d°</span>
                        </div>
                    </div>
                    <div class="temp-range">
                        <div class="temp-range-item">
                            <span class="range-label">High</span>
                            <span class="high">%d°</span>
                        </div>
                        <div class="temp-range-item">
                            <span class="range-label">Low</span>
                            <span class="low">%d°</span>
                        </div>
                    </div>
                </div>
                <div class="weather-metrics">
                    <div class="metric-box">
                        <i class="fas fa-tint metric-icon"></i>
                        <div class="metric-content">
                            <div class="metric-label">Humidity</div>
                            <div class="metric-value">%s%%</div>
                        </div>
                    </div>
                    <div class="metric-box">
                        <i class="fas fa-wind metric-icon"></i>
                        <div class="metric-content">
                            <div class="metric-label">Wind</div>
                            <div class="metric-value">%s %s</div>
                        </div>
                    </div>
                    <div class="metric-box">
                        <i class="fas fa-compress-arrows-alt metric-icon"></i>
                        <div class="metric-content">
                            <div class="metric-label">Pressure</div>
                            <div class="metric-value">%s in</div>
                        </div>
                    </div>
                    <div class="metric-box">
                        <i class="fas fa-eye metric-icon"></i>
                        <div class="metric-content">
                            <div class="metric-label">Visibility</div>
                            <div class="metric-value">%s mi</div>
                        </div>
                    </div>
                    <div class="metric-box">
                        <i class="fas fa-tint metric-icon"></i>
                        <div class="metric-content">
                            <div class="metric-label">Dew Point</div>
                            <div class="metric-value">%s°</div>
                        </div>
                    </div>
                    <div class="metric-box">
                        <i class="fas fa-cloud metric-icon"></i>
                        <div class="metric-content">
                            <div class="metric-label">Cloud Cover</div>
                            <div class="metric-value">%s%%</div>
                        </div>
                    </div>
                </div>
                <div class="conditions-main">%s</div>
            </div>',
            $current,
            $high,
            $low,
            $feels_like,
            $humidity_display,
            esc_html($today['windSpeed']),
            esc_html($today['windDirection']),
            $pressure_display,
            $visibility_display,
            $dewpoint_display,
            $cloud_cover_display,
            wp_kses_post($narrative)
        );
    }

    private function generate_forecast_section($forecast) {
        if (empty($forecast)) {
            return '';
        }

        $days_html = '';
        foreach ($forecast as $day) {
            // Make sure we have valid temperatures
            $high = isset($day['high']) ? intval($day['high']) : null;
            $low = isset($day['low']) ? intval($day['low']) : null;
            
            if ($high === null || $low === null) {
                error_log('Weather Reporter - Missing temperature for day: ' . print_r($day, true));
                continue;
            }

            $days_html .= sprintf(
                '<div class="forecast-day">
                    <div class="day-name">%s</div>
                    <div class="day-icon"><i class="%s"></i></div>
                    <div class="day-temps">
                        <span class="high">%d°</span>
                        <span class="separator"> | </span>
                        <span class="low">%d°</span>
                    </div>
                    <div class="day-conditions">%s</div>
                </div>',
                esc_html($day['name']),
                esc_attr($this->get_weather_icon($day['shortForecast'])),
                $high,
                $low,
                esc_html($day['shortForecast'])
            );
        }

        return sprintf(
            '<div class="extended-forecast">
                <div class="forecast-days">%s</div>
            </div>',
            $days_html
        );
    }

    private function generate_hourly_section($hourly) {
        return ''; // Hourly forecast disabled
    }

    public function generate_title($today) {
        if (empty($today)) {
            return '';
        }

        $temp = isset($today['temperature']) ? $today['temperature'] : '?';
        $condition = isset($today['shortForecast']) ? $today['shortForecast'] : '';
        $city = isset($today['city']) ? $today['city'] : '';
        $state = isset($today['state']) ? $today['state'] : '';
        $detailed_forecast = isset($today['detailedForecast']) ? $today['detailedForecast'] : '';
        
        // Format the date
        $date = date('n/j/y');
        
        // Generate engaging title using Reintent API
        if ($this->reintent_api && $this->reintent_api->is_configured()) {
            $prompt = sprintf(
                'Write a short, engaging weather headline for %s, %s. Do not use icons or emojis. Here are the details:\n' .
                'Date: %s\n' .
                'Current conditions: %s\n' .
                'Temperature: %d°F\n' .
                'Detailed forecast: %s\n\n' .
                'Guidelines:\n' .
                '- Start with the date in format %s\n' .
                '- Keep it short and catchy (about 60 characters total)\n' .
                '- Focus on the most interesting aspect of today\'s weather\n' .
                '- Make it relevant to %s\n' .
                'Example formats:\n' .
                '%s: Bundle Up! Arctic Blast Brings Record Lows ❄️\n' .
                '%s: Perfect Weather for Outdoor Activities ☀️\n' .
                '%s: Storms Rolling Through - Stay Weather-Aware ',
                $city,
                $state,
                $date,
                $condition,
                $temp,
                $detailed_forecast,
                $date,
                $city,
                $date,
                $date,
                $date
            );
            
            try {
                $title = $this->reintent_api->generate_text($prompt);
                if (!empty($title)) {
                    // Ensure the title starts with the date
                    if (strpos($title, $date) !== 0) {
                        $title = $date . ': ' . $title;
                    }
                    return $title;
                }
            } catch (Exception $e) {
                error_log('Weather Reporter - Error generating title: ' . $e->getMessage());
            }
        }
        
        // Generate a more interesting fallback title based on conditions
        $interesting_aspects = array(
            'sunny' => 'Sunny Skies',
            'clear' => 'Clear Skies',
            'mostly sunny' => 'Mixed Skies',
            'partly' => 'Mixed Skies',
            'cloudy' => 'Cloud Cover',
            'rain' => 'Rain Alert',
            'storm' => 'Storm Watch',
            'snow' => 'Snow Day',
            'wind' => 'Windy Conditions',
            'fog' => 'Foggy Morning'
        );
        
        $title_weather = 'Weather Update';
        $condition_lower = strtolower($condition);
        foreach ($interesting_aspects as $key => $phrase) {
            if (strpos($condition_lower, $key) !== false) {
                $title_weather = $phrase;
                break;
            }
        }
        
        return sprintf(
            '%s: %s - %d°F in %s',
            $date,
            $title_weather,
            $temp,
            $city
        );
    }

    public function get_weather_icon($condition) {
        $condition = strtolower($condition);
        
        $icon_map = array(
            'sunny' => 'fas fa-sun',
            'clear' => 'fas fa-sun',
            'mostly sunny' => 'fas fa-sun',
            'partly sunny' => 'fas fa-cloud-sun',
            'mostly cloudy' => 'fas fa-cloud',
            'cloudy' => 'fas fa-cloud',
            'rain' => 'fas fa-cloud-rain',
            'showers' => 'fas fa-cloud-rain',
            'thunderstorm' => 'fas fa-bolt',
            'snow' => 'fas fa-snowflake',
            'fog' => 'fas fa-smog',
            'wind' => 'fas fa-wind',
            'default' => 'fas fa-cloud'
        );
        
        foreach ($icon_map as $key => $icon) {
            if (strpos($condition, $key) !== false) {
                return $icon;
            }
        }
        
        return $icon_map['default'];
    }

    public function get_weather_condition($today) {
        $condition = isset($today['shortForecast']) ? $today['shortForecast'] : 'cloudy';
        error_log('Weather Write - Raw weather condition from API: ' . $condition);
        return $condition;
    }

    public function get_weather_image_id($condition) {
        $weather_images = get_option('newspaper_ai_weather_images', array());
        $condition = strtolower($condition);
        error_log('Getting weather image for condition: ' . $condition);
        
        // Map conditions to image types
        $condition_map = array(
            'sunny' => 'sunny',
            'mostly sunny' => 'sunny',
            'partly sunny' => 'partly_cloudy',
            'mostly cloudy' => 'cloudy',
            'cloudy' => 'cloudy',
            'rain' => 'rain',
            'showers' => 'rain',
            'thunderstorm' => 'storm',
            'snow' => 'snow',
            'clear' => 'clear',
            'partly cloudy' => 'partly_cloudy',
            'overcast' => 'cloudy',
            'fog' => 'fog',
            'haze' => 'fog'
        );
        
        // Get the image type from the condition
        $image_type = isset($condition_map[$condition]) ? $condition_map[$condition] : 'cloudy';
        error_log('Mapped to image type: ' . $image_type);
        
        // Try to get a random variation if multiple exist
        $variations = array();
        for ($i = 1; $i <= 3; $i++) {
            $key = $image_type . '_' . $i;
            error_log('Checking key: ' . $key);
            if (isset($weather_images[$key]) && !empty($weather_images[$key])) {
                $variations[] = $weather_images[$key];
                error_log('Found image ID: ' . $weather_images[$key]);
            }
        }
        
        if (!empty($variations)) {
            $selected_id = $variations[array_rand($variations)];
            error_log('Selected image ID: ' . $selected_id);
            return $selected_id;
        }
        
        error_log('No image found for condition: ' . $condition);
        return null;
    }

    public function render_weather_page() {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }

        $active_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : 'settings';
        ?>
        <div class="wrap">
            <h2>Weather Settings</h2>
            
            <nav class="nav-tab-wrapper">
                <a href="?page=newspaper-ai-weather" class="nav-tab <?php echo $active_tab == 'settings' ? 'nav-tab-active' : ''; ?>">Settings</a>
                <a href="?page=newspaper-ai-weather&tab=images" class="nav-tab <?php echo $active_tab == 'images' ? 'nav-tab-active' : ''; ?>">Weather Images</a>
                <a href="?page=newspaper-ai-weather&tab=schedule" class="nav-tab <?php echo $active_tab == 'schedule' ? 'nav-tab-active' : ''; ?>">Schedule</a>
                <a href="?page=newspaper-ai-weather&tab=test" class="nav-tab <?php echo $active_tab == 'test' ? 'nav-tab-active' : ''; ?>">Test Report</a>
            </nav>

            <div class="tab-content">
                <?php
                switch ($active_tab) {
                    case 'images':
                        $this->render_images_tab();
                        break;
                    case 'schedule':
                        $this->render_schedule_tab();
                        break;
                    case 'test':
                        $this->render_test_tab();
                        break;
                    default:
                        $this->render_settings_tab();
                        break;
                }
                ?>
            </div>
        </div>
        <?php
    }

    private function render_schedule_tab() {
        // Get current schedule settings
        $schedule_settings = get_option('newspaper_ai_weather_schedule', array(
            'posts_per_day' => 1,
            'times' => array('09:00')
        ));

        ?>
        <div class="weather-schedule-settings">
            <h3>Automated Posting Schedule</h3>
            <p>Configure when weather reports should be automatically generated and published.</p>

            <div id="schedule-manager">
                <div class="schedule-controls">
                    <label for="posts_per_day">Posts Per Day:</label>
                    <select name="posts_per_day" id="posts_per_day">
                        <?php for ($i = 1; $i <= 6; $i++) : ?>
                            <option value="<?php echo $i; ?>" <?php selected($schedule_settings['posts_per_day'], $i); ?>>
                                <?php echo $i; ?> time<?php echo $i > 1 ? 's' : ''; ?> per day
                            </option>
                        <?php endfor; ?>
                    </select>
                </div>

                <div id="schedule-list" class="schedule-list">
                    <?php
                    for ($i = 0; $i < $schedule_settings['posts_per_day']; $i++) {
                        $time = isset($schedule_settings['times'][$i]) ? $schedule_settings['times'][$i] : '09:00';
                        $this->render_time_slot($i + 1, $time);
                    }
                    ?>
                </div>

                <div class="schedule-actions">
                    <button type="button" id="save-schedule" class="button button-primary">Save Schedule</button>
                    <span id="save-status" style="display: none; margin-left: 10px;"></span>
                </div>
            </div>
        </div>

        <style>
            .weather-schedule-settings {
                max-width: 600px;
                margin: 20px 0;
            }
            .schedule-controls {
                margin-bottom: 20px;
            }
            .schedule-controls label {
                margin-right: 10px;
                font-weight: bold;
            }
            .schedule-list {
                margin: 20px 0;
            }
            .time-slot {
                display: flex;
                align-items: center;
                margin-bottom: 15px;
                padding: 10px;
                background: #f9f9f9;
                border: 1px solid #ddd;
                border-radius: 4px;
            }
            .time-slot .slot-number {
                font-weight: bold;
                margin-right: 15px;
                min-width: 80px;
            }
            .time-slot input[type="time"] {
                margin-right: 15px;
            }
            .time-slot .slot-actions {
                margin-left: auto;
            }
            .time-slot .slot-actions button {
                margin-left: 5px;
            }
            .schedule-actions {
                margin-top: 20px;
            }
            #save-status {
                padding: 5px 10px;
                border-radius: 3px;
            }
            #save-status.success {
                background: #d4edda;
                color: #155724;
            }
            #save-status.error {
                background: #f8d7da;
                color: #721c24;
            }
        </style>

        <script>
        jQuery(document).ready(function($) {
            // Handle posts per day change
            $('#posts_per_day').on('change', function() {
                const count = parseInt($(this).val());
                updateTimeSlots(count);
            });

            // Function to update time slots
            function updateTimeSlots(count) {
                const container = $('#schedule-list');
                const currentSlots = container.find('.time-slot').length;
                
                if (count > currentSlots) {
                    // Add new slots
                    for (let i = currentSlots + 1; i <= count; i++) {
                        addTimeSlot(i);
                    }
                } else if (count < currentSlots) {
                    // Remove excess slots
                    container.find('.time-slot').slice(count).remove();
                }
            }

            // Function to add a new time slot
            function addTimeSlot(number) {
                const slot = $(`
                    <div class="time-slot" data-slot="${number}">
                        <span class="slot-number">Time Slot ${number}:</span>
                        <input type="time" name="posting_times[]" value="09:00" required>
                        <div class="slot-actions">
                            <button type="button" class="button reset-time">Reset</button>
                        </div>
                    </div>
                `);
                $('#schedule-list').append(slot);
            }

            // Handle reset time
            $(document).on('click', '.reset-time', function() {
                $(this).closest('.time-slot').find('input[type="time"]').val('09:00');
            });

            // Handle save schedule
            $('#save-schedule').on('click', function() {
                const button = $(this);
                const statusEl = $('#save-status');
                const times = [];
                
                // Collect all times
                $('.time-slot input[type="time"]').each(function() {
                    times.push($(this).val());
                });

                // Disable button during save
                button.prop('disabled', true);
                
                // Save via AJAX
                $.ajax({
                    url: ajaxurl,
                    type: 'POST',
                    data: {
                        action: 'save_weather_schedule',
                        nonce: '<?php echo wp_create_nonce("save_weather_schedule"); ?>',
                        posts_per_day: $('#posts_per_day').val(),
                        posting_times: times
                    },
                    success: function(response) {
                        statusEl.removeClass('error').addClass('success')
                            .text('Schedule saved successfully!')
                            .fadeIn();
                        
                        setTimeout(() => statusEl.fadeOut(), 3000);
                    },
                    error: function(xhr, status, error) {
                        statusEl.removeClass('success').addClass('error')
                            .text('Error saving schedule: ' + error)
                            .fadeIn();
                        
                        setTimeout(() => statusEl.fadeOut(), 3000);
                    },
                    complete: function() {
                        button.prop('disabled', false);
                    }
                });
            });
        });
        </script>
        <?php
    }

    private function render_time_slot($number, $time) {
        ?>
        <div class="time-slot" data-slot="<?php echo esc_attr($number); ?>">
            <span class="slot-number">Time Slot <?php echo esc_html($number); ?>:</span>
            <input type="time" 
                   name="posting_times[]" 
                   value="<?php echo esc_attr($time); ?>" 
                   required>
            <div class="slot-actions">
                <button type="button" class="button reset-time">Reset</button>
            </div>
        </div>
        <?php
    }

    private function get_weather_styles() {
        return '
        <style>
            .weather-report {
                max-width: 800px;
                margin: 0 auto;
                font-family: Arial, sans-serif;
                color: #333;
            }

            .weather-narrative {
                background: #fff;
                padding: 20px;
                border-radius: 8px;
                box-shadow: 0 2px 4px rgba(0,0,0,0.1);
                margin-bottom: 20px;
            }

            .temp-display {
                display: flex;
                align-items: center;
                margin-bottom: 15px;
            }

            .temp-display i {
                font-size: 2.5em;
                margin-right: 15px;
                color: #4a90e2;
            }

            .temperature {
                flex-grow: 1;
            }

            .current-temp {
                font-size: 2.5em;
                font-weight: bold;
                margin-bottom: 5px;
            }

            .high-low {
                font-size: 1.1em;
                color: #666;
            }

            .high { color: #e74c3c; }
            .low { color: #3498db; }

            .conditions-main {
                font-size: 1.1em;
                line-height: 1.6;
                margin: 15px 0;
            }

            .conditions-details {
                color: #666;
                font-size: 0.9em;
            }

            .wind-info i {
                margin-right: 5px;
                color: #7f8c8d;
            }

            /* Extended Forecast Styling */
            .extended-forecast {
                background: #fff;
                padding: 25px;
                border-radius: 8px;
                
                margin-top: 20px;
            }

            .extended-forecast h3 {
                text-align: center;
                margin: 0 0 20px 0;
                color: #000000;
                font-size: 1.3em;
            }

            .forecast-days {
                display: grid;
                grid-template-columns: repeat(2, 1fr);
                gap: 20px;
                padding: 10px;
            }

            .forecast-day {
                background: #f8f9fa;
                border-radius: 8px;
                padding: 20px;
                text-align: center;
                transition: all 0.3s ease;
                border: 1px solid #eee;
                height: 100%;
                display: flex;
                flex-direction: column;
                justify-content: space-between;
            }

            .forecast-day:hover {
                transform: translateY(-3px);
                box-shadow: 0 6px 12px rgba(0,0,0,0.1);
                border-color: #e0e0e0;
            }

            .forecast-day-header {
                font-weight: bold;
                color: #2c3e50;
                margin-bottom: 15px;
                font-size: 1.2em;
                padding-bottom: 10px;
                border-bottom: 1px solid #eee;
            }

            .forecast-day-content {
                display: flex;
                flex-direction: column;
                gap: 12px;
                flex-grow: 1;
            }

            .forecast-temps {
                font-size: 1.4em;
                margin: 12px 0;
                display: flex;
                align-items: center;
                justify-content: center;
            }

            .forecast-high {
                color: #e74c3c;
                font-weight: bold;
            }

            .forecast-separator {
                color: #95a5a6;
                margin: 0 2px;
            }

            .forecast-low {
                color: #3498db;
            }

            .forecast-details {
                color: #666;
                font-size: 1em;
                line-height: 1.4;
                margin: 8px 0;
            }

            .forecast-wind {
                color: #7f8c8d;
                font-size: 0.9em;
                padding: 8px 0;
                border-top: 1px solid #eee;
                margin-top: auto;
            }

            .forecast-wind i {
                margin-right: 6px;
            }

            /* Make it responsive */
            @media (max-width: 600px) {
                .forecast-days {
                    grid-template-columns: 1fr;
                }
                
                .forecast-day {
                    margin-bottom: 15px;
                }
            }

            /* Hourly Forecast Styling */
            .hourly-forecast {
                background: #fff;
                padding: 20px;
                border-radius: 8px;
                box-shadow: 0 2px 4px rgba(0,0,0,0.1);
                margin-top: 20px;
                overflow-x: auto;
            }

            .hourly-forecast h3 {
                margin: 0 0 15px 0;
                color: #2c3e50;
                font-size: 1.3em;
            }

            .hourly-periods {
                display: flex;
                gap: 20px;
                padding-bottom: 10px;
            }

            .hourly-period {
                min-width: 100px;
                text-align: center;
            }

            .hour {
                font-weight: bold;
                color: #2c3e50;
                margin-bottom: 5px;
            }

            .hour-temp {
                font-size: 1.2em;
                color: #e74c3c;
                margin: 5px 0;
            }

            .hour-conditions {
                font-size: 0.9em;
                color: #666;
            }

            .hour-wind {
                font-size: 0.85em;
                color: #7f8c8d;
                margin-top: 5px;
            }

            .hour-wind i {
                margin-right: 4px;
            }
        </style>';
    }
}
