<?php
/**
 * Weather Settings Class
 */

if (!defined('ABSPATH')) {
    exit;
}

require_once dirname(__FILE__) . '/class-geocoding.php';

class NewspaperAI_Weather_Settings {
    private static $instance = null;

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        // Add settings
        add_action('admin_init', array($this, 'register_settings'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
        
        // Add resync section
        add_action('admin_init', array($this, 'add_resync_section'));
        
        // Add AJAX handlers
        add_action('wp_ajax_update_weather_location', array($this, 'handle_location_update'));
        add_action('wp_ajax_nopriv_update_weather_location', array($this, 'handle_unauthorized'));
        
        // Add JavaScript to handle form submission
        add_action('admin_footer', function() {
            ?>
            <script type="text/javascript">
            jQuery(document).ready(function($) {
                $('form#reintent-settings-form').on('submit', function(e) {
                    e.preventDefault();
                    
                    var $form = $(this);
                    var $submitButton = $form.find(':submit');
                    var originalText = $submitButton.val();
                    
                    // Disable submit and show saving
                    $submitButton.prop('disabled', true).val('Saving...');
                    
                    // First save via normal form submission
                    $.ajax({
                        url: $form.attr('action'),
                        type: 'POST',
                        data: $form.serialize(),
                        success: function() {
                            // Show saving message
                            var notice = $('<div class="notice notice-info is-dismissible"><p>API key saved. Checking account status...</p></div>');
                            $('.wrap h2').after(notice);
                            
                            // Now trigger the Re-Sync
                            $('#reintent-resync-button').trigger('click');
                        },
                        error: function() {
                            // Show error
                            var notice = $('<div class="notice notice-error is-dismissible"><p>Failed to save API key. Please try again.</p></div>');
                            $('.wrap h2').after(notice);
                            $submitButton.prop('disabled', false).val(originalText);
                        }
                    });
                });
            });
            </script>
            <?php
        });
    }

    public function handle_unauthorized() {
        wp_send_json_error(array('message' => 'You must be logged in to perform this action.'));
    }

    public function register_settings() {
        // Add action to run after the API key is updated
        add_action('update_option_newspaperai_reintent_api_key', function($old_value, $new_value) {
            if ($old_value !== $new_value) {
                // Get the ReIntent API instance
                $reintent_api = NewspaperAI_Reintent_API::get_instance();
                
                // Force a refresh just like the Re-Sync button
                $reintent_api->force_refresh_status();
            }
        }, 10, 2);
        
        // Reintent API Key
        register_setting('weatherai', 'newspaperai_reintent_api_key', array(
            'type' => 'string',
            'sanitize_callback' => array($this, 'sanitize_api_key'),
            'default' => ''
        ));

        // Location Settings
        register_setting('weatherai', 'newspaper_ai_weather_location', array(
            'type' => 'array',
            'sanitize_callback' => array($this, 'sanitize_location'),
            'default' => array(
                'lat' => '36.1627',
                'lng' => '-86.7816',
                'city' => 'Nashville',
                'state' => 'TN'
            )
        ));

        // Add action to handle location updates via AJAX
        add_action('wp_ajax_update_weather_location', array($this, 'handle_location_update'));

        // Post Settings
        register_setting('weatherai', 'newspaper_ai_weather_post_settings', array(
            'type' => 'array',
            'sanitize_callback' => array($this, 'sanitize_post_settings'),
            'default' => array(
                'category' => '',
                'post_status' => 'draft'
            )
        ));

        // Reintent API Settings Section
        add_settings_section(
            'weather_reporter_reintent',
            'Reintent API Settings',
            array($this, 'reintent_section_callback'),
            'weatherai'
        );

        add_settings_field(
            'reintent_api_key',
            'Reintent API Key',
            array($this, 'api_key_callback'),
            'weatherai',
            'weather_reporter_reintent'
        );

        // Location Settings Section
        add_settings_section(
            'weather_reporter_location',
            'Location Settings',
            array($this, 'location_section_callback'),
            'weatherai'
        );

        add_settings_field(
            'location_city',
            'City',
            array($this, 'location_city_callback'),
            'weatherai',
            'weather_reporter_location'
        );

        add_settings_field(
            'location_state',
            'State',
            array($this, 'location_state_callback'),
            'weatherai',
            'weather_reporter_location'
        );

        add_settings_field(
            'location_coordinates',
            'Coordinates',
            array($this, 'location_coordinates_callback'),
            'weatherai',
            'weather_reporter_location'
        );

        // Post Settings Section
        add_settings_section(
            'weather_reporter_post',
            'Post Settings',
            array($this, 'post_section_callback'),
            'weatherai'
        );

        add_settings_field(
            'post_category',
            'Default Category',
            array($this, 'post_category_callback'),
            'weatherai',
            'weather_reporter_post'
        );

        add_settings_field(
            'post_status',
            'Default Post Status',
            array($this, 'post_status_callback'),
            'weatherai',
            'weather_reporter_post'
        );
    }

    /**
     * Sanitize the API key
     */
    public function sanitize_api_key($key) {
        $sanitized_key = sanitize_text_field($key);
        
        // Get the old key to check if it changed
        $old_key = get_option('newspaperai_reintent_api_key');
        
        if ($old_key !== $sanitized_key) {
            // Add JavaScript to trigger Re-Sync button click after save
            add_action('admin_footer', function() {
                ?>
                <script type="text/javascript">
                jQuery(document).ready(function($) {
                    // Wait for WordPress to finish saving
                    var checkExist = setInterval(function() {
                        if ($('#setting-error-settings_updated').length) {
                            clearInterval(checkExist);
                            // Force click the Re-Sync button
                            $('#reintent-resync-button').trigger('click');
                        }
                    }, 100);
                });
                </script>
                <?php
            });
        }
        
        return $sanitized_key;
    }

    public function reintent_section_callback() {
        echo '<p>Configure your Reintent API key for enhanced weather narratives.</p>';
    }

    public function location_section_callback() {
        echo '<p>Configure the location for weather reports.</p>';
    }

    public function location_city_callback() {
        $location = get_option('newspaper_ai_weather_location', array(
            'lat' => '',
            'lng' => '',
            'city' => '',
            'state' => ''
        ));
        $city = isset($location['city']) ? $location['city'] : '';
        ?>
        <input type="text" 
               id="weather-city"
               name="newspaper_ai_weather_location[city]" 
               value="<?php echo esc_attr($city); ?>" 
               class="regular-text"
               data-last-value="<?php echo esc_attr($city); ?>">
        <?php
    }

    public function location_state_callback() {
        $location = get_option('newspaper_ai_weather_location', array(
            'lat' => '',
            'lng' => '',
            'city' => '',
            'state' => ''
        ));
        $state = isset($location['state']) ? $location['state'] : 'TN';
        ?>
        <input type="text" 
               id="weather-state"
               name="newspaper_ai_weather_location[state]" 
               value="<?php echo esc_attr($state); ?>" 
               class="regular-text"
               maxlength="2"
               style="width: 60px;"
               data-last-value="<?php echo esc_attr($state); ?>">
        <p class="description">Two-letter state code (e.g., TN)</p>
        <?php
    }

    public function location_coordinates_callback() {
        $location = get_option('newspaper_ai_weather_location', array(
            'lat' => '',
            'lng' => '',
            'city' => '',
            'state' => ''
        ));
        $lat = isset($location['lat']) ? $location['lat'] : '36.1627';
        $lng = isset($location['lng']) ? $location['lng'] : '-86.7816';
        ?>
        <div class="coordinates-wrapper">
            <div class="coordinates-display">
                <input type="hidden" 
                       name="newspaper_ai_weather_location[lat]" 
                       value="<?php echo esc_attr($lat); ?>" 
                       id="weather-lat">
                <input type="hidden" 
                       name="newspaper_ai_weather_location[lng]" 
                       value="<?php echo esc_attr($lng); ?>" 
                       id="weather-lng">
                <div class="coordinate-info">
                    <p>
                        <strong>Current Coordinates:</strong> 
                        <span id="coordinates-display"><?php echo esc_html(round(floatval($lat), 4) . ', ' . round(floatval($lng), 4)); ?></span>
                    </p>
                    <button type="button" id="update-coordinates" class="button button-primary">
                        Update Coordinates from City/State
                    </button>
                </div>
                <div id="coordinates-status" class="notice notice-info hidden" style="margin-top: 10px;">
                    <p></p>
                </div>
            </div>
        </div>
        <style>
            .coordinates-wrapper {
                margin-top: 10px;
            }
            .coordinate-info {
                display: flex;
                align-items: center;
                gap: 15px;
                margin-bottom: 10px;
            }
            .coordinate-info p {
                margin: 0;
            }
            #coordinates-status {
                margin-left: 0;
                margin-right: 0;
                padding: 5px 12px;
            }
            #coordinates-status:not(.hidden) {
                display: block;
            }
            #coordinates-status p {
                margin: 0.5em 0;
            }
        </style>
        <?php
    }

    public function sanitize_location($location) {
        if (!is_array($location)) {
            return array(
                'city' => '',
                'state' => '',
                'lat' => '',
                'lng' => ''
            );
        }

        return array(
            'city' => isset($location['city']) ? sanitize_text_field($location['city']) : '',
            'state' => isset($location['state']) ? sanitize_text_field($location['state']) : '',
            'lat' => isset($location['lat']) ? sanitize_text_field($location['lat']) : '',
            'lng' => isset($location['lng']) ? sanitize_text_field($location['lng']) : ''
        );
    }

    /**
     * Enqueue admin scripts and styles
     */
    /**
     * Add resync section to settings page (visually hidden with CSS)
     */
    public function add_resync_section() {
        add_settings_section(
            'reintent_resync_section',
            'Re-Sync Account',
            array($this, 'render_resync_section'),
            'weatherai'
        );
    }

    /**
     * Render resync section (visually hidden with CSS)
     */
    public function render_resync_section() {
        ?>
        <style type="text/css">
            /* Hide the Re-Sync Account section */
            #reintent_resync_section,
            .reintent-resync-section,
            h2:contains('Re-Sync Account'),
            .form-table tr:has(th:contains('Re-Sync Account')),
            .form-table tr:has(#reintent_resync_section) {
                display: none !important;
            }
        </style>
        <div class="reintent-resync-section" style="display: none !important;">
            <p>If you have recently made changes to your account and need to re-sync your account status, please click here:</p>
            <button type="button" class="button button-secondary" id="reintent-resync-button">Re-Sync Account</button>
        </div>
        <script type="text/javascript">
        jQuery(document).ready(function($) {
            // Hide the section header and all related elements
            $('h2:contains("Re-Sync Account")').hide();
            $('#reintent_resync_section').hide();
            $('.form-table tr:has(th:contains("Re-Sync Account"))').hide();
            $('.form-table tr:has(#reintent_resync_section)').hide();
            
            $('#reintent-resync-button').on('click', function() {
                var $button = $(this);
                $button.prop('disabled', true).text('Re-syncing...');

                $.ajax({
                    url: '<?php echo admin_url('admin-ajax.php'); ?>',
                    type: 'POST',
                    data: {
                        action: 'reintent_resync_account',
                        nonce: '<?php echo wp_create_nonce("reintent_resync_account"); ?>'
                    },
                    success: function(response) {
                        if (response.success) {
                            location.reload();
                        } else {
                            alert('Failed to resync account. Please try again.');
                            $button.prop('disabled', false).text('Re-Sync Account');
                        }
                    },
                    error: function() {
                        alert('Failed to resync account. Please try again.');
                        $button.prop('disabled', false).text('Re-Sync Account');
                    }
                });
            });
        });
        </script>
        <?php
    }

    public function enqueue_admin_scripts($hook) {
        newspaper_ai_debug_log('Enqueuing scripts on hook: ' . $hook, 'WeatherSettings');

        // Debug the current page
        newspaper_ai_debug_log('Current page: ' . $_SERVER['REQUEST_URI'], 'WeatherSettings');

        // Only load on our settings page - more flexible check
        if (!strpos($hook, 'newspaper-ai-settings') && !strpos($hook, 'weatherai')) {
            newspaper_ai_debug_log('Not loading scripts - wrong page', 'WeatherSettings');
            return;
        }

        newspaper_ai_debug_log('Loading weather settings scripts', 'WeatherSettings');

        // Enqueue jQuery first
        wp_enqueue_script('jquery');

        // Get the absolute path to the JS file
        $js_path = plugins_url('/js/weather-settings.js', __FILE__);
        newspaper_ai_debug_log('JS Path: ' . $js_path, 'WeatherSettings');

        // Enqueue with version number for cache busting
        $version = time(); // Development only, use plugin version in production
        
        wp_enqueue_script(
            'weather-settings',
            $js_path,
            array('jquery'),
            $version,
            false // Load in header for debugging
        );

        // Get the full URL including port
        $ajax_url = ( isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https://' : 'http://' ) .
                    $_SERVER['HTTP_HOST'] .
                    (strpos($_SERVER['HTTP_HOST'], ':') === false && isset($_SERVER['SERVER_PORT']) ? ':' . $_SERVER['SERVER_PORT'] : '') .
                    '/wp-admin/admin-ajax.php';

        newspaper_ai_debug_log('Using AJAX URL: ' . $ajax_url, 'WeatherSettings');

        $settings = array(
            'ajaxUrl' => $ajax_url,
            'nonce' => wp_create_nonce('weatherai-update-location')
        );

        wp_localize_script('weather-settings', 'weatherSettings', $settings);
        
        newspaper_ai_debug_log('Localized settings: ' . print_r($settings, true), 'WeatherSettings');
        
        // Check if script is enqueued
        newspaper_ai_debug_log('Script enqueued: ' . (wp_script_is('weather-settings', 'enqueued') ? 'yes' : 'no'), 'WeatherSettings');
    }

    /**
     * Handle AJAX request to update location
     */
    public function handle_location_update() {
        // Handle CORS
        $origin = isset($_SERVER['HTTP_ORIGIN']) ? $_SERVER['HTTP_ORIGIN'] : '';
        if ($origin) {
            header('Access-Control-Allow-Origin: ' . $origin);
            header('Access-Control-Allow-Credentials: true');
            header('Access-Control-Allow-Methods: POST, GET, OPTIONS');
            header('Access-Control-Allow-Headers: Content-Type');
        }

        // Handle preflight OPTIONS request
        if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
            status_header(200);
            exit();
        }

        newspaper_ai_debug_log('1. Starting location update...', 'WeatherSettings');
        newspaper_ai_debug_log('2. Request method: ' . $_SERVER['REQUEST_METHOD'], 'WeatherSettings');
        newspaper_ai_debug_log('3. POST data: ' . print_r($_POST, true), 'WeatherSettings');
        newspaper_ai_debug_log('4. Headers: ' . print_r(getallheaders(), true), 'WeatherSettings');

        // Debug nonce
        $nonce = isset($_POST['nonce']) ? $_POST['nonce'] : 'not set';
        $nonce_valid = wp_verify_nonce($nonce, 'weatherai-update-location');
        newspaper_ai_debug_log('5. Nonce check: ' . ($nonce_valid ? 'valid' : 'invalid') . ' (received: ' . $nonce . ')', 'WeatherSettings');

        if (!check_ajax_referer('weatherai-update-location', 'nonce', false)) {
            newspaper_ai_debug_log('6. Invalid nonce - sending error response', 'WeatherSettings');
            wp_send_json_error(array('message' => 'Security check failed'));
            return;
        }

        if (!current_user_can('manage_options')) {
            newspaper_ai_debug_log('Unauthorized access attempt', 'WeatherSettings');
            wp_send_json_error(array('message' => 'Unauthorized access'));
            return;
        }

        $city = isset($_POST['city']) ? sanitize_text_field($_POST['city']) : '';
        $state = isset($_POST['state']) ? sanitize_text_field($_POST['state']) : '';

        newspaper_ai_debug_log('Received location data - City: ' . $city . ', State: ' . $state, 'WeatherSettings');

        if (empty($city) || empty($state)) {
            newspaper_ai_debug_log('Missing city or state', 'WeatherSettings');
            wp_send_json_error(array('message' => 'City and state are required'));
            return;
        }

        if (strlen($state) !== 2) {
            newspaper_ai_debug_log('Invalid state format: ' . $state, 'WeatherSettings');
            wp_send_json_error(array('message' => 'State must be a two-letter code'));
            return;
        }

        try {
            newspaper_ai_debug_log('Initializing geocoder...', 'WeatherSettings');
            $geocoder = NewspaperAI_Geocoding::get_instance();
            $address = $city . ', ' . $state;
            
            $coordinates = $geocoder->geocode_address($address);
            newspaper_ai_debug_log('Geocoding result: ' . print_r($coordinates, true), 'WeatherSettings');

            if (isset($coordinates['error'])) {
                newspaper_ai_debug_log('Geocoding failed: ' . $coordinates['error'], 'WeatherSettings');
                wp_send_json_error(array('message' => $coordinates['error']));
                return;
            }

            if (!$coordinates || empty($coordinates['lat']) || empty($coordinates['lng'])) {
                newspaper_ai_debug_log('Failed to get coordinates', 'WeatherSettings');
                wp_send_json_error(array('message' => 'Could not find coordinates for ' . $city . ', ' . $state));
                return;
            }

            // Prepare the new location data
            $location = array(
                'city' => $city,
                'state' => $state,
                'lat' => $coordinates['lat'],
                'lng' => $coordinates['lng'],
                'last_updated' => current_time('mysql')
            );

            // Update the option
            $update_result = update_option('newspaper_ai_weather_location', $location);
            
            // Double check the saved value
            $saved_location = get_option('newspaper_ai_weather_location');
            newspaper_ai_debug_log('Saved location value: ' . print_r($saved_location, true), 'WeatherSettings');
            newspaper_ai_debug_log('Update result: ' . ($update_result ? 'success' : 'failed'), 'WeatherSettings');
            newspaper_ai_debug_log('Updated location: ' . print_r($location, true), 'WeatherSettings');

            wp_send_json_success(array(
                'lat' => $coordinates['lat'],
                'lng' => $coordinates['lng'],
                'message' => 'Location updated to ' . $city . ', ' . $state
            ));

        } catch (Exception $e) {
            newspaper_ai_debug_log('Error: ' . $e->getMessage(), 'WeatherSettings');
            wp_send_json_error(array('message' => 'Error updating coordinates: ' . $e->getMessage()));
        }
    }

    public function post_section_callback() {
        echo '<p>Configure default settings for generated weather posts.</p>';
    }

    public function post_category_callback() {
        $settings = get_option('newspaper_ai_weather_post_settings', array(
            'category' => '',
            'post_status' => 'draft'
        ));
        
        $categories = get_categories(array('hide_empty' => false));
        $selected_cat = $settings['category'];
        
        // If no category is selected and 'weather' category exists, select it
        if (empty($selected_cat)) {
            foreach ($categories as $category) {
                if (strtolower($category->name) === 'weather') {
                    $selected_cat = $category->term_id;
                    // Save this as the default
                    $settings['category'] = $selected_cat;
                    update_option('newspaper_ai_weather_post_settings', $settings);
                    break;
                }
            }
        }
        
        echo '<select name="newspaper_ai_weather_post_settings[category]" class="regular-text">';
        foreach ($categories as $category) {
            printf(
                '<option value="%s" %s>%s</option>',
                esc_attr($category->term_id),
                selected($selected_cat, $category->term_id, false),
                esc_html($category->name)
            );
        }
        echo '</select>';
        echo '<p class="description">Select the default category for weather posts.</p>';
    }

    public function post_status_callback() {
        $settings = get_option('newspaper_ai_weather_post_settings', array(
            'category' => '',
            'post_status' => 'draft'
        ));
        
        $statuses = array(
            'draft' => 'Save as Draft',
            'publish' => 'Publish Immediately'
        );
        
        echo '<select name="newspaper_ai_weather_post_settings[post_status]" class="regular-text">';
        foreach ($statuses as $value => $label) {
            printf(
                '<option value="%s" %s>%s</option>',
                esc_attr($value),
                selected($settings['post_status'], $value, false),
                esc_html($label)
            );
        }
        echo '</select>';
        echo '<p class="description">Choose whether to save posts as drafts or publish them immediately.</p>';
    }

    public function sanitize_post_settings($settings) {
        if (!isset($settings['category'])) {
            $settings['category'] = '';
        }
        if (!isset($settings['post_status'])) {
            $settings['post_status'] = 'draft';
        }
        return array(
            'category' => absint($settings['category']),
            'post_status' => in_array($settings['post_status'], array('draft', 'publish')) ? $settings['post_status'] : 'draft'
        );
    }

    /**
     * API key field callback
     */
    public function api_key_callback() {
        $api_key = get_option('newspaperai_reintent_api_key');
        ?>
        <input type="text" 
               name="newspaperai_reintent_api_key" 
               value="<?php echo esc_attr($api_key); ?>" 
               class="regular-text"
               autocomplete="new-password">
        <p class="description">
            Enter your Reintent API key. This key is required for generating natural and engaging weather narratives.
        </p>
        <?php
    }

    /**
     * Render the settings page
     */
    public function render_settings_page() {
        if (!current_user_can('manage_options')) {
            return;
        }

        if (isset($_GET['settings-updated'])) {
            add_settings_error(
                'weather_reporter_messages',
                'weather_reporter_message',
                'Settings Saved',
                'updated'
            );
        }
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
            <?php settings_errors('weather_reporter_messages'); ?>

            <form action="options.php" method="post">
                <?php
                settings_fields('weatherai');
                do_settings_sections('weatherai');
                submit_button('Save Settings');
                ?>
            </form>
        </div>
        <?php
    }
}
