console.log('WEATHER SETTINGS SCRIPT STARTING - ' + new Date().toISOString());
console.log('Script loaded on page:', window.location.href);

// Check if jQuery is available
if (typeof jQuery === 'undefined') {
    console.error('jQuery not loaded!');
} else {
    console.log('jQuery version:', jQuery.fn.jquery);
}

jQuery(function($) {
    'use strict';
    
    // Make sure we have access to the weatherSettings object
    if (typeof weatherSettings === 'undefined') {
        console.error('Weather settings not found. Make sure the script is properly enqueued.');
        return;
    }
    console.log('Weather settings loaded with:', weatherSettings);

    // Get references to DOM elements
    const cityInput = $('#weather-city');
    const stateInput = $('#weather-state');
    const latInput = $('#weather-lat');
    const lngInput = $('#weather-lng');
    const coordsDisplay = $('#coordinates-display');
    const statusDiv = $('#coordinates-status');
    const updateButton = $('#update-coordinates');

    // Early exit if we can't find our elements
    if (!updateButton.length) {
        // Only log error if we're on the settings page
        if (window.location.href.includes('newspaper-ai-settings')) {
            console.error('Update coordinates button not found');
        }
        return;
    }

    // Log if elements are found
    console.log('Elements found:', {
        cityInput: cityInput.length,
        stateInput: stateInput.length,
        latInput: latInput.length,
        lngInput: lngInput.length,
        coordsDisplay: coordsDisplay.length,
        statusDiv: statusDiv.length,
        updateButton: updateButton.length
    });

    function showStatus(message, type = 'info') {
        console.log('Showing status:', type, message);
        statusDiv
            .removeClass('hidden notice-info notice-error notice-success')
            .addClass('notice-' + type)
            .show()
            .find('p')
            .text(message);
    }

    function hideStatus() {
        statusDiv.fadeOut('slow', function() {
            $(this).addClass('hidden');
        });
    }

    function updateCoordinates(e) {
        console.log('1. Update coordinates function called');
        
        // Always prevent default
        e.preventDefault();
        console.log('2. Default prevented');

        // Get trimmed values
        const city = (cityInput.val() || '').trim();
        const state = (stateInput.val() || '').trim();
        console.log('3. Values retrieved - City:', city, 'State:', state);

        if (!city || !state) {
            console.log('4a. Validation failed - missing city or state');
            showStatus('Please enter both city and state', 'error');
            return;
        }

        if (state.length !== 2) {
            console.log('4b. Validation failed - invalid state format');
            showStatus('State must be a two-letter code (e.g., TN)', 'error');
            return;
        }

        console.log('4. Validation passed');

        // Show loading status
        showStatus('Updating coordinates...');
        updateButton.prop('disabled', true);
        console.log('5. Loading status shown');

        // Log the request data
        console.log('6. AJAX request details:', {
            url: weatherSettings.ajaxUrl,
            action: 'update_weather_location',
            nonce: weatherSettings.nonce,
            city: city,
            state: state
        });

        // Log the current form state
        console.log('Form state before AJAX:', {
            city: city,
            state: state,
            currentLat: latInput.val(),
            currentLng: lngInput.val(),
            ajaxUrl: weatherSettings.ajaxUrl,
            nonce: weatherSettings.nonce
        });

        // Make the AJAX request
        $.ajax({
            url: weatherSettings.ajaxUrl,
            type: 'POST',
            dataType: 'json',
            xhrFields: {
                withCredentials: true
            },
            crossDomain: true,
            data: {
                action: 'update_weather_location',
                nonce: weatherSettings.nonce,
                city: city,
                state: state
            },
            success: function(response) {
                console.log('7. AJAX Success Response:', response);
                console.log('Current form values:', {
                    city: cityInput.val(),
                    state: stateInput.val(),
                    lat: latInput.val(),
                    lng: lngInput.val()
                });
                
                if (response.success && response.data) {
                    // Update the coordinate inputs
                    latInput.val(response.data.lat);
                    lngInput.val(response.data.lng);
                    
                    // Update the display with more precise formatting
                    const formattedCoords = 
                        `${parseFloat(response.data.lat).toFixed(6)}°, ${parseFloat(response.data.lng).toFixed(6)}°`;
                    
                    coordsDisplay.text(formattedCoords);

                    // Show success message with city and state
                    showStatus(`Coordinates updated for ${city}, ${state}!`, 'success');
                    
                    // Hide success message after 5 seconds
                    setTimeout(hideStatus, 5000);
                } else {
                    const errorMessage = response.data && response.data.message ? response.data.message : 'Failed to update coordinates';
                    showStatus(errorMessage, 'error');
                }
            },
            error: function(xhr, status, error) {
                console.error('AJAX Error:', {
                    status: status,
                    error: error,
                    response: xhr.responseText
                });
                showStatus('Failed to update coordinates. Please try again.', 'error');
            },
            complete: function() {
                updateButton.prop('disabled', false);
            }
        });
    }

    // Remove any existing click handlers and attach new one
    updateButton.off('click').on('click', updateCoordinates);
  

    // Log initial state
    console.log('Initial coordinates:', {
        lat: latInput.val(),
        lng: lngInput.val(),
        city: cityInput.val(),
        state: stateInput.val()
    });
});
