<?php
/**
 * Register Weather Images
 * 
 * This script registers all existing weather images in the WordPress database.
 * It should be run from the WordPress admin area.
 */

// Prevent direct access
if (!defined('ABSPATH')) {
    die('Direct access not allowed.');
}

// Ensure user is an admin
if (!current_user_can('manage_options')) {
    wp_die('You do not have sufficient permissions to access this page.');
}

// Start output
echo '<div style="font-family: Arial, sans-serif; max-width: 800px; margin: 20px auto; padding: 20px; border: 1px solid #ddd; border-radius: 5px;">';
echo '<h1>Register Weather Images</h1>';
echo '<p>This tool will register all existing weather images in the WordPress database.</p>';

// Set up paths
$plugin_dir = plugin_dir_path(dirname(dirname(__FILE__)));
$plugin_images_dir = $plugin_dir . 'assets/images/weather/';

// Get site URL for image URLs
$site_url = site_url();
$plugin_url = trailingslashit($site_url) . 'wp-content/plugins/Publisher_Plugin/';
$images_url = $plugin_url . 'assets/images/weather/';

// Get current weather images option
$weather_images = get_option('weather_write_images', array());

// Define all possible weather types (including variations)
$all_weather_types = array(
    // Base weather types
    'sunny', 'cloudy', 'fog', 'lightning', 'overcast', 'rain', 'snow',
    
    // UI types
    'partly_cloudy', 'storm',
    
    // Additional mapped types
    'clear', 'hot', 'partly_sunny', 'mostly_cloudy', 'rain_light', 'rain_heavy',
    'showers', 'thunderstorm', 'thunderstorm_severe', 'snow_light', 'snow_heavy',
    'mist', 'haze', 'windy', 'cold', 'sunrise', 'sunset', 'sleet'
);

// Check if form was submitted
if (isset($_POST['register_images_submit']) && check_admin_referer('register_images_action', 'register_images_nonce')) {
    echo '<div style="background-color: #f0f8ff; padding: 15px; margin: 15px 0; border-radius: 5px;">';
    echo '<h2>Registering Images...</h2>';
    echo '<ul>';
    
    $success_count = 0;
    $error_count = 0;
    
    // Process each weather type
    foreach ($all_weather_types as $type) {
        for ($variation = 1; $variation <= 3; $variation++) {
            $image_path = $plugin_images_dir . $type . '-' . $variation . '.jpg';
            $image_url = $images_url . $type . '-' . $variation . '.jpg';
            
            // Check if the image file exists
            if (file_exists($image_path)) {
                echo "<li>Processing image: {$type}-{$variation}.jpg... ";
                
                // Create the database key (with underscore format)
                $db_key = $type . '_' . $variation;
                
                // Check if we already have this image in the database
                if (isset($weather_images[$db_key]) && 
                    ((is_array($weather_images[$db_key]) && !empty($weather_images[$db_key]['url'])) || 
                     (!is_array($weather_images[$db_key]) && !empty($weather_images[$db_key])))) {
                    echo "<span style='color: blue;'>Already in database.</span></li>";
                    continue;
                }
                
                // Create a WordPress attachment for the image if it doesn't exist
                $attachment_id = create_attachment_from_file($image_path, $type . '-' . $variation . '.jpg');
                
                if ($attachment_id) {
                    // Update the database entry
                    $weather_images[$db_key] = array(
                        'id' => $attachment_id,
                        'url' => $image_url
                    );
                    
                    echo "<span style='color: green;'>Added to database with ID: {$attachment_id}</span></li>";
                    $success_count++;
                } else {
                    // If we can't create an attachment, just store the URL
                    $weather_images[$db_key] = array(
                        'id' => 0,
                        'url' => $image_url
                    );
                    echo "<span style='color: orange;'>Added URL to database (no attachment)</span></li>";
                    $success_count++;
                }
            }
        }
    }
    
    // Save the updated weather images option
    update_option('weather_write_images', $weather_images);
    
    echo '</ul>';
    echo "<p><strong>Registration complete:</strong> {$success_count} images registered, {$error_count} errors.</p>";
    echo '</div>';
}

// Display form to run the registration
echo '<form method="post" action="">';
wp_nonce_field('register_images_action', 'register_images_nonce');
echo '<p><input type="submit" name="register_images_submit" class="button button-primary" value="Register Images"></p>';
echo '</form>';

// Display current registered images
echo '<h2>Currently Registered Images</h2>';
echo '<p>The following images are currently registered in the database:</p>';

// Count registered images
$registered_count = 0;
foreach ($weather_images as $key => $value) {
    if (is_array($value) && !empty($value['url'])) {
        $registered_count++;
    } elseif (!is_array($value) && !empty($value)) {
        $registered_count++;
    }
}

echo "<p><strong>{$registered_count}</strong> images are currently registered.</p>";

// Add a button to return to the weather settings page
echo '<p><a href="' . admin_url('admin.php?page=weatherai&tab=images') . '" class="button button-primary">Return to Weather Images</a></p>';

echo '</div>';

/**
 * Create a WordPress attachment from a file
 * 
 * @param string $file_path Path to the file
 * @param string $filename Filename to use for the attachment
 * @return int|false Attachment ID or false on failure
 */
function create_attachment_from_file($file_path, $filename) {
    // Check if file exists
    if (!file_exists($file_path)) {
        return false;
    }
    
    // Get file type
    $filetype = wp_check_filetype($filename, null);
    
    // Prepare attachment data
    $attachment = array(
        'guid' => $file_path,
        'post_mime_type' => $filetype['type'],
        'post_title' => preg_replace('/\.[^.]+$/', '', $filename),
        'post_content' => '',
        'post_status' => 'inherit'
    );
    
    // Insert the attachment
    $attachment_id = wp_insert_attachment($attachment, $file_path);
    
    if (!$attachment_id) {
        return false;
    }
    
    // Generate metadata for the attachment
    require_once(ABSPATH . 'wp-admin/includes/image.php');
    $attachment_data = wp_generate_attachment_metadata($attachment_id, $file_path);
    wp_update_attachment_metadata($attachment_id, $attachment_data);
    
    return $attachment_id;
}
