<?php
/**
 * Weather Module
 */

if (!defined('ABSPATH')) {
    exit;
}

class NewspaperAI_Weather {
    private $weather_reporter;
    private $reintent_api;

    private static $instance = null;

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        // Initialize dependencies
        NewspaperAI_Weather_Settings::get_instance();
        $this->weather_reporter = NewspaperAI_Weather_Reporter::get_instance();
        $this->reintent_api = NewspaperAI_Reintent_API::get_instance();
        
        // Make sure the Weather_Images class is loaded
        require_once dirname(__FILE__) . '/class-weather-images.php';
        NewspaperAI_Weather_Images::get_instance();
        
        // Include the installer and admin fix images page
        require_once dirname(__FILE__) . '/class-weather-images-installer.php';
        NewspaperAI_Weather_Images_Installer::get_instance();
        require_once dirname(__FILE__) . '/admin-fix-images.php';
        
        // Add menu items and settings
        add_action('admin_menu', array($this, 'add_menu_items'));
        add_action('admin_init', array($this, 'register_settings'));
        
        // Allow data attributes in HTML
        add_filter('wp_kses_allowed_html', array($this, 'allow_data_attributes'), 10, 2);

        // Add frontend styles
        add_action('wp_enqueue_scripts', array($this, 'enqueue_weather_styles'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));

        // AJAX handlers
        add_action('wp_ajax_test_weather_report', array($this, 'handle_test_request'));
        add_action('wp_ajax_save_weather_schedule', array($this, 'handle_ajax_save_schedule'));
        add_action('wp_ajax_save_weather_image', array($this, 'handle_ajax_save_weather_image'));
        add_action('wp_ajax_toggle_weather_debug', array($this, 'handle_toggle_debug'));
        add_action('wp_ajax_toggle_weather_scheduling', array($this, 'handle_toggle_scheduling'));

        // Frontend actions
        add_action('wp_enqueue_scripts', array($this, 'enqueue_weather_styles'));
        add_action('newspaperai_weather_cron', array($this, 'generate_scheduled_report'));
        add_shortcode('weather_map', array($this, 'weather_map_shortcode'));
        add_filter('wp_kses_allowed_html', array($this, 'allow_data_attributes'), 10, 2);

        // Initialize plugin settings
        // We're not registering schedule-related options with the settings API
        // to prevent them from being cleared when saving settings
        // They will still be managed as WordPress options
        
        // Register other settings as normal
        register_setting('weatherai', 'newspaper_ai_weather_debug_mode', array(
            'type' => 'boolean',
            'default' => false,
            'sanitize_callback' => 'rest_sanitize_boolean'
        ));
        
        // Initialize settings if they don't exist
        if (false === get_option('newspaper_ai_weather_scheduling_enabled')) {
            add_option('newspaper_ai_weather_scheduling_enabled', false);
        }
        if (false === get_option('newspaper_ai_weather_schedule')) {
            add_option('newspaper_ai_weather_schedule', array(
                'posts_per_day' => 1,
                'times' => array('09:00')
            ));
        }
        
        // Admin post handlers
        add_action('admin_post_test_weather_report', array($this, 'handle_test_request'));
        
        // AJAX handlers
        add_action('wp_ajax_save_weather_schedule', array($this, 'handle_ajax_save_schedule'));
        add_action('wp_ajax_save_weather_image', array($this, 'handle_ajax_save_weather_image'));
        add_action('wp_ajax_toggle_weather_debug', array($this, 'handle_toggle_debug'));
        add_action('wp_ajax_toggle_weather_scheduling', array($this, 'handle_toggle_scheduling'));
        
        // Allow data attributes in post content
        add_filter('wp_kses_allowed_html', array($this, 'allow_data_attributes'), 10, 2);
        
        // Enqueue scripts and styles
        add_action('wp_enqueue_scripts', array($this, 'enqueue_styles'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
        add_action('wp_enqueue_scripts', array($this, 'enqueue_weather_styles'));
        add_action('weather_write_cron', array($this, 'generate_scheduled_report'));
        add_shortcode('weather_map', array($this, 'weather_map_shortcode'));
    }

    /**
     * Register plugin settings
     */
    public function register_settings() {
        // Register the debug mode setting
        register_setting(
            'weatherai',
            'weather_write_debug_mode',
            array(
                'type' => 'boolean',
                'default' => false,
                'sanitize_callback' => function($value) {
                    return (bool)$value;
                }
            )
        );

        // Register the author setting
        register_setting(
            'weatherai',
            'weather_write_author',
            array(
                'type' => 'integer',
                'default' => 1, // Default to admin user
                'sanitize_callback' => 'absint'
            )
        );

        // Add settings section
        add_settings_section(
            'weatherai_main',
            'Weather Settings',
            array($this, 'settings_section_callback'),
            'weatherai'
        );

        add_settings_field(
            'weather_author',
            'Default Author',
            array($this, 'author_field_callback'),
            'weatherai',
            'weatherai_main'
        );
    }

    /**
     * Settings section description
     */
    public function settings_section_callback() {
        echo '<p>Configure your weather report settings below.</p>';
    }

    /**
     * Author field callback
     */
    public function author_field_callback() {
        $current_author = get_option('weather_write_author', 1);
        
        // Get users who can publish posts
        $users = get_users(array(
            'capability' => array('publish_posts'),
            'orderby' => 'display_name'
        ));
        
        if (empty($users)) {
            echo '<p>No users with publishing capabilities found.</p>';
            return;
        }
        
        echo '<select name="weather_write_author" class="regular-text">';
        foreach ($users as $user) {
            printf(
                '<option value="%d" %s>%s</option>',
                $user->ID,
                selected($current_author, $user->ID, false),
                esc_html($user->display_name)
            );
        }
        echo '</select>';
        echo '<p class="description">Select the default author for weather reports.</p>';
    }

    public function add_menu_items() {
        add_menu_page(
            'Weather Write Settings',
            'Weather Write',
            'manage_options',
            'weatherai',
            array($this, 'render_weather_page'),
            'dashicons-cloud'
        );

        add_submenu_page(
            'weatherai',
            'Weather Write Settings',
            'Settings',
            'manage_options',
            'weatherai',
            array($this, 'render_weather_page')
        );

        add_submenu_page(
            'weatherai',
            'Weather Images',
            'Images',
            'manage_options',
            'weatherai&tab=images',
            array($this, 'render_weather_page')
        );

        add_submenu_page(
            'weatherai',
            'Generate Weather Report',
            'Generate Report',
            'manage_options',
            'weatherai&tab=test',
            array($this, 'render_weather_page')
        );

        add_submenu_page(
            'weatherai',
            'Schedule Weather Reports',
            'Schedule' . ($this->is_scheduling_locked() ? ' 🔒' : ''),
            'manage_options',
            'weatherai&tab=schedule',
            array($this, 'render_weather_page')
        );
        
        // Setup Images menu removed - images are now included with the plugin
    }

    /**
     * Allow data attributes in HTML output
     */
    public function allow_data_attributes($allowed_html, $context) {
        if ($context === 'post') {
            foreach ($allowed_html as &$attributes) {
                $attributes['data-*'] = true;
            }
        }
        return $allowed_html;
    }

    public function handle_test_request() {
        try {
            // Start output buffering
            ob_start();

            // Verify nonce
            if (!isset($_POST['weather_nonce']) || !wp_verify_nonce($_POST['weather_nonce'], 'test_weather_report')) {
                wp_die('Invalid nonce');
            }

            // Verify user capabilities
            if (!current_user_can('manage_options')) {
                wp_die('Insufficient permissions');
            }

            // Get location settings
            $location = get_option('newspaper_ai_weather_location', array());
            
            if (empty($location['lat']) || empty($location['lng'])) {
                wp_die('Location coordinates not set. Please configure your location in the Weather Settings.');
            }

            // Get weather data
            $noaa_api = NewspaperAI_NOAA_API::get_instance();
            error_log('Weather Test: Fetching forecast for lat=' . $location['lat'] . ', lng=' . $location['lng']);
            
            $forecast = $noaa_api->get_forecast($location['lat'], $location['lng']);

            if (empty($forecast)) {
                error_log('Weather Test: No forecast data received');
                wp_die('Failed to fetch weather data. Please check the error log for details.');
            }

            error_log('Weather Test: Forecast data received:');
            error_log(print_r($forecast, true));

            $weather_data = array(
                'forecast' => $forecast,
                'location' => $location
            );

            // Generate report
            $report = $this->weather_reporter->generate_report($weather_data);

            if (!$report) {
                throw new Exception('Failed to generate weather report');
            }

            // Prepare today's data with location info
            $today_data = $forecast['today'];
            $today_data['city'] = $location['city'];
            $today_data['state'] = $location['state'];

            // Get the title from the weather reporter
            $title = $this->weather_reporter->generate_title($today_data);

            // Get post settings
            $post_settings = get_option('newspaper_ai_weather_post_settings', array(
                'category' => '',
                'post_status' => 'draft'
            ));
            
            // Get author setting (separate from post settings)
            $author_id = get_option('weather_write_author', 1);
            
            // Create post
            $post_data = array(
                'post_title'    => $title,
                'post_content'  => wp_kses_post($report),  // Sanitize with our allowed HTML
                'post_status'   => isset($post_settings['post_status']) ? $post_settings['post_status'] : 'draft',
                'post_type'     => 'post',
                'post_author'   => $author_id
            );

            $post_id = wp_insert_post($post_data);

            if (is_wp_error($post_id)) {
                throw new Exception($post_id->get_error_message());
            }
            
            // Assign category if specified in settings
            if (!empty($post_settings['category'])) {
                wp_set_post_categories($post_id, array(intval($post_settings['category'])));
            }

            // Set featured image
            $condition = $this->weather_reporter->get_weather_condition($forecast['today']);
            $weather_images = NewspaperAI_Weather_Images::get_instance();
            $image_id = $weather_images->get_weather_image_id($condition);
            
            if ($image_id) {
                // We have a custom image, use it
                $result = set_post_thumbnail($post_id, $image_id);
                error_log('Weather Write - Set custom image as featured image with ID: ' . $image_id . ', result: ' . ($result ? 'success' : 'failed'));
                
                if (!$result) {
                    // Try alternative method
                    update_post_meta($post_id, '_thumbnail_id', $image_id);
                    error_log('Weather Write - Tried alternative method to set featured image');
                }
            } else {
                // No custom image found, attach a default image
                $default_image_id = $weather_images->attach_default_image($post_id, $condition);
                error_log('Weather Write - Attempted to attach default image, result: ' . ($default_image_id ? 'success ID: ' . $default_image_id : 'failed'));
                
                if ($default_image_id) {
                    // Double-check that the featured image was set
                    $current_thumbnail_id = get_post_thumbnail_id($post_id);
                    if ($current_thumbnail_id != $default_image_id) {
                        error_log('Weather Write - Featured image verification failed, trying direct update');
                        update_post_meta($post_id, '_thumbnail_id', $default_image_id);
                    }
                }
            }

            // Clean output buffer
            ob_clean();

            // Redirect to the edit page
            wp_redirect(admin_url('post.php?post=' . $post_id . '&action=edit'));
            exit;

        } catch (Exception $e) {
            // Clean output buffer
            ob_clean();
            wp_die(
                esc_html($e->getMessage()),
                'Weather Report Error',
                array('back_link' => true)
            );
        }
    }

    public function handle_ajax_save_schedule() {
        if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'newspaper_ai_weather_schedule')) {
            wp_send_json_error(array('message' => 'Invalid security token'));
            return;
        }

        if (!current_user_can('manage_options')) {
            wp_send_json_error(array('message' => 'Insufficient permissions'));
            return;
        }

        // Get and validate posts per day
        $posts_per_day = isset($_POST['posts_per_day']) ? intval($_POST['posts_per_day']) : 1;
        if ($posts_per_day < 1) $posts_per_day = 1;
        if ($posts_per_day > 6) $posts_per_day = 6;

        // Get and validate times
        $times = isset($_POST['times']) ? (array)$_POST['times'] : array('09:00');

        // Validate each time and ensure we have enough slots
        $validated_times = array();
        foreach ($times as $time) {
            if (preg_match('/^([01]?[0-9]|2[0-3]):[0-5][0-9]$/', $time)) {
                $validated_times[] = $time;
            }
        }

        // Fill any missing slots with default time
        while (count($validated_times) < $posts_per_day) {
            $validated_times[] = '09:00';
        }

        // Trim to correct number of slots
        $validated_times = array_slice($validated_times, 0, $posts_per_day);

        // Save schedule
        $schedule = array(
            'posts_per_day' => $posts_per_day,
            'times' => $validated_times
        );

        // Clear existing schedules
        wp_clear_scheduled_hook('weather_write_cron');

        // Set up new schedules
        $timezone = wp_timezone();
        foreach ($schedule['times'] as $time) {
            list($hours, $minutes) = explode(':', $time);
            
            // Calculate next run time
            $date = new DateTime('today ' . $time, $timezone);
            if ($date->getTimestamp() < time()) {
                $date->modify('+1 day');
            }
            
            // Schedule the cron job
            wp_schedule_event(
                $date->getTimestamp(),
                'daily',
                'weather_write_cron'
            );
        }

        // Save the schedule
        $saved = update_option('newspaper_ai_weather_schedule', $schedule);
        
        if ($saved) {
            wp_send_json_success(array(
                'message' => 'Schedule saved successfully',
                'schedule' => $schedule
            ));
        } else {
            wp_send_json_error(array('message' => 'Failed to save schedule'));
        }
    }

    public function handle_ajax_save_weather_image() {
        error_log('Weather Write - AJAX handler called for save_weather_image');
        error_log('Weather Write - POST data: ' . print_r($_POST, true));
        
        try {
            check_ajax_referer('save_weather_image', 'nonce');
            
            if (!current_user_can('manage_options')) {
                error_log('Weather Write - User does not have manage_options capability');
                wp_send_json_error('Insufficient permissions');
                return;
            }

            $type = sanitize_text_field($_POST['type']);
            $variation = intval($_POST['variation']);
            $image_id = intval($_POST['image_id']);
            $image_url = esc_url_raw($_POST['image_url']);

            error_log('Weather Write - Processing image save: type=' . $type . ', variation=' . $variation . ', image_id=' . $image_id);

            $weather_images = NewspaperAI_Weather_Images::get_instance();
            
            // Check premium status for variations > 1
            if ($variation > 1 && !$weather_images->is_premium()) {
                error_log('Weather Write - Premium account required for variation ' . $variation);
                wp_send_json_error('Premium account required to use additional image variations.');
                return;
            }
            
            $result = $weather_images->save_weather_image($type, $variation, $image_id, $image_url);
            
            error_log('Weather Write - Save result: ' . ($result ? 'success' : 'failed'));

            wp_send_json_success(array(
                'message' => 'Image saved successfully',
                'type' => $type,
                'variation' => $variation,
                'image_id' => $image_id
            ));
        } catch (Exception $e) {
            error_log('Weather Write - Exception in save_weather_image: ' . $e->getMessage());
            wp_send_json_error('Error: ' . $e->getMessage());
        }
    }

    public function handle_toggle_debug() {
        check_ajax_referer('newspaper_ai_weather_debug', 'nonce');
        
        if (!current_user_can('manage_options')) {
            wp_send_json_error('Permission denied');
        }

        $debug_mode = isset($_POST['debug_mode']) ? (bool)$_POST['debug_mode'] : false;
        
        // Get the registered setting
        $settings = get_registered_settings();
        if (isset($settings['newspaper_ai_weather_debug_mode'])) {
            // Use the sanitize callback from the registered setting
            $sanitize_callback = $settings['newspaper_ai_weather_debug_mode']['sanitize_callback'];
            if (is_callable($sanitize_callback)) {
                $debug_mode = call_user_func($sanitize_callback, $debug_mode);
            }
        }
        
        update_option('newspaper_ai_weather_debug_mode', $debug_mode);
        
        wp_send_json_success(array(
            'debug_mode' => $debug_mode
        ));
    }

    public function is_debug_mode() {
        return (bool)get_option('newspaper_ai_weather_debug_mode', false);
    }

    public function is_scheduling_enabled() {
        return (bool)get_option('newspaper_ai_weather_scheduling_enabled', false);
    }

    private function is_scheduling_locked() {
        $api_error = get_option('reintent_api_error', '');
        return $api_error === 'inactive_user';
    }

    public function handle_toggle_scheduling() {
        check_ajax_referer('toggle_weather_scheduling', 'nonce');
        
        if (!current_user_can('manage_options')) {
            wp_send_json_error(array('message' => 'Permission denied'));
            return;
        }

        $scheduling_enabled = isset($_POST['scheduling_enabled']) ? filter_var($_POST['scheduling_enabled'], FILTER_VALIDATE_BOOLEAN) : false;
        
        // Always return success since the option might already be set to the desired value
        update_option('newspaper_ai_weather_scheduling_enabled', $scheduling_enabled);
        wp_send_json_success(array(
            'scheduling_enabled' => $scheduling_enabled,
            'message' => $scheduling_enabled ? 'Automated scheduling enabled' : 'Automated scheduling disabled'
        ));
    }

    public function generate_scheduled_report() {
        // Check if scheduling is enabled
        if (!$this->is_scheduling_enabled()) {
            return;
        }

        try {
            // Get location settings
            $location = get_option('newspaper_ai_weather_location', array());
            
            if (empty($location['lat']) || empty($location['lng'])) {
                throw new Exception('Location coordinates not set. Please configure your location in the Weather Settings.');
            }

            // Get weather data
            $noaa_api = NewspaperAI_NOAA_API::get_instance();
            $forecast = $noaa_api->get_forecast($location['lat'], $location['lng']);

            if (!$forecast) {
                throw new Exception('Failed to fetch weather data');
            }

            $weather_data = array(
                'forecast' => $forecast,
                'location' => $location
            );

            // Generate report
            $report = $this->weather_reporter->generate_report($weather_data);

            if (!$report) {
                throw new Exception('Failed to generate weather report');
            }

            // Prepare today's data with location info
            $today_data = $forecast['today'];
            $today_data['city'] = $location['city'];
            $today_data['state'] = $location['state'];

            // Get the title from the weather reporter
            $title = $this->weather_reporter->generate_title($today_data);

            // Get the weather condition and image before creating the post
            $condition = $this->weather_reporter->get_weather_condition($forecast['today']);
            error_log('Weather Write - Weather condition for featured image: ' . $condition);
            
            $weather_images = NewspaperAI_Weather_Images::get_instance();
            $image_id = $weather_images->get_weather_image_id($condition);
            error_log('Weather Write - Image ID for condition: ' . ($image_id ? $image_id : 'none found'));
            
            // Create post
            // Get post settings
            $post_settings = get_option('newspaper_ai_weather_post_settings', array(
                'category' => '',
                'post_status' => 'publish' // Default to publish for scheduled posts
            ));
            
            // Get author setting (separate from post settings)
            $author_id = get_option('weather_write_author', 1);
            
            $post_data = array(
                'post_title'    => $title,
                'post_content'  => wp_kses_post($report),  // Sanitize with our allowed HTML
                'post_status'   => isset($post_settings['post_status']) ? $post_settings['post_status'] : 'publish',
                'post_type'     => 'post',
                'post_author'   => $author_id
            );

            // Make sure we have the required WordPress functions
            if (!function_exists('wp_generate_attachment_metadata')) {
                require_once(ABSPATH . 'wp-admin/includes/image.php');
                require_once(ABSPATH . 'wp-admin/includes/file.php');
                require_once(ABSPATH . 'wp-admin/includes/media.php');
            }

            $post_id = wp_insert_post($post_data);

            if (is_wp_error($post_id)) {
                error_log('Weather Write - Error creating post: ' . $post_id->get_error_message());
                throw new Exception($post_id->get_error_message());
            }
            
            // Assign category if specified in settings
            if (!empty($post_settings['category'])) {
                wp_set_post_categories($post_id, array(intval($post_settings['category'])));
                error_log('Weather Write - Assigned post to category ID: ' . $post_settings['category']);
            }
            
            error_log('Weather Write - Created post with ID: ' . $post_id);
            
            // Force a small delay to ensure the post is fully created before attaching the image
            usleep(500000); // 0.5 seconds
            
            error_log('Weather Write - Post created with ID: ' . $post_id . ', now attaching image');

            // Set featured image
            if ($image_id) {
                // Use custom uploaded image
                $result = set_post_thumbnail($post_id, $image_id);
                if ($result) {
                    error_log('Weather Write - Successfully set featured image ' . $image_id . ' for post ' . $post_id);
                } else {
                    error_log('Weather Write - Failed to set featured image ' . $image_id . ' for post ' . $post_id);
                    // Try again with wp_update_post
                    $update_result = wp_update_post([
                        'ID' => $post_id,
                        'post_thumbnail' => $image_id
                    ]);
                    error_log('Weather Write - Attempted alternative method to set featured image: ' . ($update_result ? 'success' : 'failed'));
                }
            } else {
                // No custom image ID available, use the new method to attach a default image
                error_log('Weather Write - No custom image ID available, using default image');
                
                // Get the Weather_Images instance
                require_once dirname(__FILE__) . '/class-weather-images.php';
                $weather_images = NewspaperAI_Weather_Images::get_instance();
                
                // Attach the default image and get the attachment ID
                error_log('Weather Write - Calling attach_default_image for post ' . $post_id . ' with condition ' . $condition);
                $attachment_id = $weather_images->attach_default_image($post_id, $condition);
                
                if ($attachment_id) {
                    error_log('Weather Write - Successfully attached default image with ID: ' . $attachment_id);
                    
                    // Double-check that the featured image was set
                    $current_thumbnail_id = get_post_thumbnail_id($post_id);
                    if ($current_thumbnail_id != $attachment_id) {
                        error_log('Weather Write - Featured image verification failed, trying direct update');
                        update_post_meta($post_id, '_thumbnail_id', $attachment_id);
                    }
                } else {
                    // Local image doesn't exist, try to use a fallback image
                    error_log('Weather Write - Local image not found, using fallback image');
                    
                    // Make sure the images directory exists
                    $images_dir = $plugin_dir . 'assets/images/weather/';
                    if (!file_exists($images_dir)) {
                        wp_mkdir_p($images_dir);
                    }
                    
                    // Get the default image URL using the installer
                    require_once dirname(__FILE__) . '/class-weather-images-installer.php';
                    $installer = NewspaperAI_Weather_Images_Installer::get_instance();
                    $fallback_url = $installer->get_default_image_url($type, $variation);
                    
                    // Try to use a different variation if this one doesn't exist
                    for ($i = 1; $i <= 3; $i++) {
                        if ($i == $variation) continue;
                        $alt_path = $images_dir . $type . '-' . $i . '.jpg';
                        if (file_exists($alt_path)) {
                            error_log('Weather Write - Using alternative variation: ' . $i);
                            $image_path = $alt_path;
                            $image_url = $plugin_url . 'assets/images/weather/' . $type . '-' . $i . '.jpg';
                            break;
                        }
                    }
                    
                    // If we still don't have an image, use a generic fallback
                    if (!file_exists($image_path)) {
                        error_log('Weather Write - Using generic fallback image');
                        $image_path = $images_dir . 'cloudy-1.jpg';
                        $image_url = $plugin_url . 'assets/images/weather/cloudy-1.jpg';
                    }
                }
                
                // Store the selected image info for future reference
                update_option('weather_write_last_default_image', array(
                    'type' => $type,
                    'variation' => $variation,
                    'condition' => $condition
                ));
            }

        } catch (Exception $e) {
            error_log('Weather Report Cron Error: ' . $e->getMessage());
        }
    }

    /**
     * Attach an image from a URL to a post
     *
     * @param string $image_url The URL of the image to attach
     * @param int $post_id The post ID to attach the image to
     * @param string $title Optional title for the attachment
     * @return int|false The attachment ID on success, false on failure
     */
    public function attach_image_from_url($image_url, $post_id, $title = '') {
        error_log('Weather Write - Attaching image from URL: ' . $image_url);
        
        if (empty($image_url)) {
            error_log('Weather Write - Empty image URL provided');
            return false;
        }
        
        // Get the file name from the URL
        $file_name = basename($image_url);
        
        // Set up WordPress upload directory
        $upload_dir = wp_upload_dir();
        $image_data = file_get_contents($image_url);
        
        if ($image_data === false) {
            error_log('Weather Write - Failed to get image data from URL: ' . $image_url);
            return false;
        }
        
        // Create unique file name
        $file_name = wp_unique_filename($upload_dir['path'], $file_name);
        $file_path = $upload_dir['path'] . '/' . $file_name;
        
        // Save the image file
        $result = file_put_contents($file_path, $image_data);
        
        if ($result === false) {
            error_log('Weather Write - Failed to save image to: ' . $file_path);
            return false;
        }
        
        // Check the file type
        $file_type = wp_check_filetype($file_name, null);
        
        if (empty($file_type['type'])) {
            error_log('Weather Write - Invalid file type for: ' . $file_name);
            return false;
        }
        
        // Prepare attachment data
        $attachment = array(
            'post_mime_type' => $file_type['type'],
            'post_title'     => !empty($title) ? $title : preg_replace('/\.[^.\s]{3,4}$/', '', $file_name),
            'post_content'   => '',
            'post_status'    => 'inherit',
            'guid'           => $upload_dir['url'] . '/' . $file_name
        );
        
        // Insert attachment
        $attachment_id = wp_insert_attachment($attachment, $file_path, $post_id);
        
        if (is_wp_error($attachment_id)) {
            error_log('Weather Write - Failed to insert attachment: ' . $attachment_id->get_error_message());
            return false;
        }
        
        // Generate metadata for the attachment
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        $attachment_data = wp_generate_attachment_metadata($attachment_id, $file_path);
        wp_update_attachment_metadata($attachment_id, $attachment_data);
        
        error_log('Weather Write - Successfully attached image with ID: ' . $attachment_id);
        return $attachment_id;
    }

    public function enqueue_styles() {
        // Register Font Awesome
        wp_enqueue_style(
            'font-awesome',
            'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.0.0/css/all.min.css',
            array(),
            '6.0.0'
        );

        // Register and enqueue our weather styles
        $css_file_path = plugin_dir_path(dirname(dirname(__FILE__))) . 'assets/css/weather-report.css';
        $css_file_url = plugin_dir_url(dirname(dirname(__FILE__))) . 'assets/css/weather-report.css';
        
        // Use file modification time if the file exists, otherwise use plugin version
        $css_version = file_exists($css_file_path) ? filemtime($css_file_path) : '1.0.0';
        
        wp_enqueue_style(
            'newspaper-ai-weather',
            $css_file_url,
            array('font-awesome'),
            $css_version
        );
        
        // Register Chart.js first
        wp_register_script('chartjs', 'https://cdn.jsdelivr.net/npm/chart.js', array(), '4.4.1', true);
        
        // Then register our weather script with chartjs as a dependency
        wp_register_script(
            'weather-script',
            plugin_dir_url(__FILE__) . 'js/weather.js',
            array('jquery', 'chartjs'),
            '1.0.0',
            true
        );

        // Initialize the weather data with empty values
        wp_localize_script('weather-script', 'weatherHourlyData', array(
            'hours' => array(),
            'temperatures' => array(),
            'conditions' => array(),
            'winds' => array(),
            'chartId' => ''
        ));

        // Enqueue both scripts
        wp_enqueue_script('chartjs');
        wp_enqueue_script('weather-script');
    }

    /**
     * Enqueue frontend styles for weather display
     */
    public function enqueue_weather_styles() {
        // Add inline styles for weather post content
        $styles = "
            .weather-header {
                text-align: center;
                margin-bottom: 40px;
            }
            .current-temp {
                font-size: 72px;
                line-height: 1;
                margin-bottom: 15px;
                color: #1a1a1a;
            }
            .temp-range {
                font-size: 18px;
                color: #333;
            }
            .weather-details {
                width: 100%;
                max-width: 500px;
                margin: 0 auto 40px;
                border-collapse: collapse;
                font-size: 16px;
            }
            .weather-details td {
                padding: 12px;
                border-bottom: 1px solid #eee;
            }
            .weather-details td:first-child {
                color: #666;
                width: 120px;
            }
            .weather-details td:last-child {
                color: #1a1a1a;
                font-weight: 500;
            }
            .weather-hourly-chart-container {
                max-width: 800px;
                margin: 0 auto;
                padding: 20px 0;
            }
            .weather-hourly-chart-container h3 {
                text-align: center;
                margin-bottom: 20px;
                color: #1a1a1a;
                font-size: 24px;
            }
        ";
        
        wp_add_inline_style('newspaper-ai-weather-styles', $styles);

        // Register and enqueue styles
        wp_register_style('newspaper-ai-weather-styles', false);
        wp_enqueue_style('newspaper-ai-weather-styles');
        wp_add_inline_style('newspaper-ai-weather-styles', $styles);

    }

    public function enqueue_admin_scripts($hook) {
        if (strpos($hook, 'weatherai') === false) {
            return;
        }

        // Enqueue Font Awesome
        wp_enqueue_style(
            'font-awesome',
            'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.0.0/css/all.min.css',
            array(),
            '6.0.0'
        );

        // Enqueue admin styles
        wp_enqueue_style(
            'weather-write-admin',
            plugin_dir_url(__FILE__) . 'assets/admin/weather-admin.css',
            array('font-awesome'),
            '1.0.0'
        );

        
        // Enqueue media library
        wp_enqueue_media();
        
        wp_enqueue_script('weather-admin-script', plugin_dir_url(__FILE__) . 'js/weather-admin.js', array('jquery', 'media-upload'), '1.0.0', true);
        wp_enqueue_script('chartjs', 'https://cdn.jsdelivr.net/npm/chart.js', array(), '4.4.1', true);
        
        // Get premium status
        $weather_images = NewspaperAI_Weather_Images::get_instance();
        $is_premium = $weather_images->is_premium();
        
        wp_localize_script('weather-admin-script', 'weatherAjax', array(
            'ajaxurl' => admin_url('admin-ajax.php'),
            'scheduleNonce' => wp_create_nonce('newspaper_ai_weather_schedule'),
            'toggleNonce' => wp_create_nonce('toggle_weather_scheduling'),
            'nonce' => wp_create_nonce('save_weather_image'), // Use the correct nonce for image saving
            'isPremium' => $is_premium === true ? '1' : '0',
            'upgradeUrl' => NewspaperAI_Reintent_API::UPGRADE_URL
        ));
    }

    public function render_weather_page() {
        if (!current_user_can('manage_options')) {
            wp_die(__('Sorry, you are not allowed to access this page.'));
        }

        $active_tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : 'settings';
        ?>
        <div class="wrap weather-settings">
            <div class="weather-header">
                <img src="<?php echo plugin_dir_url(__FILE__) . 'assets/admin/weather-write-logo.png'; ?>" alt="Weather Write Logo">
            </div>
            
            <h2 class="nav-tab-wrapper">
                <a href="?page=weatherai&tab=settings" class="nav-tab <?php echo $active_tab === 'settings' ? 'nav-tab-active' : ''; ?>">
                    Settings
                </a>
                <a href="?page=weatherai&tab=images" class="nav-tab <?php echo $active_tab === 'images' ? 'nav-tab-active' : ''; ?>">
                    Weather Images
                </a>
                <a href="?page=weatherai&tab=test" class="nav-tab <?php echo $active_tab === 'test' ? 'nav-tab-active' : ''; ?>">
                    Generate Report
                </a>
                <a href="?page=weatherai&tab=schedule" class="nav-tab <?php echo $active_tab === 'schedule' ? 'nav-tab-active' : ''; ?>">
                    Schedule <?php if ($this->is_scheduling_locked()) : ?><span style="color: #fff; margin-left: 4px;">&nbsp;&#128274;</span><?php endif; ?>
                </a>
            </h2>

            <div class="tab-content">
                <?php
                switch ($active_tab) {
                    case 'images':
                        $this->render_images_tab();
                        break;
                    case 'test':
                        $this->render_test_tab();
                        break;
                    case 'schedule':
                        $this->render_schedule_tab();
                        break;
                    default:
                        $this->render_settings_tab();
                        break;
                }
                ?>
            </div>
        </div>
        <?php
    }

    private function render_settings_tab() {
        ?>
        <div class="wrap">
            <h2>Weather Write Settings</h2>
            <form method="post" action="options.php" id="reintent-settings-form">
                <?php
                settings_fields('weatherai');
                do_settings_sections('weatherai');
                submit_button();
                ?>
            </form>
        </div>
        <?php
    }

    private function render_images_tab() {
        $weather_types = array(
            'sunny' => 'Sunny/Clear/Mostly Sunny',
            'partly_cloudy' => 'Partly Cloudy/Partly Sunny',
            'cloudy' => 'Cloudy/Mostly Cloudy/Overcast',
            'rain' => 'Rain/Showers',
            'storm' => 'Thunderstorm/Severe',
            'snow' => 'Snow/Winter Weather',
            'fog' => 'Fog/Haze'
        );

        $weather_images = NewspaperAI_Weather_Images::get_instance();
        ?>
        <div class="weather-images-settings">
            <div class="info-section" style="background-color: #f8f9fa; padding: 20px; margin-bottom: 20px; border-left: 8px solid #00a3e6; border-radius: 4px;">
                <h3>Weather Images</h3>
                <p>Weather Write includes default weather images, <strong>you can upload your own</strong> to customize your weather report to your location. <strong>Add up to 3 variations per weather condition</strong>, and one will be randomly selected in reports.</p>
            </div>
            
            
            <div class="weather-image-grid">
                <?php foreach ($weather_types as $type => $label): ?>
                    <div class="weather-type-section">
                        <h3><?php echo esc_html($label); ?></h3>
                        <div class="weather-variations">
                            <?php for ($i = 1; $i <= 3; $i++): 
                                $image_url = $weather_images->get_weather_image_url($type, $i);
                            ?>
                                <div class="weather-variation">
                                    <h4>Variation <?php echo $i; ?></h4>
                                    <div class="weather-image-preview">
                                        <?php if ($image_url): ?>
                                            <img src="<?php echo esc_url($image_url); ?>" alt="<?php echo esc_attr($label); ?> weather image <?php echo $i; ?>" />
                                        <?php endif; ?>
                                    </div>
                                    <button class="button weather-image-upload" 
                                            data-type="<?php echo esc_attr($type); ?>" 
                                            data-variation="<?php echo esc_attr($i); ?>">
                                        <?php echo $image_url ? 'Change Image' : 'Upload Image'; ?>
                                    </button>
                                    <input type="hidden" name="weather_image_nonce" value="<?php echo wp_create_nonce('save_weather_image'); ?>">
                                </div>
                            <?php endfor; ?>
                        </div>
                    </div>
                <?php endforeach; ?>
            </div>
        </div>

        <style>
            .weather-images-settings {
                margin-top: 20px;
            }
            .weather-image-grid {
                display: grid;
                grid-template-columns: repeat(auto-fit, minmax(300px, 1fr));
                gap: 2rem;
                margin-top: 20px;
            }
            .weather-type-section {
                background: white;
                padding: 1.5rem;
                border-radius: 8px;
                box-shadow: 0 1px 3px rgba(0,0,0,0.1);
            }
            .weather-variations {
                display: grid;
                gap: 1rem;
            }
            .weather-variation {
                background: #f8f9fa;
                padding: 1rem;
                border-radius: 6px;
            }
            .weather-variation h4 {
                margin: 0 0 10px 0;
            }
            .weather-image-preview {
                margin: 10px 0;
                background: #000;
                border-radius: 4px;
                overflow: hidden;
                min-height: 118;
                max-height: 118px;
                display: flex;
                align-items: center;
                justify-content: center;
            }
            .weather-image-preview img {
                max-width: 100%;
                height: auto;
                display: block;
            }
            .weather-image-upload {
                width: 75%;

            }
        </style>
        <?php
    }

    private function render_test_tab() {
        // Check if city and state are set
        $location = get_option('newspaper_ai_weather_location', array());
        $city = isset($location['city']) ? trim($location['city']) : '';
        $state = isset($location['state']) ? trim($location['state']) : '';
        $location_set = !empty($city) && !empty($state);  // Only checks city and state
        
        ?>
        <div class="wrap">
            <?php if (!$location_set): ?>
            <div class="notice notice-warning">
                <p><strong>Location Not Set:</strong> Please configure your city and state in the <a href="/wp-admin/admin.php?page=weatherai&tab=settings">Settings tab</a> before generating a manual report.</p>
            </div>
            <?php endif; ?>
            
            <form method="post" action="<?php echo esc_url(admin_url('admin-post.php')); ?>">
                <?php wp_nonce_field('test_weather_report', 'weather_nonce'); ?>
                <input type="hidden" name="action" value="test_weather_report">
                <input type="hidden" name="redirect" value="<?php echo esc_url(admin_url('admin.php?page=weatherai&tab=test')); ?>">
                <div class="weather-images-settings">
            <div class="info-section" style="background-color: #f8f9fa; padding: 20px; margin-bottom: 20px; border-left: 8px solid #00a3e6; border-radius: 4px;">
            <h3>Generate a WeatherWrite Report Manually</h3>
<p>Generate a report manually as needed or to see how a weather post will look on your site.  
<br><br>
<strong>⚠ Important:</strong> If your publish settings are set to <strong>"Publish Immediately,"</strong> the test report will go out as a <strong>live published post</strong>. To avoid this, set it to Draft in the <a href="/wp-admin/admin.php?page=weatherai&tab=settings"> Settings tab</a>  

<h3>Manual reports use your monthly credits, so use them wisely.</h3></p>
</div>
                
                
                <p class="submit">
                    <input type="submit" name="submit" id="submit" 
                           class="button button-primary" 
                           value="Generate Test Report" 
                           <?php echo !$location_set ? 'disabled="disabled"' : ''; ?>
                           title="<?php echo !$location_set ? 'Please set your city and state in the Settings tab first' : ''; ?>">
                </p>
            </form>

            <div class="debug-toggle" style="display: none; margin: 20px 0; padding: 15px; background: #fff; border: 1px solid #ccd0d4; border-radius: 4px;">
                <label class="debug-toggle-label" style="display: flex; align-items: center; cursor: pointer;">
                    <input type="checkbox" id="debug-mode-toggle" <?php checked($this->is_debug_mode()); ?> style="margin-right: 10px;">
                    <span>Enable Debug Mode</span>
                </label>
                <p class="description" style="margin: 10px 0 0; color: #646970;">
                    When enabled, debug information including API calls will be shown below. This setting persists between page loads.
                </p>
            </div>

            <div class="debug-section" style="display: none;">
                <?php
                $location = get_option('newspaper_ai_weather_location', array(
                    'lat' => '',
                    'lng' => ''
                ));
                
                $noaa_api = NewspaperAI_NOAA_API::get_instance();
                $forecast = $noaa_api->get_forecast($location['lat'], $location['lng']);
                $current_conditions = $noaa_api->get_current_conditions($location['lat'], $location['lng']);
                if ($current_conditions) {
                    $condition = $this->weather_reporter->get_weather_condition($current_conditions);
                    $current_temp = isset($current_conditions['temperature']) ? $current_conditions['temperature'] : 'N/A';
                    
                    // Get the forecast URL from points data
                    $points_url = 'https://api.weather.gov/points/' . $location['lat'] . ',' . $location['lng'];
                    ?>
                    <div class="debug-info" style="margin-top: 20px; padding: 20px; background: #f0f0f1; border-radius: 4px;">
                        <h3 style="margin-top: 0;">Debug Info</h3>
                        <p>Current weather condition: <span style="display: inline-block; background: #fff; padding: 5px 10px; border-radius: 15px; border: 1px solid #ddd;"><?php echo esc_html($condition); ?></span></p>
                        <p>Current temperature: <span style="display: inline-block; background: #fff; padding: 5px 10px; border-radius: 15px; border: 1px solid #ddd;"><?php echo esc_html($current_temp); ?>°F</span></p>
                        
                        <div style="margin-top: 15px;">
                            <p style="margin-bottom: 5px;"><strong>NOAA API Endpoints:</strong></p>
                            <div style="background: #fff; padding: 10px; border-radius: 4px; border: 1px solid #ddd;">
                                <?php if (isset($current_conditions['station'])): ?>
                                    <p style="margin: 5px 0;">
                                        <strong>Station:</strong> 
                                        <a href="<?php echo esc_url($current_conditions['station']); ?>" target="_blank" rel="noopener noreferrer">
                                            <?php echo esc_html($current_conditions['station']); ?>
                                        </a>
                                    </p>
                */
                                <?php endif; ?>
                                
                                <?php if (isset($forecast['forecast'])): ?>
                                    <p style="margin: 5px 0;">
                                        <strong>Forecast:</strong> 
                                        <a href="<?php echo esc_url($forecast['forecast']); ?>" target="_blank" rel="noopener noreferrer">
                                            <?php echo esc_html($forecast['forecast']); ?>
                                        </a>
                                    </p>
                                <?php endif; ?>
                                
                                <p style="margin: 5px 0;">
                                    <strong>Points:</strong> 
                                    <a href="<?php echo esc_url($points_url); ?>" target="_blank" rel="noopener noreferrer">
                                        <?php echo esc_html($points_url); ?>
                                    </a>
                                </p>
                            </div>
                            <div style="margin-top: 20px; text-align: right;">
                                <button type="button" id="update-schedule" class="button button-primary">Update Publishing Time Slots</button>
                            </div>
                        </div>
                    </div>
                <?php
                }
                ?>
            </div>
        </div>

        <script>
        jQuery(document).ready(function($) {
            $('#debug-mode-toggle').on('change', function() {
                var isChecked = $(this).prop('checked');
                var $debugSection = $('.debug-section');
                var $toggle = $(this);
                
                $toggle.prop('disabled', true); // Disable during request
                
                $.ajax({
                    url: ajaxurl,
                    type: 'POST',
                    data: {
                        action: 'toggle_weather_debug',
                        debug_mode: isChecked,
                        nonce: '<?php echo wp_create_nonce('newspaper_ai_weather_debug'); ?>'
                    },
                    success: function(response) {
                        if (response.success) {
                            // Update UI without page reload
                            if (isChecked) {
                                $debugSection.fadeIn();
                            } else {
                                $debugSection.fadeOut();
                            }
                            // Store the state in localStorage to prevent auto-enabling
                            localStorage.setItem('weather_debug_mode', isChecked);
                        } else {
                            // Revert checkbox if failed
                            $toggle.prop('checked', !isChecked);
                            alert('Failed to update debug mode');
                        }
                    },
                    error: function() {
                        alert('Failed to update debug mode');
                    }
                });
            });
        });
        </script>
        <?php
    }

    private function render_schedule_tab() {
        // Get current settings and status
        $schedule_settings = get_option('newspaper_ai_weather_schedule', array(
            'posts_per_day' => 1,
            'times' => array('09:00')
        ));
        
        // Check premium status
        $api_key = get_option('newspaperai_reintent_api_key', '');
        $api_error = get_option('reintent_api_error', '');
        $is_premium = !empty($api_key) && $api_error !== 'inactive_user';
        $is_enabled = $this->is_scheduling_enabled();
        $is_locked = $this->is_scheduling_locked();

        // Validate and normalize settings
        $schedule_settings['posts_per_day'] = isset($schedule_settings['posts_per_day']) ? intval($schedule_settings['posts_per_day']) : 1;
        $schedule_settings['times'] = isset($schedule_settings['times']) && is_array($schedule_settings['times']) ? $schedule_settings['times'] : array('09:00');

        // Enforce premium limitations
        if (!$is_premium && $schedule_settings['posts_per_day'] > 1) {
            $schedule_settings['posts_per_day'] = 1;
            $schedule_settings['times'] = array_slice($schedule_settings['times'], 0, 1);
            update_option('newspaper_ai_weather_schedule', $schedule_settings);
        }

        // Ensure we have enough time slots
        while (count($schedule_settings['times']) < $schedule_settings['posts_per_day']) {
            $schedule_settings['times'][] = '09:00';
        }

        ?>
        <div class="wrap">
            <?php if ($is_locked): ?>
            <div class="notice notice-warning" style="margin-bottom: 20px;">
                <p><strong>⚠️ Scheduling is currently locked.</strong> Your account appears to be inactive. Please check your subscription status.</p>
            </div>
            <?php endif; ?>

            <div class="scheduling-container" style="background: #fff; border: 1px solid #ccd0d4; border-radius: 8px; overflow: hidden;">
                <style>
                    .toggle-switch {
                        position: relative;
                        display: inline-block;
                        width: 50px;
                        height: 24px;
                        margin-right: 10px;
                    }
                    .toggle-switch input {
                        opacity: 0;
                        width: 0;
                        height: 0;
                    }
                    .toggle-slider {
                        position: absolute;
                        cursor: pointer;
                        top: 0;
                        left: 0;
                        right: 0;
                        bottom: 0;
                        background-color: #ccc;
                        transition: .4s;
                        border-radius: 24px;
                    }
                    .toggle-slider:before {
                        position: absolute;
                        content: "";
                        height: 16px;
                        width: 16px;
                        left: 4px;
                        bottom: 4px;
                        background-color: white;
                        transition: .4s;
                        border-radius: 50%;
                    }
                    input:checked + .toggle-slider {
                        background-color: #00a3e6;
                    }
                    input:checked + .toggle-slider:before {
                        transform: translateX(26px);
                    }
                </style>
                <!-- Header Section -->
                <div class="scheduling-header" style="padding: 20px; background: #f8f9fa; border-bottom: 1px solid #dee2e6;">
                    <div style="display: flex; align-items: center; gap: 15px;">
                        <label class="toggle-switch">
                            <input type="checkbox" 
                                   id="scheduling-enabled-toggle" 
                                   <?php checked($is_enabled); ?>
                                   <?php disabled($is_locked); ?>>
                            <span class="toggle-slider"></span>
                        </label>
                        <h2 style="margin: 0; font-size: 1.2em; color: #1d2327;">Automated Weather Posts</h2>
                    </div>
                    <div id="scheduling-status" class="hidden" style="margin-top: 10px; padding: 10px; border-radius: 4px;"></div>
                </div>

                <!-- Main Content Section -->
                <div id="schedule-settings" class="scheduling-content" style="padding: 20px; <?php echo !$is_enabled ? 'display: none;' : ''; ?>">
                    <?php if (!$is_premium): ?>
                    
                    <?php endif; ?>

                    <!-- Schedule Configuration -->
                    <div class="schedule-config">
                        <div class="posts-per-day" style="margin-bottom: 30px;">
                            <h3 style="margin: 0 0 15px 0;">Reports Per Day</h3>
                            <div class="posts-options" style="display: grid; grid-template-columns: repeat(auto-fit, minmax(100px, 1fr)); gap: 10px;">
                                <?php 
                                $max_posts = $is_premium ? 6 : 1;
                                for ($i = 1; $i <= 6; $i++): 
                                    $disabled = $i > $max_posts;
                                ?>
                                <label class="posts-option" style="
                                    padding: 10px; 
                                    text-align: center; 
                                    border: 2px solid <?php echo $schedule_settings['posts_per_day'] == $i ? '#00a3e6' : '#dee2e6'; ?>;
                                    border-radius: 6px;
                                    background: <?php echo $schedule_settings['posts_per_day'] == $i ? '#00a3e6' : 'white'; ?>;
                                    color: <?php echo $schedule_settings['posts_per_day'] == $i ? 'white' : '#1d2327'; ?>;
                                    cursor: <?php echo $disabled ? 'not-allowed' : 'pointer'; ?>;
                                    opacity: <?php echo $disabled ? '0.5' : '1'; ?>;
                                    ">
                                    <input type="radio" 
                                           name="posts_per_day" 
                                           value="<?php echo $i; ?>" 
                                           <?php checked($schedule_settings['posts_per_day'], $i); ?>
                                           <?php disabled($disabled); ?>
                                           style="display: none;">
                                    <?php echo $i; ?> per day
                                </label>
                                <?php endfor; ?>
                            </div>
                        </div>

                        <div id="time-slots" style="margin-bottom: 20px;">
                            <h3 style="margin: 0 0 15px 0;">Publishing Times</h3>
                            <div id="time-slots-container" style="display: grid; gap: 15px;">
                                <?php foreach ($schedule_settings['times'] as $index => $time): ?>
                                <div class="time-slot" style="
                                    display: flex; 
                                    align-items: center; 
                                    gap: 10px; 
                                    padding: 15px; 
                                    background: #f8f9fa; 
                                    border-radius: 6px;">
                                    <span style="font-weight: 500;">Time <?php echo $index + 1; ?></span>
                                    <input type="time" 
                                           class="time-input" 
                                           value="<?php echo esc_attr($time); ?>" 
                                           style="flex-grow: 1; padding: 6px; border: 1px solid #dee2e6; border-radius: 4px;">
                                </div>
                                <?php endforeach; ?>
                            </div>
                        </div>
                    </div>
                </div>
            </div>

            <style>
                .toggle-switch input:checked + label { background: #2271b1; }
                .toggle-switch input:checked + label span { transform: translateX(24px); }
                .toggle-switch label:hover { opacity: 0.9; }
                .posts-option:hover:not([disabled]) { border-color: #2271b1; }
                .time-input:focus { border-color: #2271b1; outline: none; box-shadow: 0 0 0 1px #2271b1; }
                #scheduling-status.success { background: #d4edda; color: #155724; }
                #scheduling-status.error { background: #f8d7da; color: #721c24; }
            </style>

            <script>
            jQuery(document).ready(function($) {
                const $toggle = $('#scheduling-enabled-toggle');
                const $status = $('#scheduling-status');
                const $scheduleContent = $('#schedule-settings');
                const $timeSlots = $('#time-slots-container');

                function showStatus(message, isError = false) {
                    $status.removeClass('success error').addClass(isError ? 'error' : 'success')
                        .html(message)
                        .fadeIn()
                        .delay(3000)
                        .fadeOut();
                }

                function createTimeSlot(index, time = '09:00') {
                    return `
                        <div class="time-slot" style="
                            display: flex; 
                            align-items: center; 
                            gap: 10px; 
                            padding: 15px; 
                            background: #f8f9fa; 
                            border-radius: 6px;">
                            <span style="font-weight: 500;">Time ${index + 1}</span>
                            <input type="time" 
                                   class="time-input" 
                                   value="${time}" 
                                   style="flex-grow: 1; padding: 6px; border: 1px solid #dee2e6; border-radius: 4px;">
                        </div>
                    `;
                }

                function updateTimeSlots(numSlots, times = []) {
                    const $container = $('#time-slots-container');
                    $container.empty();
                    
                    for (let i = 0; i < numSlots; i++) {
                        const time = times[i] || '09:00';
                        $container.append(createTimeSlot(i, time));
                    }
                }

                function saveSchedule() {
                    const postsPerDay = parseInt($('input[name="posts_per_day"]:checked').val()) || 1;
                    const times = [];
                    $('.time-input').each(function() {
                        times.push($(this).val() || '09:00');
                    });

                    $.ajax({
                        url: ajaxurl,
                        type: 'POST',
                        data: {
                            action: 'save_weather_schedule',
                            nonce: '<?php echo wp_create_nonce("newspaper_ai_weather_schedule"); ?>',
                            posts_per_day: postsPerDay,
                            times: times
                        },
                        success: function(response) {
                            if (response.success) {
                                // Update UI with new time slots
                                updateTimeSlots(postsPerDay, times);
                                showStatus('Schedule updated successfully');
                            } else {
                                showStatus(response.data.message || 'Failed to update schedule', true);
                            }
                        },
                        error: function() {
                            showStatus('Failed to update schedule', true);
                        }
                    });
                }

                // Toggle scheduling
                $toggle.on('change', function() {
                    const isEnabled = $(this).prop('checked');
                    
                    $.ajax({
                        url: ajaxurl,
                        type: 'POST',
                        data: {
                            action: 'toggle_weather_scheduling',
                            nonce: '<?php echo wp_create_nonce("toggle_weather_scheduling"); ?>',
                            scheduling_enabled: isEnabled
                        },
                        success: function(response) {
                            if (response.success) {
                                $scheduleContent.slideToggle();
                                showStatus(response.data.message);
                            } else {
                                $toggle.prop('checked', !isEnabled);
                                showStatus(response.data.message || 'Failed to update scheduling', true);
                            }
                        },
                        error: function() {
                            $toggle.prop('checked', !isEnabled);
                            showStatus('Failed to update scheduling', true);
                        }
                    });
                });

                // Handle posts per day selection
                $('.posts-option').on('click', function() {
                    if ($(this).find('input').prop('disabled')) return;
                    
                    const $radio = $(this).find('input[type="radio"]');
                    if (!$radio.prop('checked')) {
                        $radio.prop('checked', true);
                        saveSchedule();
                    }
                });

                // Handle time changes
                $(document).on('change', '.time-input', saveSchedule);
            });
            </script>

        <div class="weather-schedule-settings" style="max-width: 1200px; margin: 0 auto;">
                <script>
                jQuery(document).ready(function($) {
                    // Handle button color changes
                    $('.posts-option').on('click', function() {
                        $('.posts-option').css({
                            'background': 'white',
                            'color': '#1d2327'
                        });
                        $(this).css({
                            'background': '#00a3e6',
                            'color': 'white'
                        });
                    });

                    // Handle toggle functionality
                    var $toggle = $('#scheduling-enabled-toggle');
                    var $scheduleContent = $('#schedule-settings');

                    $toggle.on('change', function() {
                        var isChecked = $(this).prop('checked');
                        var nonce = $('#toggle_scheduling_nonce').val();

                        $(this).prop('disabled', true);

                        $.ajax({
                            url: ajaxurl,
                            type: 'POST',
                            data: {
                                action: 'toggle_weather_scheduling',
                                nonce: nonce,
                                scheduling_enabled: isChecked
                            },
                            success: function(response) {
                                if (response.success) {
                                    $scheduleContent.slideToggle();
                                } else {
                                    $toggle.prop('checked', !isChecked);
                                }
                            },
                            error: function() {
                                $toggle.prop('checked', !isChecked);
                            },
                            complete: function() {
                                $toggle.prop('disabled', false);
                            }
                        });
                    });
                });
                </script>
            </div>
            <?php
        }
        
    /**
     * Render the fix images page
     */
    public function render_fix_images_page() {
        include_once dirname(__FILE__) . '/fix-default-images.php';
    }

}

// Initialize the weather module
NewspaperAI_Weather::get_instance();
