<?php
/**
 * Plugin Name: Weather Write
 * Plugin URI: https://weatherwrite.com
 * Description: Automated weather reporting and content generation for local publishers. <a href="admin.php?page=weatherai">Configure Settings</a>
 * Version: 1.0.0
 * Author: ReIntent
 * Author URI: https://reintent.com
 * License: GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: newspaper-ai
 */

// Prevent direct access to this file
if (!defined('ABSPATH')) {
    exit;
}

// Define plugin constants
define('NEWSPAPER_AI_VERSION', '1.0.3');
define('NEWSPAPER_AI_PATH', plugin_dir_path(__FILE__));
define('NEWSPAPER_AI_URL', plugin_dir_url(__FILE__));

// Register activation hook
register_activation_hook(__FILE__, 'newspaper_ai_activate');

function newspaper_ai_activate() {
    // Check if get_plugin_data function exists
    if (!function_exists('get_plugin_data')) {
        require_once(ABSPATH . 'wp-admin/includes/plugin.php');
    }

    // Get plugin data
    $plugin_data = get_plugin_data(__FILE__);
    $plugin_version = $plugin_data['Version'];

    // If the plugin has never been activated, or if it's a new version
    if (!get_option('newspaper_ai_version') || get_option('newspaper_ai_version') !== $plugin_version) {
        // Update the version in the database
        update_option('newspaper_ai_version', $plugin_version);
    }

    // Ensure debug mode is always off by default
    update_option('newspaper_ai_weather_debug_mode', false);
    if (false === get_option('newspaper_ai_weather_author')) {
        add_option('newspaper_ai_weather_author', 1);
    }

    // Set up default weather images if they haven't been set yet
    if (empty(get_option('weather_write_images', array()))) {
        require_once ABSPATH . 'wp-admin/includes/media.php';
        require_once ABSPATH . 'wp-admin/includes/file.php';
        require_once ABSPATH . 'wp-admin/includes/image.php';

        $default_images_path = NEWSPAPER_AI_PATH . 'assets/default_images/';
        $weather_types = array(
            'sunny' => array('sunny.png', 'sunny2.png', 'sunny3.png'),
            'partly_cloudy' => array('cloudy.png', 'cloudy2.png', 'cloudy3.png'),
            'cloudy' => array('overcast.png', 'overcast2.png', 'overcast3.png'),
            'rain' => array('rain.png', 'rain2.png', 'rain3.png'),
            'storm' => array('lightning.png', 'lightning2.png', 'lightning3.png'),
            'snow' => array('snow.png', 'snow2.png', 'snow3.png'),
            'fog' => array('fog.png', 'fog2.png', 'fog3.png')
        );

        $weather_images = array();

        foreach ($weather_types as $type => $images) {
            foreach ($images as $index => $image) {
                $file = $default_images_path . $image;
                if (file_exists($file)) {
                    $upload = wp_upload_bits($image, null, file_get_contents($file));
                    if (!$upload['error']) {
                        $wp_filetype = wp_check_filetype($image, null);
                        $attachment = array(
                            'post_mime_type' => $wp_filetype['type'],
                            'post_title' => sanitize_file_name($image),
                            'post_content' => '',
                            'post_status' => 'inherit'
                        );
                        $attach_id = wp_insert_attachment($attachment, $upload['file']);
                        $attach_data = wp_generate_attachment_metadata($attach_id, $upload['file']);
                        wp_update_attachment_metadata($attach_id, $attach_data);

                        $key = $type . '_' . ($index + 1);
                        $weather_images[$key] = array(
                            'id' => $attach_id,
                            'url' => $upload['url']
                        );
                    }
                }
            }
        }

        if (!empty($weather_images)) {
            update_option('weather_write_images', $weather_images);
        }
    }

    // Flush rewrite rules for weather test page
    flush_rewrite_rules();
}

// Include API configuration
require_once NEWSPAPER_AI_PATH . 'config/api-keys.php';

// Include required files
require_once NEWSPAPER_AI_PATH . 'functions.php';

class NewspaperAIPlugin {
    private static $instance = null;

    public static function get_instance() {
        if (self::$instance === null) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    private function __construct() {
        add_action('init', array($this, 'init'));
    }

    public function init() {
        // Load text domain for internationalization
        load_plugin_textdomain('newspaper-ai', false, dirname(plugin_basename(__FILE__)) . '/languages');
        
        // Initialize modules
        $this->init_modules();
    }

    private function init_modules() {
        // Load core modules first
        require_once NEWSPAPER_AI_PATH . 'modules/weather/class-noaa-api.php';

        require_once NEWSPAPER_AI_PATH . 'modules/weather/class-weather-images.php';
        require_once NEWSPAPER_AI_PATH . 'modules/weather/class-weather-assets.php';
        require_once NEWSPAPER_AI_PATH . 'modules/weather/class-reintent-api.php';
        require_once NEWSPAPER_AI_PATH . 'modules/weather/class-weather-settings.php';
        require_once NEWSPAPER_AI_PATH . 'modules/weather/class-weather-reporter.php';
        require_once NEWSPAPER_AI_PATH . 'modules/weather/upgrade-banner.php';
        require_once NEWSPAPER_AI_PATH . 'modules/weather/weather.php';
        
        // Initialize required components
        NewspaperAI_Weather_Settings::get_instance();
        NewspaperAI_Weather::get_instance();
    }

    public function render_admin_page() {
        if (!current_user_can('manage_options')) {
            return;
        }
        include NEWSPAPER_AI_PATH . 'templates/admin-page.php';
    }
}

// Initialize the plugin
add_action('plugins_loaded', function() {
    NewspaperAIPlugin::get_instance();
});
